@@ -43,6 +43,7 @@ int UDPATTR;
 int IGPATTR;
 int IGMPATTR;
 int IGRPATTR;
+int GREATTR;
 int ARPATTR;
 int UNKNATTR;
 
@@ -74,7 +75,7 @@ void draw_desktop(void)
 /* for some reason, the ncurses box routine does not color
    its inside.  it would therefore be better to call this
    routine before drawing the box.
- */
+ */  
 
 void colorwin(WINDOW * win)
 {
@@ -133,7 +134,7 @@ void about()
     mvwprintw(win, 2, 2, "An IP Network Statistics Utility");
     mvwprintw(win, 3, 2, "Version %s", VERSION);
     mvwprintw(win, 5, 2, "Written by Gerard Paul Java");
-    mvwprintw(win, 6, 2, "Copyright (c) Gerard Paul Java 1997, 1998");
+    mvwprintw(win, 6, 2, "Copyright (c) Gerard Paul Java 1997-2000");
     mvwprintw(win, 8, 2, "This program is distributed under the terms of");
     mvwprintw(win, 9, 2, "the GNU General Public License Version 2 or");
     mvwprintw(win, 10, 2, "any later version. See the included COPYING");
@@ -157,41 +158,6 @@ void about()
     doupdate();
 }
 
-void masq_warning(void)
-{
-    WINDOW *win;
-    PANEL *panel;
-
-    win = newwin(15, 50, (LINES - 15) / 2, (COLS - 50) / 2);
-    panel = new_panel(win);
-
-    stdwinset(win);
-    wtimeout(win, -1);
-    wattrset(win, ERRBOXATTR);
-    colorwin(win);
-    box(win, ACS_VLINE, ACS_HLINE);
-    wattrset(win, ERRTXTATTR);
-    mvwprintw(win, 1, 2, "This computer is running with IP Masquerading");
-    mvwprintw(win, 2, 2, "enabled.  If this machine is a masquerading");
-    mvwprintw(win, 3, 2, "router, the various facilities may give");
-    mvwprintw(win, 4, 2, "strange results because of the translation of");
-    mvwprintw(win, 5, 2, "the packets' IP addresses.  You will get");
-    mvwprintw(win, 6, 2, "better results if you run IPTraf on the");
-    mvwprintw(win, 7, 2, "network segments connected to this machine,");
-    mvwprintw(win, 8, 2, "but not on this machine itself.");
-    mvwprintw(win, 10, 2, "If this computer is not a masquerading router");
-    mvwprintw(win, 11, 2, "you can ignore this message.");
-    wattrset(win, ERRBOXATTR);
-    mvwprintw(win, 13, 2, ANYKEY_MSG);
-
-    wgetch(win);
-
-    del_panel(panel);
-    delwin(win);
-    update_panels();
-    doupdate();
-}
-
 void printnomem()
 {
     attrset(ERRTXTATTR);
@@ -218,18 +184,23 @@ void stdkeyhelp(WINDOW * win)
     printkeyhelp("Ctrl+X", "-cancel", win);
 }
 
-void stdexitkeyhelp()
+void sortkeyhelp(void)
 {
-    printkeyhelp("X/Ctrl+X", "-Exit", stdscr);
+    printkeyhelp("S", "-sort  ", stdscr);
+}
+
+void stdexitkeyhelp(void)
+{
+    printkeyhelp("X", "-exit", stdscr);
     coloreol(stdscr);
 }
 
-void scrollkeyhelp()
+void scrollkeyhelp(void)
 {
     printkeyhelp("Up/Down/PgUp/PgDn", "-scroll window  ", stdscr);
 }
 
-void menukeyhelp()
+void menukeyhelp(void)
 {
     move(LINES - 1, 1);
     printkeyhelp("Up/Down", "-Move selector  ", stdscr);
@@ -260,6 +231,21 @@ void indicate(char *message)
     refresh();
 }
 
+void show_sort_statwin(WINDOW **statwin, PANEL **panel)
+{
+    *statwin = newwin(5, 20, (LINES - 5) / 2, (COLS - 20) / 2);
+    *panel = new_panel(*statwin);
+    
+    wattrset(*statwin, BOXATTR);
+    colorwin(*statwin);
+    box(*statwin, ACS_VLINE, ACS_HLINE);
+    
+    mvwprintw(*statwin, 0, 1, " Sorting ");
+    wattrset(*statwin, STDATTR);
+    mvwprintw(*statwin, 2, 2, "Completed:");
+    wattrset(*statwin, HIGHATTR);
+}
+
 void printlargenum(unsigned long long i, WINDOW * win)
 {
     if (i < 100000000)		/* less than 100 million */
@@ -274,6 +260,30 @@ void printlargenum(unsigned long long i, WINDOW * win)
 	wprintw(win, "%8lluT", i / 1000000000000ULL);
 }
 
+void infobox(char *text, char *prompt)
+{
+    WINDOW *win;
+    PANEL *panel;
+    
+    win = newwin(4, 50, (LINES - 4) / 2, (COLS - 50) / 2);
+    panel = new_panel(win);
+    wattrset(win, BOXATTR);
+    colorwin(win);
+    box(win, ACS_VLINE, ACS_HLINE);
+    wattrset(win, STDATTR);
+    mvwprintw(win, 1, 2, text);
+    wattrset(win, HIGHATTR);
+    mvwprintw(win, 2, 2, prompt);
+    update_panels();
+    doupdate();
+    wgetch(win);
+    del_panel(panel);
+    delwin(win);
+
+    update_panels();
+    doupdate();
+}
+
 void standardcolors(int color)
 {
     if ((color) && (has_colors())) {
@@ -310,6 +320,7 @@ void standardcolors(int color)
 	IGMPATTR = COLOR_PAIR(10) | A_BOLD;
 	IGRPATTR = COLOR_PAIR(16) | A_BOLD;
 	ARPATTR = COLOR_PAIR(5) | A_BOLD;
+	GREATTR = COLOR_PAIR(1);
 	UNKNATTR = COLOR_PAIR(4) | A_BOLD;
     } else {
 	STDATTR = A_NORMAL;
@@ -329,6 +340,7 @@ void standardcolors(int color)
 	IGMPATTR = A_REVERSE;
 	IGRPATTR = A_REVERSE;
 	ARPATTR = A_BOLD;
+	GREATTR = A_BOLD;
 	UNKNATTR = A_BOLD;
     }
 }
