@@ -13,6 +13,7 @@ void printnomem();
 void printipcerr();
 void printkeyhelp(char *keytext, char *desc, WINDOW * win);
 void stdkeyhelp(WINDOW * win);
+void sortkeyhelp(void);
 void tabkeyhelp(WINDOW * win);
 void scrollkeyhelp();
 void stdexitkeyhelp();
@@ -20,5 +21,6 @@ void menukeyhelp();
 void listkeyhelp();
 void indicate(char *message);
 void printlargenum(unsigned long long i, WINDOW *win);
+void infobox(char *text, char *prompt);
 void standardcolors(int color);
 void refresh_screen(void);
