@@ -73,7 +73,8 @@
 #define UDPFLTIDFILE	get_path(T_WORKDIR, "udpfltchg.tag")
 #define OTHFLTIDFILE	get_path(T_WORKDIR, "othfltchg.tag")
 #define PKTSIZEIDFILE	get_path(T_WORKDIR, "packetsize.tag")
-
+#define PROCCOUNTFILE	get_path(T_WORKDIR, "iptrafprocesscount.dat")
+#define PROMISCLISTFILE get_path(T_WORKDIR, "promisclist.tmp")
 /*
  * The TCP filter list file
  */
