@@ -107,9 +107,13 @@ void save_filterlist(unsigned int protocol, struct ffnode *fltlist)
     int bw;
 
     if (protocol == F_TCP)
-	fd = open(TCPFLNAME, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
+	fd =
+	    open(TCPFLNAME, O_WRONLY | O_CREAT | O_TRUNC,
+		 S_IRUSR | S_IWUSR);
     else
-	fd = open(UDPFLNAME, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
+	fd =
+	    open(UDPFLNAME, O_WRONLY | O_CREAT | O_TRUNC,
+		 S_IRUSR | S_IWUSR);
 
     if (fd < 0) {
 	listfileerr(2);
@@ -133,7 +137,8 @@ void save_filterlist(unsigned int protocol, struct ffnode *fltlist)
     close(fd);
 }
 
-void get_filter_description(char *description, int *aborted, char *pre_edit)
+void get_filter_description(char *description, int *aborted,
+			    char *pre_edit)
 {
     struct FIELDLIST descfield;
     int dlgwintop;
