@@ -1,5 +1,4 @@
 
-
 /***
 
 hostmon.c - Host traffic monitor
@@ -24,6 +23,7 @@ details.
 #include <string.h>
 #include <unistd.h>
 #include <signal.h>
+#include <ctype.h>
 #include <sys/socket.h>
 #include <sys/types.h>
 #include <asm/types.h>
@@ -51,6 +51,7 @@ details.
 #include "error.h"
 #include "mode.h"
 #include "logvars.h"
+#include "promisc.h"
 
 #define SCROLLUP 0
 #define SCROLLDOWN 1
@@ -62,9 +63,16 @@ extern int daemonized;
  * from log.c, applicable only to this module 
  */
 
-extern void writeethlog(struct ethtabent *list, int units, long nsecs, FILE * logfile);
+extern void writeethlog(struct ethtabent *list, int units, long nsecs,
+			FILE * logfile);
 extern char *ltrim(char *buf);
 
+/*
+ * external declaration of sort status window routine
+ */
+ 
+extern void show_sort_statwin(WINDOW **win, PANEL **panel);
+
 /*
  * SIGUSR1 logfile rotation handler
  */
@@ -127,7 +135,8 @@ void initethtab(struct ethtab *table, int unit)
 
     dispmode(unit, unitstring);
 
-    wprintw(table->borderwin, " InRate and OutRate are in %s/sec ", unitstring);
+    wprintw(table->borderwin, " InRate and OutRate are in %s/sec ",
+	    unitstring);
 
     wattrset(table->tabwin, STDATTR);
     colorwin(table->tabwin);
@@ -232,7 +241,8 @@ struct ethtabent *addethentry(struct ethtab *table, unsigned int linktype,
     return ptemp;
 }
 
-struct ethtabent *in_ethtable(struct ethtab *table, unsigned int linktype, char *addr)
+struct ethtabent *in_ethtable(struct ethtab *table, unsigned int linktype,
+			      char *addr)
 {
     struct ethtabent *ptemp = table->head;
 
@@ -292,7 +302,7 @@ void printethent(struct ethtab *table, struct ethtabent *entry,
 	if (entry->un.desc.withdesc)
 	    wprintw(table->tabwin, " (%s)", entry->un.desc.desc);
 
-	wprintw(table->tabwin, " on %s", entry->un.desc.ifname);
+	wprintw(table->tabwin, " on %s       ", entry->un.desc.ifname);
 
 	entry->un.desc.printed = 1;
     } else {
@@ -343,6 +353,7 @@ void hostmonhelp()
 {
     move(LINES - 1, 1);
     scrollkeyhelp();
+    sortkeyhelp();
     stdexitkeyhelp();
 }
 
@@ -357,8 +368,8 @@ void printrates(struct ethtab *table, unsigned int target_row,
     }
 }
 
-void updateethrates(struct ethtab *table, int unit, time_t starttime, time_t now,
-		    unsigned int idx)
+void updateethrates(struct ethtab *table, int unit, time_t starttime,
+		    time_t now, unsigned int idx)
 {
     struct ethtabent *ptmp = table->head;
     unsigned int target_row = 0;
@@ -371,14 +382,18 @@ void updateethrates(struct ethtab *table, int unit, time_t starttime, time_t now
 	    ptmp->un.figs.past5 = 1;
 	    if (unit == KBITS) {
 		ptmp->un.figs.inrate =
-		    ((float) (ptmp->un.figs.inspanbr * 8 / 1000)) / ((float) (now - starttime));
+		    ((float) (ptmp->un.figs.inspanbr * 8 / 1000)) /
+		    ((float) (now - starttime));
 		ptmp->un.figs.outrate =
-		    ((float) (ptmp->un.figs.outspanbr * 8 / 1000)) / ((float) (now - starttime));
+		    ((float) (ptmp->un.figs.outspanbr * 8 / 1000)) /
+		    ((float) (now - starttime));
 	    } else {
 		ptmp->un.figs.inrate =
-		    ((float) (ptmp->un.figs.inspanbr / 1024)) / ((float) (now - starttime));
+		    ((float) (ptmp->un.figs.inspanbr / 1024)) /
+		    ((float) (now - starttime));
 		ptmp->un.figs.outrate =
-		    ((float) (ptmp->un.figs.outspanbr / 1024)) / ((float) (now - starttime));
+		    ((float) (ptmp->un.figs.outspanbr / 1024)) /
+		    ((float) (now - starttime));
 	    }
 	    if ((ptmp->index >= idx) && (ptmp->index <= idx + LINES - 5)) {
 		wattrset(table->tabwin, HIGHATTR);
@@ -442,6 +457,172 @@ void pageethwin(struct ethtab *table, int direction, int *idx)
     }
 }
 
+void show_hostsort_keywin(WINDOW ** win, PANEL ** panel)
+{
+    *win = newwin(13, 35, (LINES - 10) / 2, COLS - 40);
+    *panel = new_panel(*win);
+
+    wattrset(*win, BOXATTR);
+    colorwin(*win);
+    box(*win, ACS_VLINE, ACS_HLINE);
+
+    wattrset(*win, STDATTR);
+    mvwprintw(*win, 2, 2, "Select display field to sort by");
+    wmove(*win, 4, 2);
+    printkeyhelp("P", " - total packets in", *win);
+    wmove(*win, 5, 2);
+    printkeyhelp("I", " - IP packets in", *win);
+    wmove(*win, 6, 2);
+    printkeyhelp("B", " - total bytes in", *win);
+    wmove(*win, 7, 2);
+    printkeyhelp("K", " - total packets out", *win);
+    wmove(*win, 8, 2);
+    printkeyhelp("O", " - IP packets out", *win);
+    wmove(*win, 9, 2);
+    printkeyhelp("Y", " - total bytes out", *win);
+    wmove(*win, 10, 2);
+    printkeyhelp("Any other key", " - cancel sort", *win);
+    update_panels();
+    doupdate();
+}
+
+/*
+ * Swap two host table entries.
+ */
+
+void swaphostents(struct ethtab *list, struct ethtabent *p1,
+		  struct ethtabent *p2)
+{
+    register unsigned int tmp;
+    struct ethtabent *p1prevsaved;
+    struct ethtabent *p2nextsaved;
+
+    if (p1 == p2)
+	return;
+
+    tmp = p1->index;
+    p1->index = p2->index;
+    p2->index = tmp;
+    p1->next_entry->index = p1->index + 1;
+    p2->next_entry->index = p2->index + 1;
+
+    if (p1->prev_entry != NULL)
+	p1->prev_entry->next_entry = p2;
+    else
+	list->head = p2;
+
+    if (p2->next_entry->next_entry != NULL)
+	p2->next_entry->next_entry->prev_entry = p1->next_entry;
+    else
+	list->tail = p1->next_entry;
+
+    p2nextsaved = p2->next_entry->next_entry;
+    p1prevsaved = p1->prev_entry;
+
+    if (p1->next_entry->next_entry == p2) {
+	p2->next_entry->next_entry = p1;
+	p1->prev_entry = p2->next_entry;
+    } else {
+	p2->next_entry->next_entry = p1->next_entry->next_entry;
+	p1->prev_entry = p2->prev_entry;
+	p2->prev_entry->next_entry = p1;
+	p1->next_entry->next_entry->prev_entry = p2->next_entry;
+    }
+
+    p2->prev_entry = p1prevsaved;
+    p1->next_entry->next_entry = p2nextsaved;
+}
+
+void sort_hosttab(struct ethtab *list, int *idx, int command)
+{
+    struct ethtabent *ptemp1, *ptemp2;
+    struct ethtabent *ptemp2next;
+
+    unsigned int long long val1 = 0, val2 = 0;
+    unsigned int idxtmp;
+    
+    WINDOW *win;
+    PANEL *panel;
+
+    if (!list->head)
+	return;
+
+    command = toupper(command);
+
+    if ((command != 'P') && (command != 'I') && (command != 'B') &&
+       (command != 'K') && (command != 'O') && (command != 'Y'))
+        return;
+
+    show_sort_statwin(&win, &panel);
+    ptemp1 = list->head;
+
+    while (ptemp1 != NULL) {
+        ptemp1->un.desc.printed = 0;
+	ptemp2 = ptemp1->next_entry->next_entry;
+
+        mvwprintw(win, 2, 13, "%u%%", ptemp1->next_entry->index * 100 / list->tail->index);
+        update_panels();
+        doupdate();
+        
+	while (ptemp2 != NULL) {
+	    ptemp2next = ptemp2->next_entry->next_entry;
+
+	    switch (command) {
+	    case 'P':		/* order by packets in */
+		val1 = ptemp1->next_entry->un.figs.inpcount;
+		val2 = ptemp2->next_entry->un.figs.inpcount;
+		break;
+	    case 'I':		/* order by IP in */
+		val1 = ptemp1->next_entry->un.figs.inippcount;
+		val2 = ptemp2->next_entry->un.figs.inippcount;
+		break;
+	    case 'B':		/* order by bytes in */
+		val1 = ptemp1->next_entry->un.figs.inbcount;
+		val2 = ptemp2->next_entry->un.figs.inbcount;
+		break;
+	    case 'K':		/* order by Pkts out */
+		val1 = ptemp1->next_entry->un.figs.outpcount;
+		val2 = ptemp2->next_entry->un.figs.outpcount;
+		break;
+	    case 'O':		/* order by IP out */
+		val1 = ptemp1->next_entry->un.figs.outippcount;
+		val2 = ptemp2->next_entry->un.figs.outippcount;
+		break;
+	    case 'Y':		/* order by Bytes out */
+		val1 = ptemp1->next_entry->un.figs.outbcount;
+		val2 = ptemp2->next_entry->un.figs.outbcount;
+		break;
+	    }
+
+	    if (val1 < val2) {	/* descending sort */
+		swaphostents(list, ptemp1, ptemp2);
+		ptemp1 = ptemp2;
+	    }
+	    ptemp2 = ptemp2next;
+	}
+
+	ptemp1 = ptemp1->next_entry->next_entry;
+    }				/* end of the sorting */
+
+    del_panel(panel);
+    delwin(win);
+    update_panels();
+    doupdate();
+    
+    ptemp1 = list->firstvisible = list->head;
+    *idx = 1;
+    idxtmp = 0;
+    colorwin(list->tabwin);
+    while ((ptemp1) && (idxtmp <= LINES - 4)) {
+	printethent(list, ptemp1, *idx);
+	idxtmp++;
+	if (idxtmp <= LINES - 4)
+	    list->lastvisible = ptemp1;
+	ptemp1 = ptemp1->next_entry;
+    }
+
+}
+
 /*
  * The LAN station monitor
  */
@@ -481,32 +662,50 @@ void hostmon(const struct OPTIONS *options, int facilitytime, char *ifptr)
 
     int nomem = 0;
 
+    WINDOW *sortwin;
+    PANEL *sortpanel;
+    int keymode = 0;
+
+    struct promisc_states *promisc_list;
+
     if (!facility_active(LANMONIDFILE))
 	mark_facility(LANMONIDFILE, "LAN monitor");
     else {
-	write_error("LAN station monitor already active in another process",
-		    daemonized);
+	write_error
+	    ("LAN station monitor already active in another process",
+	     daemonized);
 	return;
     }
 
     if (ifptr != NULL) {
 	if (!iface_supported(ifptr)) {
 	    err_iface_unsupported();
-	    unlink(LANMONIDFILE);
+	    unmark_facility(LANMONIDFILE);
 	    return;
 	}
 	if (!iface_up(ifptr)) {
 	    err_iface_down();
-	    unlink(LANMONIDFILE);
+	    unmark_facility(LANMONIDFILE);
 	    return;
 	}
     }
+
     open_socket(&fd);
 
     if (fd < 0) {
-	unlink(LANMONIDFILE);
+	unmark_facility(LANMONIDFILE);
 	return;
     }
+
+    if ((first_active_facility()) && (options->promisc)) {
+	init_promisc_list(&promisc_list);
+	save_promisc_list(promisc_list);
+	srpromisc(1, promisc_list);
+	destroy_promisc_list(&promisc_list);
+    }
+
+    adjust_process_count(1);
+
     hostmonhelp();
 
     initethtab(&table, options->actmode);
@@ -523,7 +722,8 @@ void hostmon(const struct OPTIONS *options, int facilitytime, char *ifptr)
 	signal(SIGUSR1, rotate_lanlog);
 
     rotate_flag = 0;
-    writelog(logging, logfile, "******** LAN traffic monitor started");
+    writelog(logging, logfile,
+	     "******** LAN traffic monitor started ********");
     leaveok(table.tabwin, TRUE);
 
     exitloop = 0;
@@ -533,35 +733,48 @@ void hostmon(const struct OPTIONS *options, int facilitytime, char *ifptr)
     do {
 	getpacket(fd, buf, &fromaddr, &ch, &br, ifname, table.tabwin);
 
-	if (ch != ERR)
-	    switch (ch) {
-	    case KEY_UP:
-		scrollethwin(&table, SCROLLDOWN, &idx);
-		break;
-	    case KEY_DOWN:
-		scrollethwin(&table, SCROLLUP, &idx);
-		break;
-	    case KEY_PPAGE:
-	    case '-':
-		pageethwin(&table, SCROLLDOWN, &idx);
-		break;
-	    case KEY_NPAGE:
-	    case ' ':
-		pageethwin(&table, SCROLLUP, &idx);
-		break;
-	    case 12:
-	    case 'l':
-	    case 'L':
-		refresh_screen();
-		break;
-	    case 'q':
-	    case 'Q':
-	    case 'x':
-	    case 'X':
-	    case 27:
-	    case 24:
-		exitloop = 1;
+	if (ch != ERR) {
+	    if (keymode == 0) {
+		switch (ch) {
+		case KEY_UP:
+		    scrollethwin(&table, SCROLLDOWN, &idx);
+		    break;
+		case KEY_DOWN:
+		    scrollethwin(&table, SCROLLUP, &idx);
+		    break;
+		case KEY_PPAGE:
+		case '-':
+		    pageethwin(&table, SCROLLDOWN, &idx);
+		    break;
+		case KEY_NPAGE:
+		case ' ':
+		    pageethwin(&table, SCROLLUP, &idx);
+		    break;
+		case 12:
+		case 'l':
+		case 'L':
+		    refresh_screen();
+		    break;
+		case 's':
+		case 'S':
+		    show_hostsort_keywin(&sortwin, &sortpanel);
+		    keymode = 1;
+		    break;
+		case 'q':
+		case 'Q':
+		case 'x':
+		case 'X':
+		case 27:
+		case 24:
+		    exitloop = 1;
+		}
+	    } else if (keymode == 1) {
+		del_panel(sortpanel);
+		delwin(sortwin);
+		sort_hosttab(&table, &idx, ch);
+		keymode = 0;
 	    }
+	}
 	if (br > 0) {
 
 	    /* Is this session for any particular interface only? */
@@ -573,7 +786,8 @@ void hostmon(const struct OPTIONS *options, int facilitytime, char *ifptr)
 	    linktype = getlinktype(fromaddr.sll_hatype, ifname, -1, NULL);
 
 
-	    if ((linktype == LINK_ETHERNET) || (linktype == LINK_FDDI) || (linktype == LINK_PLIP)) {
+	    if ((linktype == LINK_ETHERNET) || (linktype == LINK_FDDI)
+		|| (linktype == LINK_PLIP)) {
 		if (fromaddr.sll_protocol == htons(ETH_P_IP))
 		    is_ip = 1;
 		else
@@ -584,18 +798,24 @@ void hostmon(const struct OPTIONS *options, int facilitytime, char *ifptr)
 		 */
 
 		if ((linktype == LINK_ETHERNET) || (linktype == LINK_PLIP)) {
-		    memcpy(scratch_saddr, ((struct ethhdr *) buf)->h_source, ETH_ALEN);
-		    memcpy(scratch_daddr, ((struct ethhdr *) buf)->h_dest, ETH_ALEN);
+		    memcpy(scratch_saddr,
+			   ((struct ethhdr *) buf)->h_source, ETH_ALEN);
+		    memcpy(scratch_daddr, ((struct ethhdr *) buf)->h_dest,
+			   ETH_ALEN);
 		    list = &elist;
 		} else if (linktype == LINK_FDDI) {
-		    memcpy(scratch_saddr, ((struct fddihdr *) buf)->saddr, FDDI_K_ALEN);
-		    memcpy(scratch_daddr, ((struct fddihdr *) buf)->daddr, FDDI_K_ALEN);
+		    memcpy(scratch_saddr, ((struct fddihdr *) buf)->saddr,
+			   FDDI_K_ALEN);
+		    memcpy(scratch_daddr, ((struct fddihdr *) buf)->daddr,
+			   FDDI_K_ALEN);
 		    list = &flist;
 		}
 		entry = in_ethtable(&table, linktype, scratch_saddr);
 
 		if ((entry == NULL) && (!nomem))
-		    entry = addethentry(&table, linktype, ifname, scratch_saddr, &nomem, list);
+		    entry =
+			addethentry(&table, linktype, ifname,
+				    scratch_saddr, &nomem, list);
 
 		if (entry != NULL) {
 		    updateethent(entry, br, is_ip, 1);
@@ -610,7 +830,9 @@ void hostmon(const struct OPTIONS *options, int facilitytime, char *ifptr)
 
 		entry = in_ethtable(&table, linktype, scratch_daddr);
 		if ((entry == NULL) && (!nomem))
-		    entry = addethentry(&table, linktype, ifname, scratch_daddr, &nomem, list);
+		    entry =
+			addethentry(&table, linktype, ifname,
+				    scratch_daddr, &nomem, list);
 
 		if (entry != NULL) {
 		    updateethent(entry, br, is_ip, 0);
@@ -626,16 +848,20 @@ void hostmon(const struct OPTIONS *options, int facilitytime, char *ifptr)
 	unow = tv.tv_sec * 1e+6 + tv.tv_usec;
 
 	if ((now - starttime) >= 5) {
-	    printelapsedtime(statbegin, now, LINES - 3, 15, table.borderwin);
+	    printelapsedtime(statbegin, now, LINES - 3, 15,
+			     table.borderwin);
 	    updateethrates(&table, options->actmode, starttime, now, idx);
 	    starttime = now;
 	}
 	if (((now - startlog) >= options->logspan) && (logging)) {
-	    writeethlog(table.head, options->actmode, now - statbegin, logfile);
+	    writeethlog(table.head, options->actmode, now - statbegin,
+			logfile);
 	    startlog = now;
 	}
-	if (((options->updrate != 0) && (now - updtime >= options->updrate)) ||
-	    ((options->updrate == 0) && (unow - updtime_usec >= HOSTMON_UPDATE_DELAY))) {
+	if (((options->updrate != 0)
+	     && (now - updtime >= options->updrate))
+	    || ((options->updrate == 0)
+		&& (unow - updtime_usec >= HOSTMON_UPDATE_DELAY))) {
 	    update_panels();
 	    doupdate();
 	    updtime = now;
@@ -643,16 +869,31 @@ void hostmon(const struct OPTIONS *options, int facilitytime, char *ifptr)
 	}
 	check_rotate_flag(&logfile, logging);
 
-	if ((facilitytime != 0) && (((now - statbegin) / 60) >= facilitytime))
+	if ((facilitytime != 0)
+	    && (((now - statbegin) / 60) >= facilitytime))
 	    exitloop = 1;
     } while (!exitloop);
 
+    close(fd);
+
+    if ((options->promisc) && (is_last_instance())) {
+	load_promisc_list(&promisc_list);
+	srpromisc(0, promisc_list);
+	destroy_promisc_list(&promisc_list);
+    }
+
+    adjust_process_count(-1);
+
     if (logging) {
 	signal(SIGUSR1, SIG_DFL);
-	writeethlog(table.head, options->actmode, time((time_t *) NULL) - statbegin, logfile);
-	writelog(logging, logfile, "******** LAN traffic monitor stopped");
+	writeethlog(table.head, options->actmode,
+		    time((time_t *) NULL) - statbegin, logfile);
+	writelog(logging, logfile,
+		 "******** LAN traffic monitor stopped ********");
 	fclose(logfile);
     }
+
+
     del_panel(table.tabpanel);
     delwin(table.tabwin);
     del_panel(table.borderpanel);
@@ -662,5 +903,5 @@ void hostmon(const struct OPTIONS *options, int facilitytime, char *ifptr)
     destroyethtab(&table);
     destroydesclist(&elist);
     destroydesclist(&flist);
-    unlink(LANMONIDFILE);
+    unmark_facility(LANMONIDFILE);
 }
