@@ -17,6 +17,7 @@ details.
 
 ***/
 
+#include <stdio.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -30,12 +31,65 @@ details.
 #include "error.h"
 #include "links.h"
 
-#define NUM_SUPPORTED_IFACES 8
+#define NUM_SUPPORTED_IFACES 9
 
 extern int daemonized;
 
 char ifaces[NUM_SUPPORTED_IFACES][6] =
-{"lo", "eth", "sl", "ppp", "ippp", "plip", "fddi", "isdn"};
+    { "lo", "eth", "sl", "ppp", "ippp", "plip", "fddi", "isdn", "dvb" };
+
+char *ltrim(char *buf)
+{
+    char *tmp = buf;
+
+    while ((*tmp == ' ') || (*tmp == '\t'))
+	tmp++;
+
+    strcpy(buf, tmp);
+    return buf;
+}
+
+/*
+ * Open /proc/net/dev and move file pointer past the two table header lines
+ * at the top of the file.
+ */
+
+FILE *open_procnetdev(void)
+{
+    FILE *fd;
+    char buf[161];
+
+    fd = fopen("/proc/net/dev", "r");
+
+    /*
+     * Read and discard the table header lines in the file
+     */
+
+    if (fd != NULL) {
+	fgets(buf, 160, fd);
+	fgets(buf, 160, fd);
+    }
+
+    return fd;
+}
+
+/*
+ * Get the next interface from /proc/net/dev.
+ */
+void get_next_iface(FILE * fd, char *ifname)
+{
+    char buf[161];
+
+    if (!feof(fd)) {
+	strcpy(buf, "");
+	fgets(buf, 160, fd);
+	if (strcmp(buf, "") != 0)
+	    strcpy(ifname, ltrim(strtok(buf, ":")));
+	else
+	    strcpy(ifname, "");
+    } else
+	strcpy(ifname, "");
+}
 
 /*
  * Determine if supplied interface is supported.
@@ -74,14 +128,12 @@ int iface_up(char *iface)
 
 void err_iface_unsupported(void)
 {
-    write_error("Specified interface not supported",
-		daemonized);
+    write_error("Specified interface not supported", daemonized);
 }
 
 void err_iface_down(void)
 {
-    write_error("Specified interface not active",
-		daemonized);
+    write_error("Specified interface not active", daemonized);
 }
 
 void isdn_iface_check(int *fd, char *ifname)
