@@ -53,6 +53,7 @@ details.
 #include "instances.h"
 #include "mode.h"
 #include "logvars.h"
+#include "promisc.h"
 
 #define SCROLLUP 0
 #define SCROLLDOWN 1
@@ -62,7 +63,8 @@ extern int daemonized;
 
 /* from log.c, applicable only to this module */
 
-void writegstatlog(struct iftab *table, int unit, unsigned long nsecs, FILE * logfile);
+void writegstatlog(struct iftab *table, int unit, unsigned long nsecs,
+		   FILE * logfile);
 void writedstatlog(char *ifname, int unit, float activity, float pps,
 		   float peakactivity, float peakpps,
 		   float peakactivity_in, float peakpps_in,
@@ -88,16 +90,6 @@ void rotate_dstat_log()
     signal(SIGUSR1, rotate_dstat_log);
 }
 
-char *ltrim(char *buf)
-{
-    char *tmp = buf;
-
-    while ((*tmp == ' ') || (*tmp == '\t'))
-	tmp++;
-
-    strcpy(buf, tmp);
-    return buf;
-}
 
 /*
  * Function to check if an interface is already in the interface list.
@@ -139,20 +131,16 @@ void initiflist(struct iflist **list)
 
     *list = NULL;
 
-    fd = fopen("/proc/net/dev", "r");
+    fd = open_procnetdev();
     if (fd == NULL) {
 	errbox("Unable to obtain interface list", ANYKEY_MSG, &resp);
 	return;
     }
-    fgets(buf, 160, fd);	/* dummy header lines */
-    fgets(buf, 160, fd);
 
-    while (!(feof(fd))) {
+    do {
 	strcpy(buf, "");
-	fgets(buf, 160, fd);
-	if (strcmp(buf, "") != 0) {
-	    strcpy(ifname, ltrim(strtok(buf, ":")));
-
+	get_next_iface(fd, ifname);
+	if (strcmp(ifname, "") != 0) {
 	    if (!(iface_supported(ifname)))
 		continue;
 
@@ -189,7 +177,8 @@ void initiflist(struct iflist **list)
 	    tail = itmp;
 	    itmp->next_entry = NULL;
 	}
-    }
+    } while (strcmp(ifname, "") != 0);
+
     fclose(fd);
 }
 
@@ -243,8 +232,8 @@ void destroyiflist(struct iflist *list)
     }
 }
 
-void updaterates(struct iftab *table, int unit, time_t starttime, time_t now,
-		 unsigned int idx)
+void updaterates(struct iftab *table, int unit, time_t starttime,
+		 time_t now, unsigned int idx)
 {
     struct iflist *ptmp = table->firstvisible;
 
@@ -252,10 +241,14 @@ void updaterates(struct iftab *table, int unit, time_t starttime, time_t now,
     do {
 	wmove(table->statwin, ptmp->index - idx, 52 * COLS / 80);
 	if (unit == KBITS) {
-	    ptmp->rate = ((float) (ptmp->spanbr * 8 / 1000)) / ((float) (now - starttime));
+	    ptmp->rate =
+		((float) (ptmp->spanbr * 8 / 1000)) /
+		((float) (now - starttime));
 	    wprintw(table->statwin, "%8.2f kbits/sec", ptmp->rate);
 	} else {
-	    ptmp->rate = ((float) (ptmp->spanbr / 1024)) / ((float) (now - starttime));
+	    ptmp->rate =
+		((float) (ptmp->spanbr / 1024)) /
+		((float) (now - starttime));
 	    wprintw(table->statwin, "%8.2f kbytes/sec", ptmp->rate);
 	}
 
@@ -347,7 +340,8 @@ void initiftab(struct iftab *table)
     colorwin(table->statwin);
     wattrset(table->statwin, BOXATTR);
     wmove(table->borderwin, LINES - 3, 32 * COLS / 80);
-    wprintw(table->borderwin, " Total, IP, NonIP, and BadIP are packet counts ");
+    wprintw(table->borderwin,
+	    " Total, IP, NonIP, and BadIP are packet counts ");
 }
 
 /*
@@ -389,7 +383,8 @@ void pagegstatwin(struct iftab *table, int direction, int *idx)
     int i = 1;
 
     if (direction == SCROLLUP) {
-	while ((i <= LINES - 5) && (table->lastvisible->next_entry != NULL)) {
+	while ((i <= LINES - 5)
+	       && (table->lastvisible->next_entry != NULL)) {
 	    i++;
 	    scrollgstatwin(table, direction, idx);
 	}
@@ -447,22 +442,33 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
     unsigned long updtime = 0;
     unsigned long long updtime_usec = 0;
 
+    struct promisc_states *promisc_list;
+
     if (!facility_active(GSTATIDFILE))
 	mark_facility(GSTATIDFILE, "general interface statistics");
     else {
-	write_error("General interface stats already active in another process",
-		    daemonized);
+	write_error
+	    ("General interface stats already active in another process",
+	     daemonized);
 	return;
     }
 
     open_socket(&fd);
 
     if (fd < 0) {
-	unlink(GSTATIDFILE);
+	unmark_facility(GSTATIDFILE);
 	return;
     }
     initiftab(&table);
 
+    if ((first_active_facility()) && (options->promisc)) {
+	init_promisc_list(&promisc_list);
+	save_promisc_list(promisc_list);
+	srpromisc(1, promisc_list);
+	destroy_promisc_list(&promisc_list);
+    }
+
+    adjust_process_count(1);
     if (logging) {
 	opentlog(&logfile, GSTATLOG);
 
@@ -473,7 +479,8 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 	signal(SIGUSR1, rotate_gstat_log);
 
     rotate_flag = 0;
-    writelog(logging, logfile, "******** General interface statistics started");
+    writelog(logging, logfile,
+	     "******** General interface statistics started ********");
 
     if (table.head != NULL) {
 	preparescreen(&table);
@@ -566,15 +573,19 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 
 	    if ((now - starttime) >= 7) {
 		updaterates(&table, options->actmode, starttime, now, idx);
-		printelapsedtime(statbegin, now, LINES - 3, 1, table.borderwin);
+		printelapsedtime(statbegin, now, LINES - 3, 1,
+				 table.borderwin);
 		starttime = now;
 	    }
 	    if (((now - startlog) >= options->logspan) && (logging)) {
-		writegstatlog(&table, options->actmode, time((time_t *) NULL) - statbegin, logfile);
+		writegstatlog(&table, options->actmode,
+			      time((time_t *) NULL) - statbegin, logfile);
 		startlog = now;
 	    }
-	    if (((options->updrate != 0) && (now - updtime >= options->updrate)) ||
-		((options->updrate == 0) && (unow - updtime_usec >= DEFAULT_UPDATE_DELAY))) {
+	    if (((options->updrate != 0)
+		 && (now - updtime >= options->updrate))
+		|| ((options->updrate == 0)
+		    && (unow - updtime_usec >= DEFAULT_UPDATE_DELAY))) {
 		update_panels();
 		doupdate();
 		updtime = now;
@@ -582,12 +593,22 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 	    }
 	    check_rotate_flag(&logfile, logging);
 
-	    if ((facilitytime != 0) && (((now - statbegin) / 60) >= facilitytime))
+	    if ((facilitytime != 0)
+		&& (((now - statbegin) / 60) >= facilitytime))
 		exitloop = 1;
 	}
 
 	close(fd);
     }
+
+    if ((options->promisc) && (is_last_instance())) {
+	load_promisc_list(&promisc_list);
+	srpromisc(0, promisc_list);
+	destroy_promisc_list(&promisc_list);
+    }
+
+    adjust_process_count(-1);
+
     del_panel(table.statpanel);
     delwin(table.statwin);
     del_panel(table.borderpanel);
@@ -597,13 +618,15 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 
     if (logging) {
 	signal(SIGUSR1, SIG_DFL);
-	writegstatlog(&table, options->actmode, time((time_t *) NULL) - statbegin, logfile);
-	writelog(logging, logfile, "******** General interface statistics stopped");
+	writegstatlog(&table, options->actmode,
+		      time((time_t *) NULL) - statbegin, logfile);
+	writelog(logging, logfile,
+		 "******** General interface statistics stopped ********");
 	fclose(logfile);
     }
     destroyiflist(table.head);
     destroy_isdn_table(&isdntable);
-    unlink(GSTATIDFILE);
+    unmark_facility(GSTATIDFILE);
 }
 
 
@@ -611,8 +634,10 @@ void printdetlabels(WINDOW * win, struct iftotals *totals)
 {
     /* TODO? : Adjust values for !80 column displays */
     wattrset(win, BOXATTR);
-    mvwprintw(win, 2, 14, "  Total      Total    Incoming   Incoming    Outgoing   Outgoing");
-    mvwprintw(win, 3, 14, "Packets      Bytes     Packets      Bytes     Packets      Bytes");
+    mvwprintw(win, 2, 14,
+	      "  Total      Total    Incoming   Incoming    Outgoing   Outgoing");
+    mvwprintw(win, 3, 14,
+	      "Packets      Bytes     Packets      Bytes     Packets      Bytes");
     wattrset(win, STDATTR);
     mvwprintw(win, 4, 2, "Total:");
     mvwprintw(win, 5, 2, "IP:");
@@ -634,8 +659,10 @@ void printdetlabels(WINDOW * win, struct iftotals *totals)
 
 void printstatrow(WINDOW * win, int row,
 		  unsigned long long total, unsigned long long btotal,
-	       unsigned long long total_in, unsigned long long btotal_in,
-	     unsigned long long total_out, unsigned long long btotal_out)
+		  unsigned long long total_in,
+		  unsigned long long btotal_in,
+		  unsigned long long total_out,
+		  unsigned long long btotal_out)
 {
     wmove(win, row, 12);
     printlargenum(total, win);
@@ -679,7 +706,6 @@ void printdetails(struct iftotals *totals, WINDOW * win)
 
     printstatrow(win, 9, totals->othtotal, totals->othbtotal,
 		 totals->othtotal_in, totals->othbtotal_in,
-
 		 totals->othtotal_out, totals->othbtotal_out);
 
     /* Print non-IP totals */
@@ -761,6 +787,8 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
 
     char unitstring[7];
 
+    struct promisc_states *promisc_list;
+
     /*
      * Mark this facility
      */
@@ -768,27 +796,39 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
     if (!facility_active(DSTATIDFILE))
 	mark_facility(DSTATIDFILE, "detailed interface statistics");
     else {
-	write_error("Detailed interface stats already active in another process",
-		    daemonized);
+	write_error
+	    ("Detailed interface stats already active in another process",
+	     daemonized);
 	return;
     }
 
     open_socket(&fd);
 
     if (fd < 0) {
-	unlink(DSTATIDFILE);
+	unmark_facility(DSTATIDFILE);
 	return;
     }
     if (!iface_supported(iface)) {
 	err_iface_unsupported();
-	unlink(DSTATIDFILE);
+	unmark_facility(DSTATIDFILE);
 	return;
     }
     if (!iface_up(iface)) {
 	err_iface_down();
-	unlink(DSTATIDFILE);
+	unmark_facility(DSTATIDFILE);
 	return;
     }
+
+    if ((first_active_facility()) && (options->promisc)) {
+	init_promisc_list(&promisc_list);
+	save_promisc_list(promisc_list);
+	srpromisc(1, promisc_list);
+	destroy_promisc_list(&promisc_list);
+    }
+
+    adjust_process_count(1);
+
+
     move(LINES - 1, 1);
     stdexitkeyhelp();
     statwin = newwin(LINES - 2, COLS, 1, 0);
@@ -816,7 +856,8 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
 	signal(SIGUSR1, rotate_dstat_log);
 
     rotate_flag = 0;
-    writelog(logging, logfile, "******** Detailed interface statistics started");
+    writelog(logging, logfile,
+	     "******** Detailed interface statistics started ********");
 
     printdetlabels(statwin, &totals);
     printdetails(&totals, statwin);
@@ -988,13 +1029,23 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
 	    wattrset(statwin, BOXATTR);
 	    printelapsedtime(statbegin, now, LINES - 3, 1, statwin);
 	    if (options->actmode == KBITS) {
-		activity = (float) (spanbr * 8 / 1000) / (float) (now - starttime);
-		activity_in = (float) (spanbr_in * 8 / 1000) / (float) (now - starttime);
-		activity_out = (float) (spanbr_out * 8 / 1000) / (float) (now - starttime);
+		activity =
+		    (float) (spanbr * 8 / 1000) / (float) (now -
+							   starttime);
+		activity_in =
+		    (float) (spanbr_in * 8 / 1000) / (float) (now -
+							      starttime);
+		activity_out =
+		    (float) (spanbr_out * 8 / 1000) / (float) (now -
+							       starttime);
 	    } else {
-		activity = (float) (spanbr / 1024) / (float) (now - starttime);
-		activity_in = (float) (spanbr_in / 1024) / (float) (now - starttime);
-		activity_out = (float) (spanbr_out / 1024) / (float) (now - starttime);
+		activity =
+		    (float) (spanbr / 1024) / (float) (now - starttime);
+		activity_in =
+		    (float) (spanbr_in / 1024) / (float) (now - starttime);
+		activity_out =
+		    (float) (spanbr_out / 1024) / (float) (now -
+							   starttime);
 	    }
 
 	    pps = (float) (spanpkt) / (float) (now - starttime);
@@ -1006,11 +1057,14 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
 	    starttime = now;
 
 	    wattrset(statwin, HIGHATTR);
-	    mvwprintw(statwin, 13, 19, "%8.1f %s/sec", activity, unitstring);
+	    mvwprintw(statwin, 13, 19, "%8.1f %s/sec", activity,
+		      unitstring);
 	    mvwprintw(statwin, 14, 19, "%8.1f packets/sec", pps);
-	    mvwprintw(statwin, 16, 19, "%8.1f %s/sec", activity_in, unitstring);
+	    mvwprintw(statwin, 16, 19, "%8.1f %s/sec", activity_in,
+		      unitstring);
 	    mvwprintw(statwin, 17, 19, "%8.1f packets/sec", pps_in);
-	    mvwprintw(statwin, 19, 19, "%8.1f %s/sec", activity_out, unitstring);
+	    mvwprintw(statwin, 19, 19, "%8.1f %s/sec", activity_out,
+		      unitstring);
 	    mvwprintw(statwin, 20, 19, "%8.1f packets/sec", pps_out);
 
 	    if (activity > peakactivity)
@@ -1039,8 +1093,10 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
 			  time((time_t *) NULL) - statbegin, logfile);
 	    startlog = now;
 	}
-	if (((options->updrate == 0) && (unow - updtime_usec >= DEFAULT_UPDATE_DELAY)) ||
-	    ((options->updrate != 0) && (now - updtime >= options->updrate))) {
+	if (((options->updrate == 0)
+	     && (unow - updtime_usec >= DEFAULT_UPDATE_DELAY))
+	    || ((options->updrate != 0)
+		&& (now - updtime >= options->updrate))) {
 	    printdetails(&totals, statwin);
 	    update_panels();
 	    doupdate();
@@ -1049,12 +1105,21 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
 	}
 	check_rotate_flag(&logfile, logging);
 
-	if ((facilitytime != 0) && (((now - statbegin) / 60) >= facilitytime))
+	if ((facilitytime != 0)
+	    && (((now - statbegin) / 60) >= facilitytime))
 	    exitloop = 1;
     }
 
     close(fd);
 
+    if ((options->promisc) && (is_last_instance())) {
+	load_promisc_list(&promisc_list);
+	srpromisc(0, promisc_list);
+	destroy_promisc_list(&promisc_list);
+    }
+
+    adjust_process_count(-1);
+
     if (logging) {
 	signal(SIGUSR1, SIG_DFL);
 	writedstatlog(iface, options->actmode, activity, pps,
@@ -1062,12 +1127,13 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
 		      peakactivity_in, peakpps_in,
 		      peakactivity_out, peakpps_out, &totals,
 		      time((time_t *) NULL) - statbegin, logfile);
-	writelog(logging, logfile, "******** Detailed interface statistics stopped");
+	writelog(logging, logfile,
+		 "******** Detailed interface statistics stopped ********");
 	fclose(logfile);
     }
     del_panel(statpanel);
     delwin(statwin);
-    unlink(DSTATIDFILE);
+    unmark_facility(DSTATIDFILE);
     destroy_isdn_table(&isdntable);
     update_panels();
     doupdate();
@@ -1091,7 +1157,7 @@ void selectiface(char *ifname, int withall, int *aborted)
     initiflist(&list);
     if (withall) {
 	ptmp = malloc(sizeof(struct iflist));
-	strcpy(ptmp->ifname, "All Interfaces");
+	strcpy(ptmp->ifname, "All interfaces");
 
 	ptmp->prev_entry = NULL;
 	list->prev_entry = ptmp;
