@@ -2,11 +2,10 @@
 
 IPTraf
 An IP Network Statistics Utility
-   		
-Written by Gerard Paul Java <riker@mozcom.com>
-Copyright (c) Gerard Paul Java 1997-1999
+Written by Gerard Paul Java <riker@seul.org>, <riker@mozcom.com>
+Copyright (c) Gerard Paul Java 1997-2000
    	
-Version 2.1.1
+Version 2.3.1
 Main Module
 
 This software is open-source; you can redistribute it and/or modify
@@ -21,6 +20,8 @@ details.
 
 ***/
 
+#define MAIN_MODULE
+
 #include <stdlib.h>
 #include <unistd.h>
 #include <curses.h>
@@ -41,7 +42,9 @@ details.
 #include "ifstats.h"
 #include "serv.h"
 #include "options.h"
+#include "promisc.h"
 #include "externs.h"
+#include "instances.h"
 #include "log.h"
 #include "attrs.h"
 #include "error.h"
@@ -53,10 +56,11 @@ details.
 
 int exitloop = 0;
 int daemonized = 0;
+int facility_running = 0;
+int is_first_instance;
+char active_facility_lockfile[64];
 
-extern void srpromisc(int mode);
 extern void about();
-extern void masq_warning();
 
 void removetags()
 {
@@ -70,6 +74,7 @@ void removetags()
     unlink(UDPFLTIDFILE);
     unlink(OTHFLTIDFILE);
     unlink(PKTSIZEIDFILE);
+    unlink(PROCCOUNTFILE);
 }
 
 /*
@@ -77,30 +82,31 @@ void removetags()
  * for the KILL.
  */
 
-void termhandler()
+void term_signal_handler(int signo)
 {
     erase();
     refresh();
     endwin();
-    removetags();
-    fprintf(stderr, "Received TERM signal\n");
-    fprintf(stderr, "IPTraf terminated\n\n");
-    exit(0);
-}
 
-/*
- * Handler for the hangup signal.  Simply removes all tags.
- */
+    if (signo != SIGHUP)
+	fprintf(stderr, "IPTraf process %u exiting on signal %d\n\n",
+		getpid(), signo);
+
+    if (strcmp(active_facility_lockfile, "") != 0) {
+	unlink(active_facility_lockfile);
+	adjust_process_count(-1);
+    }
+
+    if (is_first_instance)
+	unlink(IPTIDFILE);
 
-void huphandler()
-{
-    removetags();
     exit(0);
 }
 
 /* 
  * Handler for the SIGSEGV, Segmentation Fault.  Tries to clear the screen
- * and issue a better message than "Segmentation fault".
+ * and issue a better message than "Segmentation fault".  May not always
+ * clean up properly.
  */
 
 void segvhandler()
@@ -108,36 +114,51 @@ void segvhandler()
     erase();
     refresh();
     endwin();
-    removetags();
-    fprintf(stderr, "Fatal error: unable to allocate memory for a critical function\n");
-    fprintf(stderr, "IPTraf terminated abnormally\n\n");
+    fprintf(stderr, "Fatal: memory allocation error\n\n");
+    fprintf(stderr,
+	    "If you suspect a bug, please report the exact circumstances under which this\n");
+    fprintf(stderr,
+	    "error was generated.  If possible, include gdb or strace data which may point\n");
+    fprintf(stderr,
+	    "out where the error occured.  Bug reports may be sent in to iptraf@seul.org.\n\n");
+    fprintf(stderr,
+	    "An attempt will be made to clear all lock files, but if stale lock files\n");
+    fprintf(stderr,
+	    "remain, exit all other instances of IPTraf and restart with the -f\n");
+    fprintf(stderr, "command-line parameter.\n\n");
+    fprintf(stderr, "IPTraf process %u aborting on signal 11.\n\n",
+	    getpid());
+
+    if (strcmp(active_facility_lockfile, "") != 0)
+	unlink(active_facility_lockfile);
+
+    if (is_first_instance)
+	unlink(IPTIDFILE);
+
+    if (strcmp(active_facility_lockfile, "") != 0) {
+	unlink(active_facility_lockfile);
+	adjust_process_count(-1);
+    }
+
     exit(2);
 }
 
-void usr2handler()
+/*
+ * USR2 handler.  Used to normally exit a daemonized facility.
+ */
+
+void term_usr2_handler()
 {
     exitloop = 1;
 }
 
-
-int check_ip_masquerade(void)
-{
-    int fd;
-
-    fd = open("/proc/net/ip_masquerade", O_RDONLY);
-
-    if (fd >= 0) {
-	close(fd);
-	return 1;
-    }
-    return 0;
-}
-
 void init_break_menu(struct MENU *break_menu)
 {
     initmenu(break_menu, 5, 20, (LINES - 5) / 2, COLS / 2);
-    additem(break_menu, " By packet ^s^ize", "Displays packet counts by packet size range");
-    additem(break_menu, " By TCP/UDP ^p^ort", "Displays packet and byte counts by service port");
+    additem(break_menu, " By packet ^s^ize",
+	    "Displays packet counts by packet size range");
+    additem(break_menu, " By TCP/UDP ^p^ort",
+	    "Displays packet and byte counts by service port");
     additem(break_menu, " E^x^it menu", "Return to main menu");
 }
 
@@ -146,8 +167,7 @@ void init_break_menu(struct MENU *break_menu)
  */
 
 void program_interface(struct OPTIONS *options,
-		       int opt, int dontwarn, char *optarg,
-		       int facilitytime, int is_first_instance)
+		       int opt, char *optarg, int facilitytime)
 {
     struct MENU menu;
     struct MENU break_menu;
@@ -175,30 +195,26 @@ void program_interface(struct OPTIONS *options,
     indicate("");
     loadaddports(&ports);
 
-    /*
-     * Issue masquerading warning onscreen (intaractive mode) or in the log
-     * (daemonized mode).
-     */
-
-    if ((!dontwarn) && (check_ip_masquerade())) {
-	if (!daemonized)
-	    masq_warning();
-	else
-	    write_daemon_err("Warning: IP masquerading enabled, you may get strange results");
-    }
     if (opt == 0) {
 	about();
 
 	initmenu(&menu, 14, 35, (LINES - 14) / 2, (COLS - 35) / 2);
 
-	additem(&menu, " IP traffic ^m^onitor", "Displays current IP traffic information");
-	additem(&menu, " General interface ^s^tatistics", "Displays some statistics for attached interfaces");
-	additem(&menu, " ^D^etailed interface statistics", "Displays more statistics for a selected interface");
-	additem(&menu, " Statistical ^b^reakdowns", "Facilities for traffic counts by packet size or TCP/UDP port");
-	additem(&menu, " ^L^AN station monitor", "Displays statistics on detected LAN stations");
+	additem(&menu, " IP traffic ^m^onitor",
+		"Displays current IP traffic information");
+	additem(&menu, " General interface ^s^tatistics",
+		"Displays some statistics for attached interfaces");
+	additem(&menu, " ^D^etailed interface statistics",
+		"Displays more statistics for a selected interface");
+	additem(&menu, " Statistical ^b^reakdowns",
+		"Facilities for traffic counts by packet size or TCP/UDP port");
+	additem(&menu, " ^L^AN station monitor",
+		"Displays statistics on detected LAN stations");
 	additem(&menu, NULL, NULL);
-	additem(&menu, " ^T^CP display filters", "Manages TCP display filters");
-	additem(&menu, " Other ^p^rotocol filters", "Select which non-TCP protocols to display");
+	additem(&menu, " ^T^CP display filters",
+		"Manages TCP display filters");
+	additem(&menu, " Other ^p^rotocol filters",
+		"Select which non-TCP protocols to display");
 	additem(&menu, NULL, NULL);
 	additem(&menu, " C^o^nfigure", "Set various program options");
 	additem(&menu, NULL, NULL);
@@ -220,7 +236,8 @@ void program_interface(struct OPTIONS *options,
 			ifptr = NULL;
 
 		    ipmon(options,
-		    filtered, &fl, &ofilter, options->timeout, 0, ifptr);
+			  filtered, &fl, &ofilter, options->timeout, 0,
+			  ifptr);
 		}
 		break;
 	    case 2:
@@ -271,7 +288,7 @@ void program_interface(struct OPTIONS *options,
 		saveothfilter(&ofilter);
 		break;
 	    case 10:
-		setoptions(options, &ports, is_first_instance);
+		setoptions(options, &ports);
 		saveoptions(options);
 		break;
 	    case 12:
@@ -284,13 +301,15 @@ void program_interface(struct OPTIONS *options,
     } else {
 	switch (opt) {
 	case 'i':
-	    if ((strcmp(optarg, ALLSPEC) == 0) || (strcmp(optarg, "") == 0))
+	    if ((strcmp(optarg, ALLSPEC) == 0)
+		|| (strcmp(optarg, "") == 0))
 		ifptr = NULL;
 	    else
 		ifptr = optarg;
 
 	    ipmon(options,
-		  filtered, &fl, &ofilter, options->timeout, facilitytime, ifptr);
+		  filtered, &fl, &ofilter, options->timeout, facilitytime,
+		  ifptr);
 	    break;
 	case 'g':
 	    ifstats(options, facilitytime);
@@ -305,7 +324,8 @@ void program_interface(struct OPTIONS *options,
 	    packet_size_breakdown(options, optarg, facilitytime);
 	    break;
 	case 'l':
-	    if ((strcmp(optarg, ALLSPEC) == 0) || (strcmp(optarg, "") == 0))
+	    if ((strcmp(optarg, ALLSPEC) == 0)
+		|| (strcmp(optarg, "") == 0))
 		ifptr = NULL;
 	    else
 		ifptr = optarg;
@@ -329,26 +349,38 @@ void program_interface(struct OPTIONS *options,
 void commandhelp()
 {
     printf("\nSyntax:\n");
-    printf("    iptraf [ -f ][ -q ][ { -i iface | -g | -d iface | -s iface | -z iface |\n");
+    printf
+	("    iptraf [ -f ] [ { -i iface | -g | -d iface | -s iface | -z iface |\n");
     printf("           -l iface } [ -B ] [-t timeout ]]\n\n");
-    printf("Issue the iptraf command with no parameters for menu-driven operation.\n");
+    printf
+	("Issue the iptraf command with no parameters for menu-driven operation.\n");
     printf("These options can also be supplied to the command:\n\n");
-    printf("-i iface    - start the IP traffic monitor (use \"-i all\" for all interfaces)\n");
+    printf
+	("-i iface    - start the IP traffic monitor (use \"-i all\" for all interfaces)\n");
     printf("-g          - start the general interface statistics\n");
-    printf("-d iface    - start the detailed statistics facility on an interface\n");
-    printf("-s iface    - start the TCP and UDP monitor on an interface\n");
+    printf
+	("-d iface    - start the detailed statistics facility on an interface\n");
+    printf
+	("-s iface    - start the TCP and UDP monitor on an interface\n");
     printf("-z iface    - shows the packet size counts on an interface\n");
-    printf("-l iface    - start the LAN station monitor (\"-l all\" for all LAN interfaces)\n");
-    printf("-B          - Run in background (use only with one of the above parameters)\n");
-    printf("-t timeout  - when used with one of the above parameters, tells\n");
-    printf("              the facility to run only for the specified number of\n");
+    printf
+	("-l iface    - start the LAN station monitor (\"-l all\" for all LAN interfaces)\n");
+    printf
+	("-B          - Run in background (use only with one of the above parameters)\n");
+    printf
+	("-t timeout  - when used with one of the above parameters, tells\n");
+    printf
+	("              the facility to run only for the specified number of\n");
     printf("              minutes (timeout)\n");
-    printf("-f          - Remove all lock files.  This will allow multiple\n");
-    printf("              instances of facilities in different IPTraf processes.\n");
-    printf("              Use with great caution.  Normally used to recover from\n");
-    printf("              an abnormal termination.\n");
-    printf("-q          - suppress IP masquerading warning on startup\n\n");
-    printf("IPTraf %s Copyright (c) Gerard Paul Java 1997-1999\n", VERSION);
+    printf
+	("-f          - Remove all lock files.  This will allow multiple\n");
+    printf
+	("              instances of facilities in different IPTraf processes.\n");
+    printf
+	("              Use with great caution.  Normally used to recover from\n");
+    printf("              an abnormal termination.\n\n");
+    printf("IPTraf %s Copyright (c) Gerard Paul Java 1997-2000\n",
+	   VERSION);
 }
 
 int first_instance()
@@ -392,16 +424,15 @@ int main(int argc, char **argv)
     extern int optopt;
     int opt = 0;
     int command = 0;
-    int is_first_instance;
     char keyparm[12];
     int facilitytime = 0;
-    int dontwarn = 0;
 
 #ifndef ALLOWUSERS
     if (getuid() != 0) {
 	fprintf(stderr, "\nIPTraf Version %s\n", VERSION);
-	fprintf(stderr, "Copyright (c) Gerard Paul Java 1997-1999\n\n");
-	fprintf(stderr, "This program can be run only by the system administrator\n\n");
+	fprintf(stderr, "Copyright (c) Gerard Paul Java 1997-2000\n\n");
+	fprintf(stderr,
+		"This program can be run only by the system administrator\n\n");
 	exit(1);
     }
 #endif
@@ -426,12 +457,13 @@ int main(int argc, char **argv)
 		    fprintf(stderr, "\nInvalid time value\n\n");
 		    exit(1);
 		}
-	    } else if (opt == 'q') {
-		dontwarn = 1;
-	    } else if (opt == 'B') {	/* daemonization sequence */
+	    } else if (opt == 'B') {
 		daemonized = 1;
+	    } else if (opt == 'q') {
+		/* -q parameter now ignored, maintained for compatibility */
 	    } else if (opt == '?') {
-		fprintf(stderr, "\nInvalid option or missing parameter, use iptraf -h for help\n\n");
+		fprintf(stderr,
+			"\nInvalid option or missing parameter, use iptraf -h for help\n\n");
 		exit(1);
 	    } else if (opt != -1) {
 		if (optarg != 0) {
@@ -439,7 +471,7 @@ int main(int argc, char **argv)
 		    strncpy(keyparm, optarg, 11);
 		} else
 		    strcpy(keyparm, "");
-		    
+
 		command = opt;
 	    }
 	} while ((opt != '?') && (opt != -1));
@@ -463,13 +495,14 @@ int main(int argc, char **argv)
 	case 0:		/* child */
 	    setsid();
 	    freopen("/dev/null", "w", stdout);	/* redirect std output */
-	    freopen("/dev/null", "r", stdin);
-	    freopen("/dev/null", "w", stderr);
-	    signal(SIGUSR2, (void *) usr2handler);
-	    options.logging = 1;               /* daemon mode implies logging */
+	    freopen("/dev/null", "r", stdin);	/* redirect std input */
+	    freopen("/dev/null", "w", stderr);	/* redirect std error */
+	    signal(SIGUSR2, (void *) term_usr2_handler);
+	    options.logging = 1;	/* daemon mode implies logging */
 	    break;
 	case -1:		/* error */
-	    fprintf(stderr, "\nFork error, IPTraf cannot run in background\n\n");
+	    fprintf(stderr,
+		    "\nFork error, IPTraf cannot run in background\n\n");
 	    exit(1);
 	default:		/* parent */
 	    exit(0);
@@ -488,12 +521,13 @@ int main(int argc, char **argv)
 
     if ((LINES < 24) || (COLS < 80)) {
 	endwin();
-	fprintf(stderr, "\nThis program requires a screen size of at least 80 columns by 24 lines\n");
+	fprintf(stderr,
+		"\nThis program requires a screen size of at least 80 columns by 24 lines\n");
 	fprintf(stderr, "Please resize your window\n\n");
 	exit(1);
     }
-    signal(SIGTERM, (void *) termhandler);
-    signal(SIGHUP, (void *) huphandler);
+    signal(SIGTERM, (void *) term_signal_handler);
+    signal(SIGHUP, (void *) term_signal_handler);
     signal(SIGSEGV, (void *) segvhandler);
     signal(SIGTSTP, SIG_IGN);
     signal(SIGINT, SIG_IGN);
@@ -509,14 +543,7 @@ int main(int argc, char **argv)
     curs_set(0);
 #endif
 
-    if (is_first_instance)
-	srpromisc(options.promisc);
-
-    program_interface(&options, command, dontwarn,
-		      keyparm, facilitytime, is_first_instance);
-
-    if (is_first_instance)
-	srpromisc(0);
+    program_interface(&options, command, keyparm, facilitytime);
 
     endwin();
 
