@@ -20,12 +20,14 @@ details.
 #include <sys/wait.h>
 #include <errno.h>
 #include <signal.h>
+#include <ctype.h>
 #include "tcptable.h"
 #include "othptab.h"
 #include "isdntab.h"
 #include "packet.h"
 #include "ifaces.h"
 #include "ipcsum.h"
+#include "promisc.h"
 #include "utfdefs.h"
 #include "utfilter.h"
 #include "othfilter.h"
@@ -51,7 +53,8 @@ extern int daemonized;
 void writetcplog(int logging, FILE * fd, struct tcptableent *entry,
 		 unsigned int pktlen, char *message);
 void write_tcp_unclosed(int logging, FILE * fd, struct tcptable *table);
-
+void write_timeout_log(int logging, FILE *logfile, struct tcptableent *tcpnode);
+void show_sort_statwin(WINDOW **statwin, PANEL **statpanel);
 
 void rotate_ipmon_log()
 {
@@ -65,19 +68,20 @@ void rotate_ipmon_log()
 void ipmonhelp()
 {
     move(LINES - 1, 1);
-    printkeyhelp("Up/Dn/PgUp/PgDn", "-scrl actv win  ", stdscr);
-    move(LINES - 1, 50);
+    printkeyhelp("Up/Dn/PgUp/PgDn", "-scroll  ", stdscr);
+    move(LINES - 1, 43);
     printkeyhelp("W", "-chg actv win  ", stdscr);
+    printkeyhelp("S", "-sort TCP  ", stdscr);
     stdexitkeyhelp();
 };
 
 void uniq_help(int what)
 {
-    move(LINES - 1, 32);
+    move(LINES - 1, 25);
     if (!what)
 	printkeyhelp("M", "-more TCP info   ", stdscr);
     else
-	printkeyhelp("Lft/Rt", "-vert scrl  ", stdscr);
+	printkeyhelp("Lft/Rt", "-vtcl scrl  ", stdscr);
 }
 
 /* Mark general packet count indicators */
@@ -123,7 +127,8 @@ void markactive(int curwin, WINDOW * tw, WINDOW * ow)
 
 void show_ip_stats(WINDOW * win, unsigned long long iptotal,
 		   unsigned long long tcptotal,
-	       unsigned long long udptotal, unsigned long long icmptotal,
+		   unsigned long long udptotal,
+		   unsigned long long icmptotal,
 		   unsigned long long noniptotal)
 {
     wmove(win, 0, 6 * COLS / 80);
@@ -143,7 +148,8 @@ void show_ip_stats(WINDOW * win, unsigned long long iptotal,
  * Scrolling and paging routines for the upper (TCP) window
  */
 
-void scrollupperwin(struct tcptable *table, int direction, int *idx, int mode)
+void scrollupperwin(struct tcptable *table, int direction,
+		    unsigned long *idx, int mode)
 {
     char sp_buf[10];
     sprintf(sp_buf, "%%%dc", COLS - 2);
@@ -173,18 +179,21 @@ void scrollupperwin(struct tcptable *table, int direction, int *idx, int mode)
     }
 }
 
-void pageupperwin(struct tcptable *table, int direction, int *idx, int mode)
+void pageupperwin(struct tcptable *table, int direction,
+		  unsigned long *idx, int mode)
 {
     int i = 1;
 
     wattrset(table->tcpscreen, STDATTR);
     if (direction == SCROLLUP) {
-	while ((i <= table->imaxy - 3) && (table->lastvisible != table->tail)) {
+	while ((i <= table->imaxy - 3)
+	       && (table->lastvisible != table->tail)) {
 	    i++;
 	    scrollupperwin(table, direction, idx, mode);
 	}
     } else {
-	while ((i <= table->imaxy - 3) && (table->firstvisible != table->head)) {
+	while ((i <= table->imaxy - 3)
+	       && (table->firstvisible != table->head)) {
 	    i++;
 	    scrollupperwin(table, direction, idx, mode);
 	}
@@ -208,7 +217,8 @@ void scrolllowerwin(struct othptable *table, int direction)
 		whline(table->borderwin, ACS_HLINE, 8);
 		table->htstat = NOHTIND;
 	    }
-	    printothpentry(table, table->lastvisible, table->oimaxy - 1, 0, (FILE *) NULL);
+	    printothpentry(table, table->lastvisible, table->oimaxy - 1, 0,
+			   (FILE *) NULL);
 	}
     } else {
 	if (table->firstvisible != table->head) {
@@ -221,7 +231,8 @@ void scrolllowerwin(struct othptable *table, int direction)
 		whline(table->borderwin, ACS_HLINE, 8);
 		table->htstat = NOHTIND;
 	    }
-	    printothpentry(table, table->firstvisible, 0, 0, (FILE *) NULL);
+	    printothpentry(table, table->firstvisible, 0, 0,
+			   (FILE *) NULL);
 	}
     }
 }
@@ -231,18 +242,192 @@ void pagelowerwin(struct othptable *table, int direction)
     int i = 1;
 
     if (direction == SCROLLUP) {
-	while ((i <= table->oimaxy - 2) && (table->lastvisible != table->tail)) {
+	while ((i <= table->oimaxy - 2)
+	       && (table->lastvisible != table->tail)) {
 	    i++;
 	    scrolllowerwin(table, direction);
 	}
     } else {
-	while ((i <= table->oimaxy - 2) && (table->firstvisible != table->head)) {
+	while ((i <= table->oimaxy - 2)
+	       && (table->firstvisible != table->head)) {
 	    i++;
 	    scrolllowerwin(table, direction);
 	}
     }
 }
 
+/*
+ * Pop up sorting key window
+ */
+
+void show_tcpsort_win(WINDOW ** win, PANEL ** panel)
+{
+    *win = newwin(9, 35, (LINES - 8) / 2, COLS - 40);
+    *panel = new_panel(*win);
+
+    wattrset(*win, BOXATTR);
+    colorwin(*win);
+    box(*win, ACS_VLINE, ACS_HLINE);
+    wattrset(*win, STDATTR);
+    mvwprintw(*win, 2, 2, "Select sort criterion");
+    wmove(*win, 4, 2);
+    printkeyhelp("P", " - sort by packet count", *win);
+    wmove(*win, 5, 2);
+    printkeyhelp("B", " - sort by byte count", *win);
+    wmove(*win, 6, 2);
+    printkeyhelp("Any other key", " - cancel sort", *win);
+    update_panels();
+    doupdate();
+}
+
+/*
+ * Routine to swap two TCP entries.  p1 and p2 are pointers to TCP entries,
+ * but p1 must be ahead than p2.  It's a linked list thing.
+ */
+void swap_tcp_entries(struct tcptable *table,
+		      struct tcptableent *p1, struct tcptableent *p2)
+{
+    struct tcptableent *p2nextsaved;
+    struct tcptableent *p1prevsaved;
+    unsigned int tmp;
+
+    if (p1 == p2)
+	return;
+
+    tmp = p1->index;
+    p1->index = p2->index;
+    p2->index = tmp;
+
+    p1->next_entry->index = p1->index + 1;
+    p2->next_entry->index = p2->index + 1;
+
+    if (p1->prev_entry != NULL)
+	p1->prev_entry->next_entry = p2;
+    else
+	table->head = p2;
+
+    if (p2->next_entry->next_entry != NULL)
+	p2->next_entry->next_entry->prev_entry = p1->next_entry;
+    else
+	table->tail = p1->next_entry;
+
+    p2nextsaved = p2->next_entry->next_entry;
+    p1prevsaved = p1->prev_entry;
+
+    if (p1->next_entry->next_entry == p2) {	/* swapping adjacent entries */
+	p2->next_entry->next_entry = p1;
+	p1->prev_entry = p2->next_entry;
+    } else {
+	p2->next_entry->next_entry = p1->next_entry->next_entry;
+	p1->prev_entry = p2->prev_entry;
+	p2->prev_entry->next_entry = p1;
+	p1->next_entry->next_entry->prev_entry = p2->next_entry;
+    }
+
+    p2->prev_entry = p1prevsaved;
+    p1->next_entry->next_entry = p2nextsaved;
+}
+
+/* 
+ * Sort TCP window entries.  Contributed by Gal Laszlo.  Edited and
+ * modified, GPJ <riker@seul.org>
+ * Contributor's note: by LG - Gal Laszlo ( SlowTCP@hotmail.com ) 2000.07.19.
+ * Author's note: Special (accented) characters removed from contributed
+ *                source.
+ */
+void sortipents(struct tcptable *table, unsigned long *idx, int ch,
+		int mode, int logging, FILE *logfile,
+		time_t timeout, int *nomem)
+{
+    struct tcptableent *tcptmp1, *tcptmp2, *tcp_next2;
+    unsigned long long val1 = 0, val2 = 0;
+    unsigned long lastidx;
+    unsigned int idxtmp;
+    time_t now;
+    WINDOW *statwin;
+    PANEL *panel;
+    
+    if ((table->head == NULL)
+	|| (table->head->next_entry->next_entry == NULL))
+        return;
+
+    ch = toupper(ch);
+    
+    if ((ch != 'P') && (ch != 'B'))
+        return;
+            
+    show_sort_statwin(&statwin, &panel);
+    
+    lastidx = table->tail->index;
+    
+    tcptmp1 = table->head;
+
+    now = time(NULL);
+    
+    while (tcptmp1 != NULL) {
+	tcptmp2 = tcptmp1->next_entry->next_entry;
+	/*
+	 * While we're examining, might as well mark any timed
+	 * out entries too.
+	 */
+	 
+        if ((timeout > 0) && (tcptmp1->lastupdate / 60 > timeout) && !(tcptmp1->inclosed)) {
+            tcptmp1->timedout = tcptmp1->oth_connection->timedout = 1;
+            addtoclosedlist(table, tcptmp1, nomem);
+            if (logging)
+                write_timeout_log(logging, logfile, tcptmp1);
+        }
+    
+        mvwprintw(statwin, 2, 13, "%u%%", tcptmp1->next_entry->index * 100 / lastidx);
+        update_panels();
+        doupdate();
+             
+        /*
+         * Inner loop
+         */
+	while (tcptmp2 != NULL) {
+	    tcp_next2 = tcptmp2->next_entry->next_entry;
+	    switch (ch) {
+	    case 'B':
+		val1 =
+		    max(tcptmp1->bcount, tcptmp1->oth_connection->bcount);
+		val2 =
+		    max(tcptmp2->bcount, tcptmp2->oth_connection->bcount);
+		break;
+	    case 'P':
+		val1 =
+		    max(tcptmp1->pcount, tcptmp1->oth_connection->pcount);
+		val2 =
+		    max(tcptmp2->pcount, tcptmp2->oth_connection->pcount);
+		break;
+	    }
+	    if (val1 < val2) {	/* descending sort */
+		swap_tcp_entries(table, tcptmp1, tcptmp2);
+		tcptmp1 = tcptmp2;
+	    }
+
+	    tcptmp2 = tcp_next2;
+	}
+
+	tcptmp1 = tcptmp1->next_entry->next_entry;
+    }
+     
+    del_panel(panel);
+    delwin(statwin);
+    update_panels();
+    doupdate();
+    colorwin(table->tcpscreen);
+    tcptmp1 = table->firstvisible = table->head;
+    *idx = 1;
+    idxtmp = 0;
+    while ((tcptmp1 != NULL) && (idxtmp <= table->imaxy - 1)) {
+	printentry(table, tcptmp1, *idx, mode);
+	if (idxtmp++ <= table->imaxy - 1)
+	    table->lastvisible = tcptmp1;
+	tcptmp1 = tcptmp1->next_entry;
+    }
+}
+
 /*
  * Attempt to communicate with rvnamed, and if it doesn't respond, try
  * to start it.
@@ -269,7 +454,8 @@ int checkrvnamed(void)
 
 	    _exit(1);
 	} else if (cpid == -1) {
-	    write_error("Can't spawn new process; lookups will block", daemonized);
+	    write_error("Can't spawn new process; lookups will block",
+			daemonized);
 	    return 0;
 	} else {
 	    while (waitpid(cpid, &cstat, 0) < 0)
@@ -277,7 +463,8 @@ int checkrvnamed(void)
 		    break;
 
 	    if (WEXITSTATUS(cstat) == 1) {
-		write_error("Can't start rvnamed; lookups will block", daemonized);
+		write_error("Can't start rvnamed; lookups will block",
+			    daemonized);
 		return 0;
 	    } else {
 		sleep(1);
@@ -310,7 +497,7 @@ void ipmon(const struct OPTIONS *options, int filtered,
     int firstin = 0;
     char sp_buf[10];
 
-    unsigned int screen_idx = 1;
+    unsigned long screen_idx = 1;
 
     struct timeval tv;
     unsigned long starttime = 0;
@@ -319,10 +506,14 @@ void ipmon(const struct OPTIONS *options, int filtered,
     unsigned long updtime = 0;
     unsigned long long updtime_usec = 0;
     unsigned long long unow = 0;
+    unsigned long closedint = 0;
 
     WINDOW *statwin;
     PANEL *statpanel;
 
+    WINDOW *sortwin;
+    PANEL *sortpanel;
+
     FILE *logfile = NULL;
 
     int curwin = 0;
@@ -362,9 +553,12 @@ void ipmon(const struct OPTIONS *options, int filtered,
     int fragment = 0;		/* Set to 1 if not first fragment */
 
     int ch;
+    int keymode = 0;
     char msgstring[40];
     int nomem = 0;
 
+    struct promisc_states *promisc_list;
+
     int rvnfd = 0;
 
     /* 
@@ -374,28 +568,41 @@ void ipmon(const struct OPTIONS *options, int filtered,
     if (!facility_active(IPMONIDFILE))
 	mark_facility(IPMONIDFILE, "IP traffic monitor");
     else {
-	write_error("IP Traffic Monitor active in another process", daemonized);
+	write_error("IP Traffic Monitor active in another process",
+		    daemonized);
 	return;
     }
 
     if (ifptr != NULL) {
 	if (!iface_supported(ifptr)) {
 	    err_iface_unsupported();
-	    unlink(IPMONIDFILE);
+	    unmark_facility(IPMONIDFILE);
 	    return;
 	}
 	if (!iface_up(ifptr)) {
 	    err_iface_down();
-	    unlink(IPMONIDFILE);
+	    unmark_facility(IPMONIDFILE);
 	    return;
 	}
     }
+
     open_socket(&fd);
 
     if (fd < 0) {
-	unlink(IPMONIDFILE);
+	unmark_facility(IPMONIDFILE);
 	return;
     }
+
+    if (options->promisc) {
+	if (first_active_facility()) {
+	    init_promisc_list(&promisc_list);
+	    save_promisc_list(promisc_list);
+	    srpromisc(1, promisc_list);
+	}
+    }
+
+    adjust_process_count(1);
+
     init_tcp_table(&table);
     init_othp_table(&othptbl);
 
@@ -445,82 +652,110 @@ void ipmon(const struct OPTIONS *options, int filtered,
 
     exitloop = 0;
     gettimeofday(&tv, NULL);
-    starttime = timeint = tv.tv_sec;
+    starttime = timeint = closedint = tv.tv_sec;
 
     while (!exitloop) {
-	getpacket(fd, tpacket, &fromaddr, &ch, &readlen, ifname, table.tcpscreen);
+	getpacket(fd, tpacket, &fromaddr, &ch, &readlen, ifname,
+		  table.tcpscreen);
 
 	if (ch != ERR) {
-	    switch (ch) {
-	    case KEY_UP:
-		if (!curwin)
-		    scrollupperwin(&table, SCROLLDOWN, &screen_idx, mode);
-		else
-		    scrolllowerwin(&othptbl, SCROLLDOWN);
-		break;
-	    case KEY_DOWN:
-		if (!curwin)
-		    scrollupperwin(&table, SCROLLUP, &screen_idx, mode);
-		else
-		    scrolllowerwin(&othptbl, SCROLLUP);
-		break;
-	    case KEY_RIGHT:
-		if (curwin) {
-		    if (othptbl.strindex != VSCRL_OFFSET)
-			othptbl.strindex = VSCRL_OFFSET;
+	    if (keymode == 0) {
+		switch (ch) {
+		case KEY_UP:
+		    if (!curwin)
+			scrollupperwin(&table, SCROLLDOWN, &screen_idx,
+				       mode);
+		    else
+			scrolllowerwin(&othptbl, SCROLLDOWN);
+		    break;
+		case KEY_DOWN:
+		    if (!curwin)
+			scrollupperwin(&table, SCROLLUP, &screen_idx,
+				       mode);
+		    else
+			scrolllowerwin(&othptbl, SCROLLUP);
+		    break;
+		case KEY_RIGHT:
+		    if (curwin) {
+			if (othptbl.strindex != VSCRL_OFFSET)
+			    othptbl.strindex = VSCRL_OFFSET;
 
-		    refresh_othwindow(&othptbl);
-		}
-		break;
-	    case KEY_LEFT:
-		if (curwin) {
-		    if (othptbl.strindex != 0)
-			othptbl.strindex = 0;
+			refresh_othwindow(&othptbl);
+		    }
+		    break;
+		case KEY_LEFT:
+		    if (curwin) {
+			if (othptbl.strindex != 0)
+			    othptbl.strindex = 0;
 
-		    refresh_othwindow(&othptbl);
-		}
-		break;
-	    case KEY_PPAGE:
-	    case '-':
-		if (!curwin)
-		    pageupperwin(&table, SCROLLDOWN, &screen_idx, mode);
-		else
-		    pagelowerwin(&othptbl, SCROLLDOWN);
-		break;
-	    case KEY_NPAGE:
-	    case ' ':
-		if (!curwin)
-		    pageupperwin(&table, SCROLLUP, &screen_idx, mode);
-		else
-		    pagelowerwin(&othptbl, SCROLLUP);
-		break;
-	    case KEY_F(6):
-	    case 'w':
-	    case 'W':
-		curwin = ~curwin;
-		markactive(curwin, table.borderwin, othptbl.borderwin);
-		uniq_help(curwin);
-		break;
-	    case 'm':
-	    case 'M':
-		if (!curwin) {
-		    mode = ~mode;
+			refresh_othwindow(&othptbl);
+		    }
+		    break;
+		case KEY_PPAGE:
+		case '-':
+		    if (!curwin)
+			pageupperwin(&table, SCROLLDOWN, &screen_idx,
+				     mode);
+		    else
+			pagelowerwin(&othptbl, SCROLLDOWN);
+		    break;
+		case KEY_NPAGE:
+		case ' ':
+		    if (!curwin)
+			pageupperwin(&table, SCROLLUP, &screen_idx, mode);
+		    else
+			pagelowerwin(&othptbl, SCROLLUP);
+		    break;
+		case KEY_F(6):
+		case 'w':
+		case 'W':
+		case 9:
+		    curwin = ~curwin;
+		    markactive(curwin, table.borderwin, othptbl.borderwin);
+		    uniq_help(curwin);
+		    break;
+		case 'm':
+		case 'M':
+		    if (!curwin) {
+			mode = ~mode;
+			refreshtcpwin(&table, screen_idx, mode);
+		    }
+		    break;
+		case 12:
+		case 'l':
+		case 'L':
+		    refresh_screen();
+		    break;
+
+		case 'F':
+		case 'f':
+		case 'c':
+		case 'C':
+		    flushclosedentries(&table, tcptm, &screen_idx,
+				       options->logging, logfile);
 		    refreshtcpwin(&table, screen_idx, mode);
+		    break;
+		case 's':
+		case 'S':
+		    keymode = 1;
+		    show_tcpsort_win(&sortwin, &sortpanel);
+		    break;
+		case 'Q':
+		case 'q':
+		case 'X':
+		case 'x':
+		case 24:
+		case 27:
+		    exitloop = 1;
 		}
-		break;
-	    case 12:
-	    case 'l':
-	    case 'L':
-		refresh_screen();
-		break;
-
-	    case 'Q':
-	    case 'q':
-	    case 'X':
-	    case 'x':
-	    case 24:
-	    case 27:
-		exitloop = 1;
+	    } else if (keymode == 1) {
+		keymode = 0;
+		del_panel(sortpanel);
+		delwin(sortwin);
+		update_panels();
+		doupdate();
+		sortipents(&table, &screen_idx, ch, mode, logging, logfile,
+		           options->timeout, &nomem);
 	    }
 	}
 	if (readlen > 0) {
@@ -538,7 +773,11 @@ void ipmon(const struct OPTIONS *options, int filtered,
 				   isdn_fd, &isdntable);
 
 	    fromaddr.sll_protocol = htons(fromaddr.sll_protocol);
-	    adjustpacket(tpacket, linktype, &packet, aligned_buf, &readlen);
+	    adjustpacket(tpacket, linktype, &packet, aligned_buf,
+			 &readlen);
+
+	    if (packet == NULL)
+		continue;
 
 	    if (fromaddr.sll_protocol != ETH_P_IP) {
 		if ((fromaddr.sll_protocol == ETH_P_ARP) ||
@@ -557,10 +796,13 @@ void ipmon(const struct OPTIONS *options, int filtered,
 
 		if ((nifltok) && (!nomem))
 		    othpent = add_othp_entry(&othptbl, &table,
-				     0, 0, NOT_IP, fromaddr.sll_protocol,
-			     linktype, (char *) tpacket, packet, readlen,
-				       ifname, 0, 0, 0, logging, logfile,
-					  options->servnames, 0, &nomem);
+					     0, 0, NOT_IP,
+					     fromaddr.sll_protocol,
+					     linktype, (char *) tpacket,
+					     packet, readlen, ifname, 0, 0,
+					     0, logging, logfile,
+					     options->servnames, 0,
+					     &nomem);
 
 		noniptotal += readlen;
 		continue;
@@ -569,10 +811,6 @@ void ipmon(const struct OPTIONS *options, int filtered,
 	     * move past data link header, if present
 	     */
 
-
-	    if (packet == NULL)
-		continue;
-
 	    ippacket = (struct iphdr *) packet;
 	    iphlen = ippacket->ihl * 4;
 	    transpacket = (struct tcphdr *) (packet + iphlen);
@@ -611,7 +849,9 @@ void ipmon(const struct OPTIONS *options, int filtered,
 		 */
 
 		if ((ntohs(ippacket->frag_off) & 0x3fff) != 0) {
-		    br = processfragment(ippacket, &sport, &dport, &firstin);
+		    br =
+			processfragment(ippacket, &sport, &dport,
+					&firstin);
 		    if (!firstin)
 			continue;
 		} else {
@@ -626,14 +866,15 @@ void ipmon(const struct OPTIONS *options, int filtered,
 
 		if ((filtered) &&
 		    (!(utfilter(fl, ippacket->saddr, ippacket->daddr,
-				ntohs(sport), ntohs(dport))))) {
-		    show_ip_stats(statwin, iptotal, tcptotal, udptotal, icmptotal, noniptotal);
+				ntohs(sport), ntohs(dport), IPPROTO_TCP)))) {
+		    show_ip_stats(statwin, iptotal, tcptotal, udptotal,
+				  icmptotal, noniptotal);
 		    goto cont;
 		}
-		tcpentry = in_table(&table, ippacket->saddr, ippacket->daddr,
-				    ntohs(sport), ntohs(dport), ifname,
-				    tcptm, logging, logfile,
-				    &nomem);
+		tcpentry =
+		    in_table(&table, ippacket->saddr, ippacket->daddr,
+			     ntohs(sport), ntohs(dport), ifname, tcptm,
+			     logging, logfile, &nomem);
 
 		/* 
 		 * Add a new entry if it doesn't exist, and if it is a 
@@ -656,14 +897,18 @@ void ipmon(const struct OPTIONS *options, int filtered,
 
 			if (!nomem) {
 			    tcpentry = addentry(&table,
-					 (unsigned long) ippacket->saddr,
-					 (unsigned long) ippacket->daddr,
-				sport, dport, ippacket->protocol, ifname,
-						options->revlook, rvnfd,
-					     options->servnames, &nomem);
+						(unsigned long)
+						ippacket->saddr,
+						(unsigned long)
+						ippacket->daddr, sport,
+						dport, ippacket->protocol,
+						ifname, options->revlook,
+						rvnfd, options->servnames,
+						&nomem);
 
 			    if (tcpentry != NULL)
-				printentry(&table, tcpentry->oth_connection,
+				printentry(&table,
+					   tcpentry->oth_connection,
 					   screen_idx, mode);
 			}
 		    }
@@ -696,13 +941,18 @@ void ipmon(const struct OPTIONS *options, int filtered,
 			    p_dstat = tcpentry->d_fstat;
 			}
 			updateentry(&table, tcpentry, transpacket, nbplen,
-			br, ippacket->frag_off, logging, options->revlook,
-				    rvnfd, logfile, &nomem);
+				    br, ippacket->frag_off, logging,
+				    options->revlook, rvnfd, logfile,
+				    &nomem);
 
-			if ((options->revlook) && (((p_sstat != RESOLVED) && (tcpentry->s_fstat == RESOLVED)) ||
-						   ((p_dstat != RESOLVED) && (tcpentry->d_fstat == RESOLVED)))) {
+			if ((options->revlook)
+			    && (((p_sstat != RESOLVED)
+				 && (tcpentry->s_fstat == RESOLVED))
+				|| ((p_dstat != RESOLVED)
+				    && (tcpentry->d_fstat == RESOLVED)))) {
 			    clearaddr(&table, tcpentry, screen_idx);
-			    clearaddr(&table, tcpentry->oth_connection, screen_idx);
+			    clearaddr(&table, tcpentry->oth_connection,
+				      screen_idx);
 			}
 			printentry(&table, tcpentry, screen_idx, mode);
 
@@ -717,11 +967,17 @@ void ipmon(const struct OPTIONS *options, int filtered,
 			 */
 
 			if (((tcpentry->oth_connection->finsent == 2) &&	/* FINed and ACKed */
-			     (ntohl(transpacket->seq) == tcpentry->oth_connection->finack)) ||
-			    ((options->revlook) &&
-			     (((p_sstat != RESOLVED) && (tcpentry->s_fstat == RESOLVED)) ||
-			      ((p_dstat != RESOLVED) && (tcpentry->d_fstat == RESOLVED)))))
-			    printentry(&table, tcpentry->oth_connection, screen_idx, mode);
+			     (ntohl(transpacket->seq) ==
+			      tcpentry->oth_connection->finack))
+			    || ((options->revlook)
+				&&
+				(((p_sstat
+				   != RESOLVED)
+				  && (tcpentry->s_fstat == RESOLVED))
+				 || ((p_dstat != RESOLVED)
+				     && (tcpentry->d_fstat == RESOLVED)))))
+			    printentry(&table, tcpentry->oth_connection,
+				       screen_idx, mode);
 		    }
 		}
 	    } else {		/* now for the other IP protocols */
@@ -737,9 +993,12 @@ void ipmon(const struct OPTIONS *options, int filtered,
 		     * is received.
 		     */
 
-		    if (((struct icmphdr *) transpacket)->type == ICMP_DEST_UNREACH)
-			process_dest_unreach(&table, (char *) transpacket,
-					     ifname, &nomem);
+		    if (((struct icmphdr *) transpacket)->type ==
+			ICMP_DEST_UNREACH) process_dest_unreach(&table,
+								(char *)
+								transpacket,
+								ifname,
+								&nomem);
 
 		    icmptotal += nbplen;
 		} else
@@ -751,41 +1010,79 @@ void ipmon(const struct OPTIONS *options, int filtered,
 		 */
 
 		if ((!nomem) && (othfilterok(ofilter, ippacket->protocol,
-					ippacket->saddr, ippacket->daddr,
-			  ntohs(((struct udphdr *) transpacket)->source),
-			ntohs(((struct udphdr *) transpacket)->dest)))) {
-		    othpent = add_othp_entry(&othptbl, &table,
-					ippacket->saddr, ippacket->daddr,
-					     IS_IP, ippacket->protocol, 0,
-				     (char *) transpacket, (char *) NULL,
-					nbplen, ifname, options->revlook,
-					  rvnfd, tcptm, logging, logfile,
-					     options->servnames, fragment,
-					     &nomem);
+					     ippacket->saddr,
+					     ippacket->daddr,
+					     ntohs(((struct udphdr *)
+						    transpacket)->source),
+					     ntohs(((struct udphdr *)
+						    transpacket)->dest)))) {
+		    othpent =
+			add_othp_entry(&othptbl, &table, ippacket->saddr,
+				       ippacket->daddr, IS_IP,
+				       ippacket->protocol, 0,
+				       (char *) transpacket, (char *) NULL,
+				       nbplen, ifname, options->revlook,
+				       rvnfd, tcptm, logging, logfile,
+				       options->servnames, fragment,
+				       &nomem);
 		}
 	    }
 
 	  cont:
-	    show_ip_stats(statwin, iptotal, tcptotal, udptotal, icmptotal, noniptotal);
+	    show_ip_stats(statwin, iptotal, tcptotal, udptotal, icmptotal,
+			  noniptotal);
 	}
 	gettimeofday(&tv, NULL);
 	now = tv.tv_sec;
 	unow = tv.tv_sec * 1e+06 + tv.tv_usec;
-
+        
+        /* 
+         * Print timer at bottom of screen
+         */
+         
 	if (now - timeint >= 5) {
-	    printelapsedtime(starttime, now, othptbl.obmaxy - 1, 15, othptbl.borderwin);
+	    printelapsedtime(starttime, now, othptbl.obmaxy - 1, 15,
+			     othptbl.borderwin);
 	    timeint = now;
 	}
-	if (((options->updrate != 0) && (now - updtime >= options->updrate)) ||
-	    ((options->updrate == 0) && (unow - updtime_usec >= DEFAULT_UPDATE_DELAY))) {
+
+	/*
+	 * Automatically clear closed/timed out entries
+	 */
+
+	if ((options->closedint != 0) &&
+	    ((now - closedint) / 60 >= options->closedint)) {
+	    flushclosedentries(&table, tcptm, &screen_idx,
+			       options->logging, logfile);
+	    refreshtcpwin(&table, screen_idx, mode);
+	    closedint = now;
+	}
+
+	/*
+	 * Update screen at configured intervals.
+	 */
+
+	if (((options->updrate != 0)
+	     && (now - updtime >= options->updrate))
+	    || ((options->updrate == 0)
+		&& (unow - updtime_usec >= DEFAULT_UPDATE_DELAY))) {
 	    update_panels();
 	    doupdate();
 	    updtime = now;
 	    updtime_usec = unow;
 	}
-	if ((facilitytime != 0) && (((now - starttime) / 60) >= facilitytime))
+	
+	/*
+	 * Terminate facility should a lifetime be specified at the
+	 * command line
+	 */
+	if ((facilitytime != 0)
+	    && (((now - starttime) / 60) >= facilitytime))
 	    exitloop = 1;
 
+        /*
+         * Close and rotate log file if signal was received
+         */
 	if ((rotate_flag == 1) && (logging)) {
 	    announce_rotate_prepare(logfile);
 	    write_tcp_unclosed(logging, logfile, &table);
@@ -802,6 +1099,14 @@ void ipmon(const struct OPTIONS *options, int filtered,
 
     close_rvn_socket(rvnfd);
     close(isdn_fd);
+
+    if ((options->promisc) && (is_last_instance())) {
+	load_promisc_list(&promisc_list);
+	srpromisc(0, promisc_list);
+	destroy_promisc_list(&promisc_list);
+    }
+    adjust_process_count(-1);
+
     del_panel(table.tcppanel);
     del_panel(table.borderpanel);
     del_panel(othptbl.othppanel);
@@ -819,8 +1124,9 @@ void ipmon(const struct OPTIONS *options, int filtered,
     destroyothptable(&othptbl);
     destroyfraglist();
     destroy_isdn_table(&isdntable);
-    writelog(logging, logfile, "******** IP traffic monitor stopped\n");
-    unlink(IPMONIDFILE);
+    writelog(logging, logfile,
+	     "******** IP traffic monitor stopped ********\n");
+    unmark_facility(IPMONIDFILE);
     if (logfile != NULL)
 	fclose(logfile);
 
