@@ -190,7 +190,8 @@ void operate_descselect(struct desclist *list, struct desclistent **node,
 		    wprintw(win, sp_buf, ' ');
 		    scrollok(win, 1);
 		    wmove(win, 17, 2);
-		    wprintw(win, "%s    %s", (*node)->rec.address, (*node)->rec.desc);
+		    wprintw(win, "%s    %s", (*node)->rec.address,
+			    (*node)->rec.desc);
 		}
 	    }
 	    break;
@@ -205,7 +206,8 @@ void operate_descselect(struct desclist *list, struct desclistent **node,
 		    wmove(win, 0, 0);
 		    wprintw(win, sp_buf, ' ');
 		    wmove(win, 0, 2);
-		    wprintw(win, "%s    %s", (*node)->rec.address, (*node)->rec.desc);
+		    wprintw(win, "%s    %s", (*node)->rec.address,
+			    (*node)->rec.desc);
 		}
 	    }
 	    break;
@@ -226,7 +228,8 @@ void operate_descselect(struct desclist *list, struct desclistent **node,
     } while (!exitloop);
 }
 
-void selectdesc(struct desclist *list, struct desclistent **node, int *aborted)
+void selectdesc(struct desclist *list, struct desclistent **node,
+		int *aborted)
 {
     WINDOW *bw;
     PANEL *bp;
@@ -268,7 +271,8 @@ void selectdesc(struct desclist *list, struct desclistent **node, int *aborted)
     doupdate();
 }
 
-void descdlg(struct descrec *rec, char *initaddr, char *initdesc, int *aborted)
+void descdlg(struct descrec *rec, char *initaddr, char *initdesc,
+	     int *aborted)
 {
     WINDOW *win;
     PANEL *panel;
@@ -375,9 +379,12 @@ void ethdescmgr(unsigned int linktype)
     loaddesclist(&list, linktype);
 
     initmenu(&menu, 6, 31, (LINES - 6) / 2, (COLS - 31) / 2);
-    additem(&menu, " ^A^dd description...", "Adds a description for an Ethernet address");
-    additem(&menu, " ^E^dit description...", "Modifies an existing Ethernet address description");
-    additem(&menu, " ^D^elete description...", "Deletes an existing Ethernet address description");
+    additem(&menu, " ^A^dd description...",
+	    "Adds a description for an Ethernet address");
+    additem(&menu, " ^E^dit description...",
+	    "Modifies an existing Ethernet address description");
+    additem(&menu, " ^D^elete description...",
+	    "Deletes an existing Ethernet address description");
     additem(&menu, " E^x^it menu", "Returns to the main menu");
 
     do {
