@@ -62,6 +62,7 @@ void opentlog(FILE ** fd, char *logfilename)
 
 void genatime(time_t now, char *atime)
 {
+    bzero(atime, TIME_TARGET_MAX);
     strncpy(atime, ctime(&now), 26);
     atime[strlen(atime) - 1] = '\0';
 }
@@ -74,6 +75,8 @@ void writelog(int logging, FILE * fd, char *msg)
 	genatime(time((time_t *) NULL), atime);
 	fprintf(fd, "%s; %s\n", atime, msg);
     }
+
+    fflush(fd);
 }
 
 void write_daemon_err(char *msg)
@@ -83,7 +86,7 @@ void write_daemon_err(char *msg)
 
     genatime(time((time_t *) NULL), atime);
     fd = fopen(DAEMONLOG, "a");
-    fprintf(fd, "%s: %s\n", atime, msg);
+    fprintf(fd, "%s iptraf[%u]: %s\n", atime, getpid(), msg);
     fclose(fd);
 }
 
@@ -109,26 +112,27 @@ void write_tcp_unclosed(int logging, FILE * fd, struct tcptable *table)
 
     while (entry != NULL) {
 	if ((entry->finsent == 0) &&
-	    ((entry->stat & FLAG_RST) == 0) &&
-	    (!(entry->inclosed))) {
-	    sprintf(msgbuf, "TCP; %s; active; from %s:%s to %s:%s; %lu packets, %lu bytes",
+	    ((entry->stat & FLAG_RST) == 0) && (!(entry->inclosed))) {
+	    sprintf(msgbuf,
+		    "TCP; %s; active; from %s:%s to %s:%s; %lu packets, %lu bytes",
 		    entry->ifname, entry->s_fqdn, entry->s_sname,
-	    entry->d_fqdn, entry->d_sname, entry->pcount, entry->bcount);
+		    entry->d_fqdn, entry->d_sname, entry->pcount,
+		    entry->bcount);
 	    writelog(logging, fd, msgbuf);
 	}
 	entry = entry->next_entry;
     }
 }
 
-void writeothplog(int logging, FILE * fd, char *protname, char *description,
-		  char *additional, int is_ip, int withmac,
-		  struct othptabent *entry)
+void writeothplog(int logging, FILE * fd, char *protname,
+		  char *description, char *additional, int is_ip,
+		  int withmac, struct othptabent *entry)
 {
     char msgbuffer[MSGSTRING_MAX];
     char scratchpad[MSGSTRING_MAX];
 
     if (logging) {
-	bzero(msgbuffer, 240);
+	bzero(msgbuffer, MSGSTRING_MAX);
 
 	strcpy(msgbuffer, protname);
 	strcat(msgbuffer, "; ");
@@ -142,9 +146,11 @@ void writeothplog(int logging, FILE * fd, char *protname, char *description,
 			entry->s_fqdn, entry->un.udp.s_sname,
 			entry->d_fqdn, entry->un.udp.d_sname);
 	    else
-		sprintf(scratchpad, " from %s to %s", entry->s_fqdn, entry->d_fqdn);
+		sprintf(scratchpad, " from %s to %s", entry->s_fqdn,
+			entry->d_fqdn);
 	} else
-	    sprintf(scratchpad, " from %s to %s ", entry->smacaddr, entry->dmacaddr);
+	    sprintf(scratchpad, " from %s to %s ", entry->smacaddr,
+		    entry->dmacaddr);
 
 	strcat(msgbuffer, scratchpad);
 	strcpy(scratchpad, "");
@@ -161,31 +167,58 @@ void writeothplog(int logging, FILE * fd, char *protname, char *description,
     }
 }
 
-void writegstatlog(struct iftab *table, int unit, unsigned long nsecs, FILE * fd)
+void write_timeout_log(int logging, FILE * logfile,
+		       struct tcptableent *tcpnode)
+{
+    char msgstring[MSGSTRING_MAX];
+
+    if (logging) {
+	sprintf(msgstring,
+		"TCP; Connection %s:%s to %s:%s timed out, %lu packets, %lu bytes; opposite direction %lu packets, %lu bytes",
+		tcpnode->s_fqdn, tcpnode->s_sname,
+		tcpnode->d_fqdn, tcpnode->d_sname,
+		tcpnode->pcount, tcpnode->bcount,
+		tcpnode->oth_connection->pcount,
+		tcpnode->oth_connection->bcount);
+	writelog(logging, logfile, msgstring);
+    }
+}
+
+void writegstatlog(struct iftab *table, int unit, unsigned long nsecs,
+		   FILE * fd)
 {
     struct iflist *ptmp = table->head;
     char atime[TIME_TARGET_MAX];
     char unitstring[7];
 
     genatime(time((time_t *) NULL), atime);
-    fprintf(fd, "\n*** General interface statistics log generated %s\n\n", atime);
+    fprintf(fd, "\n*** General interface statistics log generated %s\n\n",
+	    atime);
 
     while (ptmp != NULL) {
 
-	fprintf(fd, "%s: %llu total, %llu IP, %llu non-IP, %lu IP checksum errors",
-		ptmp->ifname, ptmp->total, ptmp->iptotal, ptmp->noniptotal, ptmp->badtotal);
+	fprintf(fd,
+		"%s: %llu total, %llu IP, %llu non-IP, %lu IP checksum errors",
+		ptmp->ifname, ptmp->total, ptmp->iptotal, ptmp->noniptotal,
+		ptmp->badtotal);
 
 	if (nsecs > 5) {
 	    dispmode(unit, unitstring);
 
 	    if (unit == KBITS) {
-		fprintf(fd, ", average activity %.2f %s/s", (float) (ptmp->br * 8 / 1000) / (float) nsecs, unitstring);
+		fprintf(fd, ", average activity %.2f %s/s",
+			(float) (ptmp->br * 8 / 1000) / (float) nsecs,
+			unitstring);
 	    } else {
-		fprintf(fd, ", average activity %.2f %s/s", (float) (ptmp->br / 1024) / (float) nsecs, unitstring);
+		fprintf(fd, ", average activity %.2f %s/s",
+			(float) (ptmp->br / 1024) / (float) nsecs,
+			unitstring);
 	    }
 
-	    fprintf(fd, ", peak activity %.2f %s/s", ptmp->peakrate, unitstring);
-	    fprintf(fd, ", last 5-second activity %.2f %s/s", ptmp->rate, unitstring);
+	    fprintf(fd, ", peak activity %.2f %s/s", ptmp->peakrate,
+		    unitstring);
+	    fprintf(fd, ", last 5-second activity %.2f %s/s", ptmp->rate,
+		    unitstring);
 	}
 	fprintf(fd, "\n");
 
@@ -193,14 +226,14 @@ void writegstatlog(struct iftab *table, int unit, unsigned long nsecs, FILE * fd
     }
 
     fprintf(fd, "\n%lu seconds running time\n", nsecs);
+    fflush(fd);
 }
 
 void writedstatlog(char *ifname, int unit, float activity, float pps,
 		   float peakactivity, float peakpps,
 		   float peakactivity_in, float peakpps_in,
 		   float peakactivity_out, float peakpps_out,
-		   struct iftotals *ts, unsigned long nsecs,
-		   FILE * fd)
+		   struct iftotals *ts, unsigned long nsecs, FILE * fd)
 {
     char atime[TIME_TARGET_MAX];
     char unitstring[7];
@@ -209,30 +242,52 @@ void writedstatlog(char *ifname, int unit, float activity, float pps,
 
     genatime(time((time_t *) NULL), atime);
 
-    fprintf(fd, "\n*** Detailed statistics for interface %s, generated %s\n\n",
+    fprintf(fd,
+	    "\n*** Detailed statistics for interface %s, generated %s\n\n",
 	    ifname, atime);
 
-    fprintf(fd, "Total: \t%llu packets, %llu bytes\n", ts->total, ts->bytestotal);
-    fprintf(fd, "\t(incoming: %llu packets, %llu bytes; outgoing: %llu packets, %llu bytes)\n",
-     ts->total_in, ts->bytestotal_in, ts->total_out, ts->bytestotal_out);
-    fprintf(fd, "IP: \t%llu packets, %llu bytes\n", ts->iptotal, ts->ipbtotal);
-    fprintf(fd, "\t(incoming: %llu packets, %llu bytes; outgoing: %llu packets, %llu bytes)\n",
-     ts->iptotal_in, ts->ipbtotal_in, ts->iptotal_out, ts->ipbtotal_out);
-    fprintf(fd, "TCP: %llu packets, %llu bytes\n", ts->tcptotal, ts->tcpbtotal);
-    fprintf(fd, "\t(incoming: %llu packets, %llu bytes; outgoing: %llu packets, %llu bytes)\n",
-	    ts->tcptotal_in, ts->tcpbtotal_in, ts->tcptotal_out, ts->tcpbtotal_out);
-    fprintf(fd, "UDP: %llu packets, %llu bytes\n", ts->udptotal, ts->udpbtotal);
-    fprintf(fd, "\t(incoming: %llu packets, %llu bytes; outgoing: %llu packets, %llu bytes)\n",
-	    ts->udptotal_in, ts->udpbtotal_in, ts->udptotal_out, ts->udpbtotal_out);
-    fprintf(fd, "ICMP: %llu packets, %llu bytes\n", ts->icmptotal, ts->icmpbtotal);
-    fprintf(fd, "\t(incoming: %llu packets, %llu bytes; outgoing: %llu packets, %llu bytes)\n",
-	    ts->icmptotal_in, ts->icmpbtotal_in, ts->icmptotal_out, ts->icmpbtotal_out);
-    fprintf(fd, "Other IP: %llu packets, %llu bytes\n", ts->othtotal, ts->othbtotal);
-    fprintf(fd, "\t(incoming: %llu packets, %llu bytes; outgoing: %llu packets, %llu bytes)\n",
-	    ts->othtotal_in, ts->othbtotal_in, ts->othtotal_out, ts->othbtotal_out);
-    fprintf(fd, "Non-IP: %llu packets, %llu bytes\n", ts->noniptotal, ts->nonipbtotal);
-    fprintf(fd, "\t(incoming: %llu packets, %llu bytes; outgoing: %llu packets, %llu bytes)\n",
-	    ts->noniptotal_in, ts->nonipbtotal_in, ts->noniptotal_out, ts->nonipbtotal_out);
+    fprintf(fd, "Total: \t%llu packets, %llu bytes\n", ts->total,
+	    ts->bytestotal);
+    fprintf(fd,
+	    "\t(incoming: %llu packets, %llu bytes; outgoing: %llu packets, %llu bytes)\n",
+	    ts->total_in, ts->bytestotal_in, ts->total_out,
+	    ts->bytestotal_out);
+    fprintf(fd, "IP: \t%llu packets, %llu bytes\n", ts->iptotal,
+	    ts->ipbtotal);
+    fprintf(fd,
+	    "\t(incoming: %llu packets, %llu bytes; outgoing: %llu packets, %llu bytes)\n",
+	    ts->iptotal_in, ts->ipbtotal_in, ts->iptotal_out,
+	    ts->ipbtotal_out);
+    fprintf(fd, "TCP: %llu packets, %llu bytes\n", ts->tcptotal,
+	    ts->tcpbtotal);
+    fprintf(fd,
+	    "\t(incoming: %llu packets, %llu bytes; outgoing: %llu packets, %llu bytes)\n",
+	    ts->tcptotal_in, ts->tcpbtotal_in, ts->tcptotal_out,
+	    ts->tcpbtotal_out);
+    fprintf(fd, "UDP: %llu packets, %llu bytes\n", ts->udptotal,
+	    ts->udpbtotal);
+    fprintf(fd,
+	    "\t(incoming: %llu packets, %llu bytes; outgoing: %llu packets, %llu bytes)\n",
+	    ts->udptotal_in, ts->udpbtotal_in, ts->udptotal_out,
+	    ts->udpbtotal_out);
+    fprintf(fd, "ICMP: %llu packets, %llu bytes\n", ts->icmptotal,
+	    ts->icmpbtotal);
+    fprintf(fd,
+	    "\t(incoming: %llu packets, %llu bytes; outgoing: %llu packets, %llu bytes)\n",
+	    ts->icmptotal_in, ts->icmpbtotal_in, ts->icmptotal_out,
+	    ts->icmpbtotal_out);
+    fprintf(fd, "Other IP: %llu packets, %llu bytes\n", ts->othtotal,
+	    ts->othbtotal);
+    fprintf(fd,
+	    "\t(incoming: %llu packets, %llu bytes; outgoing: %llu packets, %llu bytes)\n",
+	    ts->othtotal_in, ts->othbtotal_in, ts->othtotal_out,
+	    ts->othbtotal_out);
+    fprintf(fd, "Non-IP: %llu packets, %llu bytes\n", ts->noniptotal,
+	    ts->nonipbtotal);
+    fprintf(fd,
+	    "\t(incoming: %llu packets, %llu bytes; outgoing: %llu packets, %llu bytes)\n",
+	    ts->noniptotal_in, ts->nonipbtotal_in, ts->noniptotal_out,
+	    ts->nonipbtotal_out);
 
     if (nsecs > 5) {
 	fprintf(fd, "\nAverage rates:\n");
@@ -242,10 +297,12 @@ void writedstatlog(char *ifname, int unit, float activity, float pps,
 		    ((float) (ts->bytestotal * 8 / 1000) / (float) nsecs),
 		    ((float) (ts->total) / (float) nsecs));
 	    fprintf(fd, "  Incoming:\t%.2f kbits/s, %.2f packets/s\n",
-		(float) (ts->bytestotal_in * 8 / 1000) / (float) (nsecs),
+		    (float) (ts->bytestotal_in * 8 / 1000) /
+		    (float) (nsecs),
 		    (float) (ts->total_in) / (float) (nsecs));
 	    fprintf(fd, "  Outgoing:\t%.2f kbits/s, %.2f packets/s\n",
-	       (float) (ts->bytestotal_out * 8 / 1000) / (float) (nsecs),
+		    (float) (ts->bytestotal_out * 8 / 1000) /
+		    (float) (nsecs),
 		    (float) (ts->total_out) / (float) (nsecs));
 	} else {
 	    fprintf(fd, "%.2f kbytes/s, %.2f packets/s\n",
@@ -264,10 +321,11 @@ void writedstatlog(char *ifname, int unit, float activity, float pps,
 	fprintf(fd, "Peak incoming rate: %.2f %s/s, %.2f packets/s\n",
 		peakactivity_in, unitstring, peakpps_in);
 	fprintf(fd, "Peak outgoing rate: %.2f %s/s, %.2f packets/s\n\n",
-		peakactivity_in, unitstring, peakpps_in);
+		peakactivity_out, unitstring, peakpps_out);
     }
     fprintf(fd, "IP checksum errors: %lu\n\n", ts->badtotal);
     fprintf(fd, "Running time: %lu seconds\n", nsecs);
+    fflush(fd);
 }
 
 void writeutslog(struct portlistent *list, unsigned long nsecs, FILE * fd)
@@ -285,17 +343,22 @@ void writeutslog(struct portlistent *list, unsigned long nsecs, FILE * fd)
 	else
 	    fprintf(fd, "UDP/%s: ", ptmp->servname);
 
-	fprintf(fd, "%llu packets, %llu bytes total; ", ptmp->count, ptmp->bcount);
-	fprintf(fd, "%llu packets, %llu bytes incoming; ", ptmp->icount, ptmp->ibcount);
-	fprintf(fd, "%llu packets, %llu bytes outgoing\n\n", ptmp->ocount, ptmp->obcount);
+	fprintf(fd, "%llu packets, %llu bytes total; ", ptmp->count,
+		ptmp->bcount);
+	fprintf(fd, "%llu packets, %llu bytes incoming; ", ptmp->icount,
+		ptmp->ibcount);
+	fprintf(fd, "%llu packets, %llu bytes outgoing\n\n", ptmp->ocount,
+		ptmp->obcount);
 
 	ptmp = ptmp->next_entry;
     }
 
     fprintf(fd, "\nRunning time: %lu seconds\n", nsecs);
+    fflush(fd);
 }
 
-void writeethlog(struct ethtabent *list, int unit, unsigned long nsecs, FILE * fd)
+void writeethlog(struct ethtabent *list, int unit, unsigned long nsecs,
+		 FILE * fd)
 {
     char atime[TIME_TARGET_MAX];
     struct ethtabent *ptmp = list;
@@ -310,7 +373,8 @@ void writeethlog(struct ethtabent *list, int unit, unsigned long nsecs, FILE * f
     while (ptmp != NULL) {
 	if (ptmp->type == 0) {
 	    if (ptmp->un.desc.linktype == LINK_ETHERNET)
-		fprintf(fd, "\nEthernet address: %s", ptmp->un.desc.ascaddr);
+		fprintf(fd, "\nEthernet address: %s",
+			ptmp->un.desc.ascaddr);
 	    else if (ptmp->un.desc.linktype == LINK_PLIP)
 		fprintf(fd, "\nPLIP address: %s", ptmp->un.desc.ascaddr);
 	    else if (ptmp->un.desc.linktype == LINK_FDDI)
@@ -321,23 +385,34 @@ void writeethlog(struct ethtabent *list, int unit, unsigned long nsecs, FILE * f
 
 	    fprintf(fd, "\n");
 	} else {
-	    fprintf(fd, "\tIncoming total %llu packets, %llu bytes; %llu IP packets\n",
-		    ptmp->un.figs.inpcount, ptmp->un.figs.inbcount, ptmp->un.figs.inippcount);
-	    fprintf(fd, "\tOutgoing total %llu packets, %llu bytes; %llu IP packets\n",
-		    ptmp->un.figs.outpcount, ptmp->un.figs.outbcount, ptmp->un.figs.outippcount);
+	    fprintf(fd,
+		    "\tIncoming total %llu packets, %llu bytes; %llu IP packets\n",
+		    ptmp->un.figs.inpcount, ptmp->un.figs.inbcount,
+		    ptmp->un.figs.inippcount);
+	    fprintf(fd,
+		    "\tOutgoing total %llu packets, %llu bytes; %llu IP packets\n",
+		    ptmp->un.figs.outpcount, ptmp->un.figs.outbcount,
+		    ptmp->un.figs.outippcount);
 
 	    fprintf(fd, "\tAverage rates: ");
 	    if (unit == KBITS)
-		fprintf(fd, "%.2f kbits/s incoming, %.2f kbits/s outgoing\n",
-			(float) (ptmp->un.figs.inbcount * 8 / 1000) / (float) nsecs,
-			(float) (ptmp->un.figs.outbcount * 8 / 1000) / (float) nsecs);
+		fprintf(fd,
+			"%.2f kbits/s incoming, %.2f kbits/s outgoing\n",
+			(float) (ptmp->un.figs.inbcount * 8 / 1000) /
+			(float) nsecs,
+			(float) (ptmp->un.figs.outbcount * 8 / 1000) /
+			(float) nsecs);
 	    else
-		fprintf(fd, "%.2f kbytes/s incoming, %.2f kbytes/s outgoing\n",
-		 (float) (ptmp->un.figs.inbcount / 1024) / (float) nsecs,
-		(float) (ptmp->un.figs.outbcount / 1024) / (float) nsecs);
+		fprintf(fd,
+			"%.2f kbytes/s incoming, %.2f kbytes/s outgoing\n",
+			(float) (ptmp->un.figs.inbcount / 1024) /
+			(float) nsecs,
+			(float) (ptmp->un.figs.outbcount / 1024) /
+			(float) nsecs);
 
 	    if (nsecs > 5)
-		fprintf(fd, "\tLast 5-second rates: %.2f %s/s incoming, %.2f %s/s outgoing\n",
+		fprintf(fd,
+			"\tLast 5-second rates: %.2f %s/s incoming, %.2f %s/s outgoing\n",
 			ptmp->un.figs.inrate, unitstring,
 			ptmp->un.figs.outrate, unitstring);
 	}
@@ -346,6 +421,7 @@ void writeethlog(struct ethtabent *list, int unit, unsigned long nsecs, FILE * f
     }
 
     fprintf(fd, "\nRunning time: %lu seconds\n", nsecs);
+    fflush(fd);
 }
 
 void write_size_log(struct ifstat_brackets *brackets, unsigned long nsecs,
@@ -355,15 +431,17 @@ void write_size_log(struct ifstat_brackets *brackets, unsigned long nsecs,
     int i;
 
     genatime(time((time_t *) NULL), atime);
-    fprintf(logfile, "*** Packet Size Distribution, generated %s\n\n", atime);
+    fprintf(logfile, "*** Packet Size Distribution, generated %s\n\n",
+	    atime);
     fprintf(logfile, "Interface: %s   MTU: %u\n\n", ifname, mtu);
     fprintf(logfile, "Packet Size (bytes)\tCount\n");
 
     for (i = 0; i <= 19; i++) {
-	fprintf(logfile, "%u to %u:\t\t%lu\n", brackets[i].floor, brackets[i].ceil,
-		brackets[i].count);
+	fprintf(logfile, "%u to %u:\t\t%lu\n", brackets[i].floor,
+		brackets[i].ceil, brackets[i].count);
     }
     fprintf(logfile, "\nRunning time: %lu seconds\n", nsecs);
+    fflush(logfile);
 }
 
 
@@ -377,7 +455,8 @@ void rotate_logfile(FILE ** fd, char *name)
 
 void announce_rotate_prepare(FILE * fd)
 {
-    writelog(1, fd, "***** USR1 signal received, preparing to reopen log file *****");
+    writelog(1, fd,
+	     "***** USR1 signal received, preparing to reopen log file *****");
 }
 
 void announce_rotate_complete(FILE * fd)
