@@ -32,32 +32,47 @@ details.
 #include "input.h"
 #include "landesc.h"
 #include "error.h"
+#include "promisc.h"
 #include "dirs.h"
+#include "instances.h"
 
 #define ALLOW_ZERO 1
 #define DONT_ALLOW_ZERO 0
 
-extern void srpromisc(int mode);
-
 void makeoptionmenu(struct MENU *menu)
 {
-    initmenu(menu, 20, 37, (LINES - 20) / 2, (COLS - 37) / 4);
-    additem(menu, " ^R^everse DNS lookups", "Toggles resolution of IP addresses into host names");
-    additem(menu, " TCP/UDP ^s^ervice names", "Displays TCP/UDP service names instead of numeric ports");
-    additem(menu, " ^P^romiscuous operation", "Toggles capture of all packets by LAN interfaces");
-    additem(menu, " ^C^olor", "Turns color on or off (restart IPTraf to effect change)");
-    additem(menu, " ^L^ogging", "Toggles logging of traffic to a data file");
-    additem(menu, " Acti^v^ity mode", "Toggles activity indicators between kbits/s and kbytes/s");
+    initmenu(menu, 21, 40, (LINES - 21) / 2, (COLS - 40) / 16);
+    additem(menu, " ^R^everse DNS lookups",
+	    "Toggles resolution of IP addresses into host names");
+    additem(menu, " TCP/UDP ^s^ervice names",
+	    "Displays TCP/UDP service names instead of numeric ports");
+    additem(menu, " Force ^p^romiscuous mode",
+	    "Toggles capture of all packets by LAN interfaces");
+    additem(menu, " ^C^olor",
+	    "Turns color on or off (restart IPTraf to effect change)");
+    additem(menu, " ^L^ogging",
+	    "Toggles logging of traffic to a data file");
+    additem(menu, " Acti^v^ity mode",
+	    "Toggles activity indicators between kbits/s and kbytes/s");
     additem(menu, NULL, NULL);
-    additem(menu, " TCP ^t^imeout...", "Determines how long TCP entries may remain idle");
-    additem(menu, " Logging ^i^nterval...", "Sets the time between loggings for interface, host, and service stats");
-    additem(menu, " Screen ^u^pdate interval...", "Sets the screen update interval in seconds (set to 0 for fastest updates)");
+    additem(menu, " TCP ^t^imeout...",
+	    "Determines how long TCP entries may remain idle");
+    additem(menu, " Logging ^i^nterval...",
+	    "Sets the time between loggings for interface, host, and service stats");
+    additem(menu, " Screen ^u^pdate interval...",
+	    "Sets the screen update interval in seconds (set to 0 for fastest updates)");
+    additem(menu, " TCP cl^o^sed/idle persistence...",
+	    "Determines how long closed/idle/reset entries stay onscreen");
     additem(menu, NULL, NULL);
-    additem(menu, " ^A^dditional ports...", "Allows you to add port numbers higher than 1023 for the service stats");
-    additem(menu, " ^D^elete port/range...", "Deletes a port or range of ports earlier added");
+    additem(menu, " ^A^dditional ports...",
+	    "Allows you to add port numbers higher than 1023 for the service stats");
+    additem(menu, " ^D^elete port/range...",
+	    "Deletes a port or range of ports earlier added");
     additem(menu, NULL, NULL);
-    additem(menu, " ^E^thernet/PLIP host descriptions", "Manages descriptions for Ethernet/PLIP addresses");
-    additem(menu, " ^F^DDI host descriptions", "Manages descriptions for FDDI addresses");
+    additem(menu, " ^E^thernet/PLIP host descriptions",
+	    "Manages descriptions for Ethernet/PLIP addresses");
+    additem(menu, " ^F^DDI host descriptions",
+	    "Manages descriptions for FDDI addresses");
     additem(menu, NULL, NULL);
     additem(menu, " E^x^it configuration", "Returns to main menu");
 }
@@ -72,7 +87,7 @@ void printoptonoff(unsigned int option, WINDOW * win)
 
 void indicatesetting(int row, struct OPTIONS *options, WINDOW * win)
 {
-    wmove(win, row, 25);
+    wmove(win, row, 30);
     wattrset(win, HIGHATTR);
 
     switch (row) {
@@ -92,7 +107,7 @@ void indicatesetting(int row, struct OPTIONS *options, WINDOW * win)
 	printoptonoff(options->logging, win);
 	break;
     case 6:
-	wmove(win, row, 20);
+	wmove(win, row, 25);
 	if (options->actmode == KBITS)
 	    wprintw(win, " kbits/s");
 	else
@@ -131,6 +146,7 @@ void setdefaultopts(struct OPTIONS *options)
     options->timeout = 15;
     options->logspan = 3600;
     options->updrate = 0;
+    options->closedint = 0;
 }
 
 void loadoptions(struct OPTIONS *options)
@@ -152,12 +168,10 @@ void loadoptions(struct OPTIONS *options)
 void updatetimes(struct OPTIONS *options, WINDOW * win)
 {
     wattrset(win, HIGHATTR);
-    wmove(win, 8, 20);
-    wprintw(win, "%3u mins", options->timeout);
-    wmove(win, 9, 20);
-    wprintw(win, "%3u mins", options->logspan / 60);
-    wmove(win, 10, 20);
-    wprintw(win, "%3u secs", options->updrate);
+    mvwprintw(win, 8, 25, "%3u mins", options->timeout);
+    mvwprintw(win, 9, 25, "%3u mins", options->logspan / 60);
+    mvwprintw(win, 10, 25, "%3u secs", options->updrate);
+    mvwprintw(win, 11, 25, "%3u mins", options->closedint);
 }
 
 void showoptions(struct OPTIONS *options, WINDOW * win)
@@ -170,7 +184,8 @@ void showoptions(struct OPTIONS *options, WINDOW * win)
     updatetimes(options, win);
 }
 
-void settimeout(unsigned int *value, const char *units, int allow_zero, int *aborted)
+void settimeout(unsigned int *value, const char *units, int allow_zero,
+		int *aborted)
 {
     WINDOW *dlgwin;
     PANEL *dlgpanel;
@@ -215,7 +230,7 @@ void settimeout(unsigned int *value, const char *units, int allow_zero, int *abo
     doupdate();
 }
 
-void setoptions(struct OPTIONS *options, struct porttab **ports, int is_first_instance)
+void setoptions(struct OPTIONS *options, struct porttab **ports)
 {
     int row = 1;
     int aborted;
@@ -226,40 +241,32 @@ void setoptions(struct OPTIONS *options, struct porttab **ports, int is_first_in
     PANEL *statpanel;
 
     if (!is_first_instance) {
-	errbox("Only the first instance of IPTraf can configure", ANYKEY_MSG, &resp);
+	errbox("Only the first instance of IPTraf can configure",
+	       ANYKEY_MSG, &resp);
 	return;
     }
     makeoptionmenu(&menu);
 
-    statwin = newwin(12, 30, (LINES - 20) / 2, (COLS - 37) / 4 + 37);
+    statwin = newwin(13, 35, (LINES - 20) / 2, (COLS - 40) / 16 + 40);
     statpanel = new_panel(statwin);
 
     wattrset(statwin, BOXATTR);
     colorwin(statwin);
     box(statwin, ACS_VLINE, ACS_HLINE);
     wmove(statwin, 7, 1);
-    whline(statwin, ACS_HLINE, 28);
-    wmove(statwin, 0, 1);
-    wprintw(statwin, " Current Settings ");
+    whline(statwin, ACS_HLINE, 33);
+    mvwprintw(statwin, 0, 1, " Current Settings ");
     wattrset(statwin, STDATTR);
-    wmove(statwin, 1, 2);
-    wprintw(statwin, "Reverse DNS lookups:");
-    wmove(statwin, 2, 2);
-    wprintw(statwin, "Service names: ");
-    wmove(statwin, 3, 2);
-    wprintw(statwin, "Promiscuous: ");
-    wmove(statwin, 4, 2);
-    wprintw(statwin, "Color: ");
-    wmove(statwin, 5, 2);
-    wprintw(statwin, "Logging: ");
-    wmove(statwin, 6, 2);
-    wprintw(statwin, "Activity mode: ");
-    wmove(statwin, 8, 2);
-    wprintw(statwin, "TCP timeout:");
-    wmove(statwin, 9, 2);
-    wprintw(statwin, "Log interval:");
-    wmove(statwin, 10, 2);
-    wprintw(statwin, "Update interval:");
+    mvwprintw(statwin, 1, 2, "Reverse DNS lookups:");
+    mvwprintw(statwin, 2, 2, "Service names: ");
+    mvwprintw(statwin, 3, 2, "Promiscuous: ");
+    mvwprintw(statwin, 4, 2, "Color: ");
+    mvwprintw(statwin, 5, 2, "Logging: ");
+    mvwprintw(statwin, 6, 2, "Activity mode: ");
+    mvwprintw(statwin, 8, 2, "TCP timeout:");
+    mvwprintw(statwin, 9, 2, "Log interval:");
+    mvwprintw(statwin, 10, 2, "Update interval:");
+    mvwprintw(statwin, 11, 2, "Closed/idle persist:");
     showoptions(options, statwin);
 
     do {
@@ -275,7 +282,6 @@ void setoptions(struct OPTIONS *options, struct porttab **ports, int is_first_in
 	    break;
 	case 3:
 	    options->promisc = ~(options->promisc);
-	    srpromisc(options->promisc);
 	    break;
 	case 4:
 	    options->color = ~(options->color);
@@ -287,12 +293,14 @@ void setoptions(struct OPTIONS *options, struct porttab **ports, int is_first_in
 	    options->actmode = ~(options->actmode);
 	    break;
 	case 8:
-	    settimeout(&(options->timeout), "minutes", DONT_ALLOW_ZERO, &aborted);
+	    settimeout(&(options->timeout), "minutes", DONT_ALLOW_ZERO,
+		       &aborted);
 	    if (!aborted)
 		updatetimes(options, statwin);
 	    break;
 	case 9:
-	    settimeout((unsigned int *) &(options->logspan), "minutes", DONT_ALLOW_ZERO, &aborted);
+	    settimeout((unsigned int *) &(options->logspan), "minutes",
+		       DONT_ALLOW_ZERO, &aborted);
 	    if (!aborted) {
 		options->logspan = options->logspan * 60;
 		updatetimes(options, statwin);
@@ -303,22 +311,28 @@ void setoptions(struct OPTIONS *options, struct porttab **ports, int is_first_in
 	    if (!aborted)
 		updatetimes(options, statwin);
 	    break;
-	case 12:
-	    addmoreports(ports);
+	case 11:
+	    settimeout(&options->closedint, "minutes", ALLOW_ZERO,
+		       &aborted);
+	    if (!aborted)
+		updatetimes(options, statwin);
 	    break;
 	case 13:
+	    addmoreports(ports);
+	    break;
+	case 14:
 	    removeaport(ports);
 	    break;
-	case 15:
+	case 16:
 	    ethdescmgr(LINK_ETHERNET);
 	    break;
-	case 16:
+	case 17:
 	    ethdescmgr(LINK_FDDI);
 	    break;
 	}
 
 	indicatesetting(row, options, statwin);
-    } while (row != 18);
+    } while (row != 19);
 
     destroymenu(&menu);
     del_panel(statpanel);
