@@ -41,13 +41,15 @@ details.
 
 void makeothfiltermenu(struct MENU *menu)
 {
-    initmenu(menu, 12, 15, (LINES - 11) / 2, (COLS - 30) / 2 - 5);
+    initmenu(menu, 14, 15, (LINES - 14) / 2, (COLS - 30) / 2 - 5);
     additem(menu, " ^U^DP...", "User Datagram Protocol");
     additem(menu, " ^I^CMP", "Internet Control Message Protocol");
     additem(menu, " ^O^SPF", "Open Shortest Path First");
     additem(menu, " I^G^P", "Interior Gateway Protocol");
     additem(menu, " IG^M^P", "Internet Group Management Protocol");
     additem(menu, " IG^R^P", "Interior Gateway Routing Protocol");
+    additem(menu, " GR^E^ ", "General Routing Encapsulation");
+    additem(menu, " ^O^ther IP", "Other IP protocols");
     additem(menu, " ^A^RP", "Address Resolution Protocol");
     additem(menu, " RAR^P^ ", "Reverse Address Resolution Protocol");
     additem(menu, " ^N^on-IP", "Non-IP packets");
@@ -75,7 +77,7 @@ void setothfilter(struct othpoptions *filter, unsigned int row)
     case 3:
 	filter->ospf = ~(filter->ospf);
 	break;
-    case 4:
+    case 4: 
 	filter->igp = ~(filter->igp);
 	break;
     case 5:
@@ -85,12 +87,18 @@ void setothfilter(struct othpoptions *filter, unsigned int row)
 	filter->igrp = ~(filter->igrp);
 	break;
     case 7:
+        filter->gre = ~(filter->gre);
+        break;
+    case 8:
+        filter->unknip = ~(filter->unknip);
+        break;
+    case 9:
 	filter->arp = ~(filter->arp);
 	break;
-    case 8:
+    case 10:
 	filter->rarp = ~(filter->rarp);
 	break;
-    case 9:
+    case 11:
 	filter->nonip = ~(filter->nonip);
 	break;
     }
@@ -106,7 +114,7 @@ void toggleprotodisplay(WINDOW * win, struct othpoptions *filter,
 {
     wmove(win, row, 2);
     switch (row) {
-    case 1:
+    case 1:   
 	if (filter->udp)
 	    wprintw(win, "UDP");
 	else
@@ -149,20 +157,34 @@ void toggleprotodisplay(WINDOW * win, struct othpoptions *filter,
 
 	break;
     case 7:
+        if (filter->gre)
+            wprintw(win, "GRE");
+        else
+            blankoutentry(win);
+            
+        break;
+    case 8:
+        if (filter->unknip)
+            wprintw(win, "Other IP");
+        else
+            blankoutentry(win);
+        
+        break;
+    case 9:
 	if (filter->arp)
 	    wprintw(win, "ARP");
 	else
 	    blankoutentry(win);
 
 	break;
-    case 8:
+    case 10:
 	if (filter->rarp)
 	    wprintw(win, "RARP");
 	else
 	    blankoutentry(win);
 
 	break;
-    case 9:
+    case 11:
 	if (filter->nonip)
 	    wprintw(win, "Non-IP");
 	else
@@ -185,8 +207,10 @@ int othfilterok(struct othpoptions *filter, unsigned int protocol,
 	else if (filter->filtercode == 1)
 	    result = 1;
 	else if (filter->filtercode == 2)
-	    if (utfilter(&(filter->fl), saddr, daddr, sport, dport))
-		result = 1;
+	    if (utfilter
+		(&(filter->fl), saddr, daddr, sport, dport,
+		 IPPROTO_UDP))
+	        result = 1;
 	break;
     case IPPROTO_ICMP:
 	result = filter->icmp;
@@ -203,6 +227,9 @@ int othfilterok(struct othpoptions *filter, unsigned int protocol,
     case IPPROTO_IGRP:
 	result = filter->igrp;
 	break;
+    case IPPROTO_GRE:
+        result = filter->gre;
+        break;
     case ETH_P_ARP:
 	result = filter->arp;
 	break;
@@ -212,6 +239,8 @@ int othfilterok(struct othpoptions *filter, unsigned int protocol,
     case 0:
 	result = filter->nonip;
 	break;
+    default:
+        result = filter->unknip;
     }
 
     return result;
@@ -225,7 +254,7 @@ void othfilterselect(struct othpoptions *filter)
     int row;
     int aborted;
 
-    statwin = newwin(11, 23, (LINES - 11) / 2, (COLS - 30) / 2 + 10);
+    statwin = newwin(13, 23, (LINES - 14) / 2, (COLS - 30) / 2 + 10);
     statpanel = new_panel(statwin);
     wattrset(statwin, BOXATTR);
     colorwin(statwin);
@@ -250,7 +279,7 @@ void othfilterselect(struct othpoptions *filter)
 	    setothfilter(filter, row);
 	    toggleprotodisplay(statwin, filter, row);
 	}
-    } while (row != 10);
+    } while (row != 12);
 
     destroymenu(&menu);
     del_panel(statpanel);
@@ -263,7 +292,8 @@ void setodefaults(struct othpoptions *filter)
 {
     bzero(filter, sizeof(struct othpoptions));
     filter->filtercode = 1;
-    filter->udp = filter->icmp = filter->ospf = filter->nonip = (u_int16_t) ~ 0;
+    filter->udp = filter->icmp = filter->ospf = filter->nonip =
+	(u_int16_t) ~ 0;
     filter->arp = filter->rarp = (u_int16_t) ~ 0;
 }
 
