@@ -34,9 +34,9 @@ details.
 #define SHORTSTRING_MAX	40
 
 void convmacaddr(char *addr, char *result);	/* external; from hostmon.c */
-void writeothplog(int logging, FILE * fd, char *protname, char *description,
-		  char *additional, int is_ip, int withmac,
-		  struct othptabent *entry);
+void writeothplog(int logging, FILE * fd, char *protname,
+		  char *description, char *additional, int is_ip,
+		  int withmac, struct othptabent *entry);
 
 void init_othp_table(struct othptable *table)
 {
@@ -71,8 +71,8 @@ void init_othp_table(struct othptable *table)
     table->oimaxy = table->obmaxy - 2;
 }
 
-void process_dest_unreach(struct tcptable *table, char *packet, char *ifname,
-			  int *nomem)
+void process_dest_unreach(struct tcptable *table, char *packet,
+			  char *ifname, int *nomem)
 {
     struct iphdr *ip;
     struct tcphdr *tcp;
@@ -101,13 +101,15 @@ void process_dest_unreach(struct tcptable *table, char *packet, char *ifname,
 }
 
 struct othptabent *add_othp_entry(struct othptable *table,
-			    struct tcptable *tcptab, unsigned long saddr,
-			    unsigned long daddr, int is_ip, int protocol,
-		   unsigned short linkproto, char *packet, char *packet2,
-				  unsigned int br,
-		char *ifname, int rev_lookup, int rvnfd, unsigned int tm,
-		int logging, FILE * logfile, int servnames, int fragment,
-				  int *nomem)
+				  struct tcptable *tcptab,
+				  unsigned long saddr, unsigned long daddr,
+				  int is_ip, int protocol,
+				  unsigned short linkproto, char *packet,
+				  char *packet2, unsigned int br,
+				  char *ifname, int rev_lookup, int rvnfd,
+				  unsigned int tm, int logging,
+				  FILE * logfile, int servnames,
+				  int fragment, int *nomem)
 {
     struct othptabent *new_entry;
     struct othptabent *temp;
@@ -134,37 +136,52 @@ struct othptabent *add_othp_entry(struct othptable *table,
 
 	if (!fragment) {
 	    if (protocol == IPPROTO_ICMP) {
-		new_entry->un.icmp.type = ((struct icmphdr *) packet)->type;
-		new_entry->un.icmp.code = ((struct icmphdr *) packet)->code;
+		new_entry->un.icmp.type =
+		    ((struct icmphdr *) packet)->type;
+		new_entry->un.icmp.code =
+		    ((struct icmphdr *) packet)->code;
 	    } else if (protocol == IPPROTO_UDP) {
 		servlook(servnames, ((struct udphdr *) packet)->source,
 			 IPPROTO_UDP, new_entry->un.udp.s_sname, 10);
 		servlook(servnames, ((struct udphdr *) packet)->dest,
 			 IPPROTO_UDP, new_entry->un.udp.d_sname, 10);
 	    } else if (protocol == IPPROTO_OSPFIGP) {
-		new_entry->un.ospf.type = ((struct ospfhdr *) packet)->ospf_type;
-		new_entry->un.ospf.area = ntohl(((struct ospfhdr *) packet)->ospf_areaid.s_addr);
-		strcpy(new_entry->un.ospf.routerid, inet_ntoa(((struct ospfhdr *) packet)->ospf_routerid));
+		new_entry->un.ospf.type =
+		    ((struct ospfhdr *) packet)->ospf_type;
+		new_entry->un.ospf.area =
+		    ntohl(((struct ospfhdr *) packet)->ospf_areaid.s_addr);
+		strcpy(new_entry->un.ospf.routerid,
+		       inet_ntoa(
+				 ((struct ospfhdr *)
+				  packet)->ospf_routerid));
 	    }
 	}
     } else {
 	new_entry->linkproto = linkproto;
 
 	if ((linkproto == LINK_ETHERNET) || (linkproto == LINK_PLIP)) {
-	    convmacaddr(((struct ethhdr *) packet)->h_source, new_entry->smacaddr);
-	    convmacaddr(((struct ethhdr *) packet)->h_dest, new_entry->dmacaddr);
+	    convmacaddr(((struct ethhdr *) packet)->h_source,
+			new_entry->smacaddr);
+	    convmacaddr(((struct ethhdr *) packet)->h_dest,
+			new_entry->dmacaddr);
 	} else if (linkproto == LINK_FDDI) {
-	    convmacaddr(((struct fddihdr *) packet)->saddr, new_entry->smacaddr);
-	    convmacaddr(((struct fddihdr *) packet)->daddr, new_entry->dmacaddr);
+	    convmacaddr(((struct fddihdr *) packet)->saddr,
+			new_entry->smacaddr);
+	    convmacaddr(((struct fddihdr *) packet)->daddr,
+			new_entry->dmacaddr);
 	}
 	if (protocol == ETH_P_ARP) {
 	    new_entry->un.arp.opcode = ((struct arp_hdr *) packet2)->ar_op;
-	    memcpy(&(new_entry->un.arp.src_ip_address), &(((struct arp_hdr *) packet2)->ar_sip), 4);
-	    memcpy(&(new_entry->un.arp.dest_ip_address), &(((struct arp_hdr *) packet2)->ar_tip), 4);
+	    memcpy(&(new_entry->un.arp.src_ip_address),
+		   &(((struct arp_hdr *) packet2)->ar_sip), 4);
+	    memcpy(&(new_entry->un.arp.dest_ip_address),
+		   &(((struct arp_hdr *) packet2)->ar_tip), 4);
 	} else if (protocol == ETH_P_RARP) {
 	    new_entry->un.rarp.opcode = ((struct arphdr *) packet2)->ar_op;
-	    memcpy(&(new_entry->un.rarp.src_mac_address), &(((struct arp_hdr *) packet2)->ar_sha), 6);
-	    memcpy(&(new_entry->un.rarp.dest_mac_address), &(((struct arp_hdr *) packet2)->ar_tha), 6);
+	    memcpy(&(new_entry->un.rarp.src_mac_address),
+		   &(((struct arp_hdr *) packet2)->ar_sha), 6);
+	    memcpy(&(new_entry->un.rarp.dest_mac_address),
+		   &(((struct arp_hdr *) packet2)->ar_tha), 6);
 	}
     }
 
@@ -186,7 +203,8 @@ struct othptabent *add_othp_entry(struct othptable *table,
     if (table->count == 512) {
 	if (table->firstvisible == table->head) {
 	    wscrl(table->othpwin, 1);
-	    printothpentry(table, table->lastvisible->next_entry, table->oimaxy - 1, logging, logfile);
+	    printothpentry(table, table->lastvisible->next_entry,
+			   table->oimaxy - 1, logging, logfile);
 	    table->firstvisible = table->firstvisible->next_entry;
 	    table->lastvisible = table->lastvisible->next_entry;
 	}
@@ -209,12 +227,14 @@ struct othptabent *add_othp_entry(struct othptable *table,
 
     if (table->count <= table->oimaxy) {
 	table->lastvisible = new_entry;
-	printothpentry(table, new_entry, table->count - 1, logging, logfile);
+	printothpentry(table, new_entry, table->count - 1, logging,
+		       logfile);
     } else if (table->lastvisible == table->tail->prev_entry) {
 	wscrl(table->othpwin, 1);
 	table->firstvisible = table->firstvisible->next_entry;
 	table->lastvisible = table->tail;
-	printothpentry(table, new_entry, table->oimaxy - 1, logging, logfile);
+	printothpentry(table, new_entry, table->oimaxy - 1, logging,
+		       logfile);
     }
     return new_entry;
 }
@@ -227,8 +247,7 @@ struct othptabent *add_othp_entry(struct othptable *table,
 char *packetlookup(unsigned int protocol)
 {
     unsigned int i = 0;
-    static struct packetstruct packettypes[] =
-    {
+    static struct packetstruct packettypes[] = {
 	{"DEC MOP dump/load", 0x6001},
 	{"DEC MOP remote console", 0x6002},
 	{"DEC DECnet Phase IV", 0x6003},
@@ -276,7 +295,8 @@ void printothpentry(struct othptable *table, struct othptabent *entry,
 	(table->count >= table->oimaxy)) {
 	wprintw(table->borderwin, " Bottom ");
 	table->htstat = TIND;
-    } else if ((table->firstvisible == table->head) && (table->htstat != HIND)) {
+    } else if ((table->firstvisible == table->head)
+	       && (table->htstat != HIND)) {
 	wprintw(table->borderwin, " Top ");
 	table->htstat = HIND;
     }
@@ -312,7 +332,8 @@ void printothpentry(struct othptable *table, struct othptabent *entry,
 	    switch (ntohs(entry->un.rarp.opcode)) {
 	    case ARPOP_RREQUEST:
 		strcat(msgstring, "request for ");
-		convmacaddr(entry->un.rarp.dest_mac_address, rarp_mac_addr);
+		convmacaddr(entry->un.rarp.dest_mac_address,
+			    rarp_mac_addr);
 		break;
 	    case ARPOP_RREPLY:
 		strcat(msgstring, "reply from ");
@@ -349,7 +370,8 @@ void printothpentry(struct othptable *table, struct othptabent *entry,
 	}
 	startstr = msgstring + table->strindex;
 	waddnstr(table->othpwin, startstr, COLS - 4);
-	writeothplog(logging, logfile, protname, "", "", 0, withmac, entry);
+	writeothplog(logging, logfile, protname, "", "", 0, withmac,
+		     entry);
 	return;
     }
     strcpy(additional, "");
@@ -380,9 +402,13 @@ void printothpentry(struct othptable *table, struct othptabent *entry,
 	wattrset(table->othpwin, IGRPATTR);
 	strcpy(protname, "IGRP");
 	break;
+    case IPPROTO_GRE:  
+        wattrset(table->othpwin, GREATTR);
+        strcpy(protname, "GRE");
+        break;
     default:
 	wattrset(table->othpwin, UNKNATTR);
-	sprintf(protname, "Protocol %u", entry->protocol);
+	sprintf(protname, "IP protocol");
 	unknown = 1;
     }
 
@@ -517,7 +543,6 @@ void printothpentry(struct othptable *table, struct othptabent *entry,
 	strcat(msgstring, scratchpad);
     }
     if (unknown) {
-	strcat(msgstring, "protocol code: ");
 	sprintf(scratchpad, "%u ", entry->protocol);
 	strcat(msgstring, scratchpad);
     }
@@ -529,7 +554,8 @@ void printothpentry(struct othptable *table, struct othptabent *entry,
 		entry->s_fqdn, entry->un.udp.s_sname,
 		entry->d_fqdn, entry->un.udp.d_sname);
     } else {
-	sprintf(scratchpad, "from %.25s to %.25s", entry->s_fqdn, entry->d_fqdn);
+	sprintf(scratchpad, "from %.25s to %.25s", entry->s_fqdn,
+		entry->d_fqdn);
     }
 
     strcat(msgstring, scratchpad);
@@ -545,7 +571,8 @@ void printothpentry(struct othptable *table, struct othptabent *entry,
     waddnstr(table->othpwin, startstr, COLS - 4);
 
     if (logging)
-	writeothplog(logging, logfile, protname, description, additional, 1, 0, entry);
+	writeothplog(logging, logfile, protname, description, additional,
+		     1, 0, entry);
 }
 
 void refresh_othwindow(struct othptable *table)
