@@ -4,7 +4,7 @@ packet.c - routines to open the raw socket, read socket data and
            adjust the initial packet pointer
            
 Written by Gerard Paul Java
-Copyright (c) Gerard Paul Java 1997, 1998
+Copyright (c) Gerard Paul Java 1997-2000
 
 This software is open source; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -57,8 +57,8 @@ void open_socket(int *fd)
     }
 }
 
-unsigned short getlinktype(unsigned short family, char *ifname, int isdn_fd,
-			   struct isdntab *isdnlist)
+unsigned short getlinktype(unsigned short family, char *ifname,
+			   int isdn_fd, struct isdntab *isdnlist)
 {
     unsigned short result = 0;
     struct isdntabent *isdnent;
@@ -69,6 +69,10 @@ unsigned short getlinktype(unsigned short family, char *ifname, int isdn_fd,
 	    result = LINK_ETHERNET;
 	else if (strncmp(ifname, "plip", 4) == 0)
 	    result = LINK_PLIP;
+	else if (strncmp(ifname, "fddi", 4) == 0)	/* For some Ethernet */
+	    result = LINK_ETHERNET;	/* emulated FDDI ifaces */
+	else if (strncmp(ifname, "dvb", 3) == 0)
+	    result = LINK_ETHERNET;
 	else if ((strncmp(ifname, "isdn", 4) == 0) && (isdn_fd != -1)) {
 	    isdnent = isdn_table_lookup(isdnlist, ifname, isdn_fd);
 
