@@ -43,6 +43,7 @@ details.
 #include "instances.h"
 #include "log.h"
 #include "logvars.h"
+#include "promisc.h"
 
 extern int exitloop;
 extern int daemonized;
@@ -59,7 +60,8 @@ void rotate_size_log()
 }
 
 int initialize_brackets(char *ifname, struct ifstat_brackets *brackets,
-		 unsigned int *interval, unsigned int *mtu, WINDOW * win)
+			unsigned int *interval, unsigned int *mtu,
+			WINDOW * win)
 {
     struct ifreq ifr;
     int fd;
@@ -71,7 +73,8 @@ int initialize_brackets(char *ifname, struct ifstat_brackets *brackets,
     fd = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
 
     if (fd < 0) {
-	write_error("Unable to open socket for MTU determination", daemonized);
+	write_error("Unable to open socket for MTU determination",
+		    daemonized);
 	return 1;
     }
     istat = ioctl(fd, SIOCGIFMTU, &ifr);
@@ -116,7 +119,8 @@ int initialize_brackets(char *ifname, struct ifstat_brackets *brackets,
     return 0;
 }
 
-void update_size_distrib(unsigned int length, struct ifstat_brackets *brackets,
+void update_size_distrib(unsigned int length,
+			 struct ifstat_brackets *brackets,
 			 unsigned int interval, WINDOW * win)
 {
     unsigned int i;
@@ -139,7 +143,8 @@ void update_size_distrib(unsigned int length, struct ifstat_brackets *brackets,
     wprintw(win, "%8lu", brackets[i].count);
 }
 
-void packet_size_breakdown(struct OPTIONS *options, char *ifname, int facilitytime)
+void packet_size_breakdown(struct OPTIONS *options, char *ifname,
+			   int facilitytime)
 {
     WINDOW *win;
     PANEL *panel;
@@ -170,21 +175,25 @@ void packet_size_breakdown(struct OPTIONS *options, char *ifname, int facilityti
     int logging = options->logging;
     FILE *logfile = NULL;
 
+    struct promisc_states *promisc_list;
+
     if (!facility_active(PKTSIZEIDFILE))
 	mark_facility(PKTSIZEIDFILE, "Packet size breakdown");
     else {
-	write_error("Packet size breakdown already active in another process", daemonized);
+	write_error
+	    ("Packet size breakdown already active in another process",
+	     daemonized);
 	return;
     }
 
     if (!iface_supported(ifname)) {
 	err_iface_unsupported();
-	unlink(PKTSIZEIDFILE);
+	unmark_facility(PKTSIZEIDFILE);
 	return;
     }
     if (!iface_up(ifname)) {
 	err_iface_down();
-	unlink(PKTSIZEIDFILE);
+	unmark_facility(PKTSIZEIDFILE);
 	return;
     }
     borderwin = newwin(LINES - 2, COLS, 1, 0);
@@ -228,7 +237,8 @@ void packet_size_breakdown(struct OPTIONS *options, char *ifname, int facilityti
     if (logging)
 	signal(SIGUSR1, rotate_size_log);
 
-    writelog(logging, logfile, "******** Packet size distribution facility started");
+    writelog(logging, logfile,
+	     "******** Packet size distribution facility started ********");
 
     exitloop = 0;
     gettimeofday(&tv, NULL);
@@ -236,6 +246,20 @@ void packet_size_breakdown(struct OPTIONS *options, char *ifname, int facilityti
 
     open_socket(&fd);
 
+    if (fd < 0) {
+	unmark_facility(PKTSIZEIDFILE);
+	return;
+    }
+
+    if ((first_active_facility()) && (options->promisc)) {
+	init_promisc_list(&promisc_list);
+	save_promisc_list(promisc_list);
+	srpromisc(1, promisc_list);
+	destroy_promisc_list(&promisc_list);
+    }
+
+    adjust_process_count(1);
+
     do {
 	getpacket(fd, buf, &fromaddr, &ch, &br, iface, win);
 
@@ -257,13 +281,15 @@ void packet_size_breakdown(struct OPTIONS *options, char *ifname, int facilityti
 	}
 	if ((br > 0) && (strcmp(iface, ifname) == 0))
 	    update_size_distrib(br, brackets, interval, win);
-	
+
 	gettimeofday(&tv, NULL);
 	now = tv.tv_sec;
 	unow = tv.tv_sec * 1e+6 + tv.tv_usec;
 
-	if (((options->updrate != 0) && (now - updtime >= options->updrate)) ||
-	    ((options->updrate == 0) && (unow - updtime_usec >= DEFAULT_UPDATE_DELAY))) {
+	if (((options->updrate != 0)
+	     && (now - updtime >= options->updrate))
+	    || ((options->updrate == 0)
+		&& (unow - updtime_usec >= DEFAULT_UPDATE_DELAY))) {
 	    update_panels();
 	    doupdate();
 	    updtime = now;
@@ -274,25 +300,37 @@ void packet_size_breakdown(struct OPTIONS *options, char *ifname, int facilityti
 	    timeint = now;
 	}
 	if ((now - startlog >= options->logspan) && (logging)) {
-	    write_size_log(brackets, now - starttime, ifname, mtu, logfile);
+	    write_size_log(brackets, now - starttime, ifname, mtu,
+			   logfile);
 	    startlog = now;
 	}
 	check_rotate_flag(&logfile, logging);
 
-	if ((facilitytime != 0) && (((now - starttime) / 60) >= facilitytime))
+	if ((facilitytime != 0)
+	    && (((now - starttime) / 60) >= facilitytime))
 	    exitloop = 1;
     } while (!exitloop);
 
     if (logging) {
 	signal(SIGUSR1, SIG_DFL);
 	write_size_log(brackets, now - starttime, ifname, mtu, logfile);
-	writelog(logging, logfile, "******** Packet size distribution facility stopped");
+	writelog(logging, logfile,
+		 "******** Packet size distribution facility stopped ********");
 	fclose(logfile);
     }
     close(fd);
+
+    if ((options->promisc) && (is_last_instance())) {
+	load_promisc_list(&promisc_list);
+	srpromisc(0, promisc_list);
+	destroy_promisc_list(&promisc_list);
+    }
+
+    adjust_process_count(-1);
+
     del_panel(panel);
     delwin(win);
     del_panel(borderpanel);
     delwin(borderwin);
-    unlink(PKTSIZEIDFILE);
+    unmark_facility(PKTSIZEIDFILE);
 }
