@@ -96,7 +96,8 @@ void process_rvn_packet(struct rvn *rvnpacket)
     strcpy(ccsa.sun_path, CHILDSOCKNAME);
 
     sendto(ccfd, rvnpacket, sizeof(struct rvn), 0,
-	    (struct sockaddr *) &ccsa, sizeof(ccsa.sun_family) + strlen(ccsa.sun_path));
+	   (struct sockaddr *) &ccsa,
+	   sizeof(ccsa.sun_family) + strlen(ccsa.sun_path));
     close(ccfd);
 }
 
@@ -104,12 +105,14 @@ void process_rvn_packet(struct rvn *rvnpacket)
  * Check if name is already resolved and in the cache.
  */
 
-int name_resolved(struct rvn *rvnpacket, struct hosts *hostlist, unsigned int lastfree)
+int name_resolved(struct rvn *rvnpacket, struct hosts *hostlist,
+		  unsigned int lastfree)
 {
     unsigned int i = 0;
 
     while (i != lastfree) {
-	if ((rvnpacket->saddr.s_addr == hostlist[i].addr) && (hostlist[i].ready == RESOLVED))
+	if ((rvnpacket->saddr.s_addr == hostlist[i].addr)
+	    && (hostlist[i].ready == RESOLVED))
 	    return i;
 
 	i++;
@@ -123,7 +126,8 @@ int name_resolved(struct rvn *rvnpacket, struct hosts *hostlist, unsigned int la
  * the given IP address
  */
 
-int addrstat(struct rvn *rvnpacket, struct hosts *hostlist, unsigned int lastfree)
+int addrstat(struct rvn *rvnpacket, struct hosts *hostlist,
+	     unsigned int lastfree)
 {
     unsigned int i = 0;
 
@@ -221,21 +225,29 @@ int main(void)
     cfd = socket(PF_UNIX, SOCK_DGRAM, 0);
 
     if (cfd < 0) {
-	writervnlog(logfile, "Unable to open child communication socket, aborting");
+	writervnlog(logfile,
+		    "Unable to open child communication socket, aborting");
 	exit(1);
     }
-    if (bind(cfd, (struct sockaddr *) &csa, sizeof(csa.sun_family) + strlen(csa.sun_path)) < 0) {
-	writervnlog(logfile, "Error binding child communication socket, aborting");
+    if (bind
+	(cfd, (struct sockaddr *) &csa,
+	 sizeof(csa.sun_family) + strlen(csa.sun_path)) < 0) {
+	writervnlog(logfile,
+		    "Error binding child communication socket, aborting");
 	exit(1);
     }
     ifd = socket(PF_UNIX, SOCK_DGRAM, 0);
 
     if (ifd < 0) {
-	writervnlog(logfile, "Unable to open client communication socket, aborting");
+	writervnlog(logfile,
+		    "Unable to open client communication socket, aborting");
 	exit(1);
     }
-    if (bind(ifd, (struct sockaddr *) &isa, sizeof(isa.sun_family) + strlen(isa.sun_path)) < 0) {
-	writervnlog(logfile, "Error binding client communication socket, aborting");
+    if (bind
+	(ifd, (struct sockaddr *) &isa,
+	 sizeof(isa.sun_family) + strlen(isa.sun_path)) < 0) {
+	writervnlog(logfile,
+		    "Error binding client communication socket, aborting");
 	exit(1);
     }
     while (1) {
@@ -244,11 +256,14 @@ int main(void)
 	FD_SET(ifd, &sockset);
 
 	do {
-	    ss = select(ifd + 1, &sockset, NULL, NULL, (struct timeval *) NULL);
+	    ss =
+		select(ifd + 1, &sockset, NULL, NULL,
+		       (struct timeval *) NULL);
 	} while ((ss < 0) && (errno != ENOMEM));
 
 	if (errno == ENOMEM) {
-	    writervnlog(logfile, "Fatal error: no memory for descriptor monitoring");
+	    writervnlog(logfile,
+			"Fatal error: no memory for descriptor monitoring");
 	    close(ifd);
 	    close(cfd);
 	    fclose(logfile);
@@ -259,9 +274,11 @@ int main(void)
 	 */
 
 	if (FD_ISSET(cfd, &sockset)) {
-	    fromlen = sizeof(fromaddr.sun_family) + strlen(fromaddr.sun_path);
-	    br = recvfrom(cfd, &rvnpacket, sizeof(struct rvn), 0,
-			   (struct sockaddr *) &fromaddr, &fromlen);
+	    fromlen =
+		sizeof(fromaddr.sun_family) + strlen(fromaddr.sun_path);
+	    br =
+		recvfrom(cfd, &rvnpacket, sizeof(struct rvn), 0,
+			 (struct sockaddr *) &fromaddr, &fromlen);
 
 	    if (br > 0) {
 		hi = 0;
@@ -292,16 +309,19 @@ int main(void)
 	 */
 
 	if (FD_ISSET(ifd, &sockset)) {
-	    fromlen = sizeof(fromaddr.sun_family) + strlen(fromaddr.sun_path);
-	    br = recvfrom(ifd, &rvnpacket, sizeof(struct rvn), 0,
-			   (struct sockaddr *) &fromaddr, &fromlen);
+	    fromlen =
+		sizeof(fromaddr.sun_family) + strlen(fromaddr.sun_path);
+	    br =
+		recvfrom(ifd, &rvnpacket, sizeof(struct rvn), 0,
+			 (struct sockaddr *) &fromaddr, &fromlen);
 	    if (br > 0) {
 		switch (rvnpacket.type) {
 		case RVN_HELLO:
 		    fromaddr.sun_family = AF_UNIX;
 		    strcpy(fromaddr.sun_path, IPTCLSOCKNAME);
 		    sendto(ifd, &rvnpacket, sizeof(struct rvn), 0,
-			    (struct sockaddr *) &fromaddr, sizeof(fromaddr.sun_family) +
+			   (struct sockaddr *) &fromaddr,
+			   sizeof(fromaddr.sun_family) +
 			   strlen(fromaddr.sun_path));
 		    break;
 		case RVN_QUIT:
@@ -318,18 +338,21 @@ int main(void)
 		    exit(0);
 #endif
 		case RVN_REQUEST:
-		    readyidx = name_resolved(&rvnpacket, hostlist, lastfree);
+		    readyidx =
+			name_resolved(&rvnpacket, hostlist, lastfree);
 		    if (readyidx >= 0) {
 			rvnpacket.type = RVN_REPLY;
 			bzero(rvnpacket.fqdn, 45);
-			strncpy(rvnpacket.fqdn, hostlist[readyidx].fqdn, 44);
+			strncpy(rvnpacket.fqdn, hostlist[readyidx].fqdn,
+				44);
 			rvnpacket.ready = RESOLVED;
 
 			fromaddr.sun_family = AF_UNIX;
 			strcpy(fromaddr.sun_path, IPTCLSOCKNAME);
 			br = sendto(ifd, &rvnpacket, sizeof(struct rvn), 0,
-				     (struct sockaddr *) &fromaddr,
-				    sizeof(fromaddr.sun_family) + strlen(fromaddr.sun_path));
+				    (struct sockaddr *) &fromaddr,
+				    sizeof(fromaddr.sun_family) +
+				    strlen(fromaddr.sun_path));
 		    } else {
 
 			/* 
@@ -337,8 +360,10 @@ int main(void)
 			 * new one.
 			 */
 
-			if (addrstat(&rvnpacket, hostlist, lastfree) == NOTRESOLVED) {
-			    hostlist[hostindex].addr = rvnpacket.saddr.s_addr;
+			if (addrstat(&rvnpacket, hostlist, lastfree) ==
+			    NOTRESOLVED) {
+			    hostlist[hostindex].addr =
+				rvnpacket.saddr.s_addr;
 			    hostlist[hostindex].ready = RESOLVING;
 
 			    hostindex++;
@@ -357,7 +382,8 @@ int main(void)
 				process_rvn_packet(&rvnpacket);
 				exit(0);
 			    case -1:
-				writervnlog(logfile, "Error on fork, returning IP address");
+				writervnlog(logfile,
+					    "Error on fork, returning IP address");
 				break;
 			    default:	/* parent */
 				break;
@@ -372,8 +398,9 @@ int main(void)
 			strcpy(fromaddr.sun_path, IPTCLSOCKNAME);
 
 			br = sendto(ifd, &rvnpacket, sizeof(struct rvn), 0,
-				     (struct sockaddr *) &fromaddr,
-				    sizeof(fromaddr.sun_family) + strlen(fromaddr.sun_path));
+				    (struct sockaddr *) &fromaddr,
+				    sizeof(fromaddr.sun_family) +
+				    strlen(fromaddr.sun_path));
 
 		    }
 		}
