@@ -24,6 +24,7 @@ details.
 #include <signal.h>
 #include <sys/time.h>
 #include <netdb.h>
+#include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
@@ -48,6 +49,7 @@ details.
 #include "error.h"
 #include "log.h"
 #include "timer.h"
+#include "promisc.h"
 #include "options.h"
 #include "instances.h"
 #include "packet.h"
@@ -59,7 +61,9 @@ details.
 extern int exitloop;
 extern int daemonized;
 
-extern void writeutslog(struct portlistent *list, unsigned long nsecs, FILE * logfile);
+extern void show_sort_statwin(WINDOW **win, PANEL **panel);
+extern void writeutslog(struct portlistent *list, unsigned long nsecs,
+			FILE * logfile);
 
 /*
  * SIGUSR1 logfile rotation signal handler
@@ -110,8 +114,9 @@ void initportlist(struct portlist *list)
     doupdate();
 }
 
-struct portlistent *addtoportlist(struct portlist *list, unsigned int protocol,
-			    unsigned int port, int *nomem, int servnames)
+struct portlistent *addtoportlist(struct portlist *list,
+				  unsigned int protocol, unsigned int port,
+				  int *nomem, int servnames)
 {
     struct portlistent *ptemp;
 
@@ -204,7 +209,7 @@ void printportent(struct portlist *list, struct portlistent *entry,
 	return;
 
     target_row = entry->idx - idx;
-
+    
     wmove(list->win, target_row, 1);
     if (entry->protocol == IPPROTO_TCP) {
 	wattrset(list->win, STDATTR);
@@ -213,7 +218,8 @@ void printportent(struct portlist *list, struct portlistent *entry,
 	wattrset(list->win, PTRATTR);
 	wprintw(list->win, "UDP");
     }
-    wprintw(list->win, "/%s: ", entry->servname);
+    
+    wprintw(list->win, "/%s:         ", entry->servname);
     wattrset(list->win, HIGHATTR);
     wmove(list->win, target_row, 17 * screen_scale);
     printlargenum(entry->count, list->win);
@@ -246,9 +252,9 @@ void destroyportlist(struct portlist *list)
     }
 }
 
-void printport(struct portlist *list, unsigned int protocol, unsigned int port,
-	     int br, unsigned int idx, int fromto, struct porttab *ports,
-	       int servnames)
+void printport(struct portlist *list, unsigned int protocol,
+	       unsigned int port, int br, unsigned int idx, int fromto,
+	       struct porttab *ports, int servnames)
 {
     struct portlistent *listent;
     int nomem = 0;
@@ -257,7 +263,8 @@ void printport(struct portlist *list, unsigned int protocol, unsigned int port,
 	listent = inportlist(list, protocol, port);
 
 	if ((listent == NULL) && (!nomem))
-	    listent = addtoportlist(list, protocol, port, &nomem, servnames);
+	    listent =
+		addtoportlist(list, protocol, port, &nomem, servnames);
 
 	if (listent == NULL)
 	    return;
@@ -277,6 +284,158 @@ void printport(struct portlist *list, unsigned int protocol, unsigned int port,
     }
 }
 
+/*
+ * Sorting for TCP/UDP service monitor, contributed by Gal Laszlo.
+ * Revised, GPJ <riker@seul.org> 10/4/2000.
+ * Contributor's note:
+ *      by LG - Gal Laszlo ( SlowTCP@hotmail.com ) 2000.07.18. 
+ *
+ * Author's note: accented characters removed for better terminal
+ * compatibility; comments in standard C style.
+ */
+
+/*
+ * Swap two portlist nodes
+ */
+
+void swapportents(struct portlist *list,
+		  struct portlistent *p1, struct portlistent *p2)
+{
+    register unsigned int tmp;
+    struct portlistent *p1prevsaved;
+    struct portlistent *p2nextsaved;
+
+    if (p1 == p2)
+	return;
+
+    tmp = p1->idx;
+    p1->idx = p2->idx;
+    p2->idx = tmp;
+
+    if (p1->prev_entry != NULL)
+	p1->prev_entry->next_entry = p2;
+    else
+	list->head = p2;
+
+    if (p2->next_entry != NULL)
+	p2->next_entry->prev_entry = p1;
+    else
+	list->tail = p1;
+
+    p2nextsaved = p2->next_entry;
+    p1prevsaved = p1->prev_entry;
+
+    if (p1->next_entry == p2) {
+	p2->next_entry = p1;
+	p1->prev_entry = p2;
+    } else {
+	p2->next_entry = p1->next_entry;
+	p1->prev_entry = p2->prev_entry;
+	p2->prev_entry->next_entry = p1;
+	p1->next_entry->prev_entry = p2;
+    }
+
+    p2->prev_entry = p1prevsaved;
+    p1->next_entry = p2nextsaved;
+}
+
+void sortportents(struct portlist *list, int *idx, int command)
+{
+    struct portlistent *ptemp1, *ptemp2;
+    struct portlistent *ptemp2next;
+
+    unsigned int long long val1 = 0, val2 = 0;
+    unsigned int idxtmp;
+    WINDOW *win;
+    PANEL *panel;
+    
+    if (!list->head)
+        return;
+        
+    command = toupper(command);
+
+    if ((command != 'R') && (command != 'B') && (command != 'O') &&
+       (command != 'M') && (command != 'P') && (command != 'T') &&
+       (command != 'F'))
+        return;
+        
+    ptemp1 = list->head;
+
+    show_sort_statwin(&win, &panel);
+    while (ptemp1 != NULL) {
+	ptemp2 = ptemp1->next_entry;
+
+        mvwprintw(win, 2, 13, "%u%%", ptemp1->idx * 100 / list->tail->idx);
+        update_panels();
+        doupdate();
+        
+	while (ptemp2 != NULL) {
+	    ptemp2next = ptemp2->next_entry;
+
+	    switch (command) {
+	    case 'R':		/* order by port number */
+		val2 = ptemp1->port;
+		val1 = ptemp2->port;
+		break;
+	    case 'B':		/* order by bytes */
+		val1 = ptemp1->bcount;
+		val2 = ptemp2->bcount;
+		break;
+	    case 'O':		/* order by bytesTo */
+		val1 = ptemp1->ibcount;
+		val2 = ptemp2->ibcount;
+		break;
+	    case 'M':		/* order by bytesFrom */
+		val1 = ptemp1->obcount;
+		val2 = ptemp2->obcount;
+		break;
+	    case 'P':		/* order by Pkts */
+		val1 = ptemp1->count;
+		val2 = ptemp2->count;
+		break;
+	    case 'T':		/* order by PktsTo */
+		val1 = ptemp1->icount;
+		val2 = ptemp2->icount;
+		break;
+	    case 'F':		/* order by PktsFrom */
+		val1 = ptemp1->ocount;
+		val2 = ptemp2->ocount;
+		break;
+	    }
+
+	    if (val1 < val2) {	/* descending sort */
+		swapportents(list, ptemp1, ptemp2);
+		ptemp1 = ptemp2;
+	    }
+	    ptemp2 = ptemp2next;
+	}
+
+	ptemp1 = ptemp1->next_entry;
+    }				/* end of the sorting */
+    
+    del_panel(panel);
+    delwin(win);
+    update_panels();
+    doupdate();
+    
+    ptemp1 = list->firstvisible = list->head;
+    *idx = 1;
+    idxtmp = 0;
+    while ((ptemp1) && (idxtmp <= LINES - 4)) {	/* printout */
+	printportent(list, ptemp1, *idx);
+	if (++idxtmp <= LINES - 4)
+	    list->lastvisible = ptemp1;
+	ptemp1 = ptemp1->next_entry;
+    }
+}
+
+/* 
+ * Contributor's note: 
+ *  	end of "by LG"
+ */
+
+
+
 void scrollservwin(struct portlist *table, int direction, int *idx)
 {
     char sp_buf[10];
@@ -324,6 +483,37 @@ void pageservwin(struct portlist *table, int direction, int *idx)
     }
 }
 
+void show_portsort_keywin(WINDOW ** win, PANEL ** panel)
+{
+    *win = newwin(14, 35, (LINES - 10) / 2, COLS - 40);
+    *panel = new_panel(*win);
+
+    wattrset(*win, BOXATTR);
+    colorwin(*win);
+    box(*win, ACS_VLINE, ACS_HLINE);
+
+    wattrset(*win, STDATTR);
+    mvwprintw(*win, 2, 2, "Select display field to sort by");
+    wmove(*win, 4, 2);
+    printkeyhelp("R", " - port number", *win);
+    wmove(*win, 5, 2);
+    printkeyhelp("P", " - total packets", *win);
+    wmove(*win, 6, 2);
+    printkeyhelp("B", " - total bytes", *win);
+    wmove(*win, 7, 2);
+    printkeyhelp("T", " - packets to", *win);
+    wmove(*win, 8, 2);
+    printkeyhelp("O", " - bytes to", *win);
+    wmove(*win, 9, 2);
+    printkeyhelp("F", " - packets from", *win);
+    wmove(*win, 10, 2);
+    printkeyhelp("M", " - bytes from", *win);
+    wmove(*win, 11, 2);
+    printkeyhelp("Any other key", " - cancel sort", *win);
+    update_panels();
+    doupdate();
+}
+
 /*
  * The TCP/UDP service monitor
  */
@@ -340,6 +530,8 @@ void servmon(char *ifname, struct porttab *ports,
     char *ipacket;
     char *tpacket;
 
+    int keymode = 0;
+
     unsigned int iphlen;
     struct sockaddr_ll fromaddr;
     unsigned short linktype;
@@ -370,6 +562,11 @@ void servmon(char *ifname, struct porttab *ports,
 
     FILE *logfile = NULL;
 
+    struct promisc_states *promisc_list;
+
+    WINDOW *sortwin;
+    PANEL *sortpanel;
+
     /*
      * Mark this facility
      */
@@ -377,30 +574,43 @@ void servmon(char *ifname, struct porttab *ports,
     if (!facility_active(TCPUDPIDFILE))
 	mark_facility(TCPUDPIDFILE, "TCP/UDP monitor");
     else {
-	write_error("TCP/UDP monitor already active in another process", daemonized);
+	write_error("TCP/UDP monitor already active in another process",
+		    daemonized);
 	return;
     }
 
+
     open_socket(&fd);
 
     if (fd < 0) {
-	unlink(TCPUDPIDFILE);
+	unmark_facility(TCPUDPIDFILE);
 	return;
     }
     if (!iface_supported(ifname)) {
 	err_iface_unsupported();
-	unlink(TCPUDPIDFILE);
+	unmark_facility(TCPUDPIDFILE);
 	return;
     }
     if (!iface_up(ifname)) {
 	err_iface_down();
-	unlink(TCPUDPIDFILE);
+	unmark_facility(TCPUDPIDFILE);
 	return;
     }
+
+    if ((first_active_facility()) && (options->promisc)) {
+	init_promisc_list(&promisc_list);
+	save_promisc_list(promisc_list);
+	srpromisc(1, promisc_list);
+	destroy_promisc_list(&promisc_list);
+    }
+
+    adjust_process_count(1);
+
     initportlist(&list);
 
     move(LINES - 1, 1);
     scrollkeyhelp();
+    sortkeyhelp();
     stdexitkeyhelp();
 
     update_panels();
@@ -419,7 +629,8 @@ void servmon(char *ifname, struct porttab *ports,
 	signal(SIGUSR1, rotate_serv_log);
 
     rotate_flag = 0;
-    writelog(logging, logfile, "******** TCP/UDP service monitor started");
+    writelog(logging, logfile,
+	     "******** TCP/UDP service monitor started ********");
 
     bzero(&isdntable, sizeof(struct isdntab));
 
@@ -430,35 +641,49 @@ void servmon(char *ifname, struct porttab *ports,
     while (!exitloop) {
 	getpacket(fd, buf, &fromaddr, &ch, &br, iface, list.win);
 
-	if (ch != ERR)
-	    switch (ch) {
-	    case KEY_UP:
-		scrollservwin(&list, SCROLLDOWN, &idx);
-		break;
-	    case KEY_DOWN:
-		scrollservwin(&list, SCROLLUP, &idx);
-		break;
-	    case KEY_PPAGE:
-	    case '-':
-		pageservwin(&list, SCROLLDOWN, &idx);
-		break;
-	    case KEY_NPAGE:
-	    case ' ':
-		pageservwin(&list, SCROLLUP, &idx);
-		break;
-	    case 12:
-	    case 'l':
-	    case 'L':
-		refresh_screen();
-		break;
-	    case 'q':
-	    case 'Q':
-	    case 'x':
-	    case 'X':
-	    case 27:
-	    case 24:
-		exitloop = 1;
+	if (ch != ERR) {
+	    if (keymode == 0) {
+		switch (ch) {
+		case KEY_UP:
+		    scrollservwin(&list, SCROLLDOWN, &idx);
+		    break;
+		case KEY_DOWN:
+		    scrollservwin(&list, SCROLLUP, &idx);
+		    break;
+		case KEY_PPAGE:
+		case '-':
+		    pageservwin(&list, SCROLLDOWN, &idx);
+		    break;
+		case KEY_NPAGE:
+		case ' ':
+		    pageservwin(&list, SCROLLUP, &idx);
+		    break;
+		case 12:
+		case 'l':
+		case 'L':
+		    refresh_screen();
+		    break;
+		case 's':
+		case 'S':
+		    show_portsort_keywin(&sortwin, &sortpanel);
+		    keymode = 1;
+		    break;
+		case 'q':
+		case 'Q':
+		case 'x':
+		case 'X':
+		case 27:
+		case 24:
+		    exitloop = 1;
+		}
+	    } else if (keymode == 1) {
+		del_panel(sortpanel);
+		delwin(sortwin);
+		sortportents(&list, &idx, ch);
+		keymode = 0;
 	    }
+	}
+
 	if (br > 0) {
 	    if ((fromaddr.sll_protocol == ntohs(ETH_P_IP)) &&
 		((strcmp(iface, ifname) == 0))) {
@@ -485,29 +710,37 @@ void servmon(char *ifname, struct porttab *ports,
 		 * the fragment processor and get the ports from there.
 		 */
 
-		if ((ntohs(((struct iphdr *) ipacket)->frag_off) & 0x3fff) != 0) {
-		    br = processfragment((struct iphdr *) ipacket, &sport, &dport, &firstin);
+		if ((ntohs(((struct iphdr *) ipacket)->frag_off) & 0x3fff)
+		    != 0) {
+		    br =
+			processfragment((struct iphdr *) ipacket, &sport,
+					&dport, &firstin);
 		    if (!firstin)
 			continue;
 		} else {
 		    tpacket = ipacket + iphlen;
 
-		    if (((struct iphdr *) ipacket)->protocol == IPPROTO_TCP) {
+		    if (((struct iphdr *) ipacket)->protocol ==
+			IPPROTO_TCP) {
 			sport = ntohs(((struct tcphdr *) tpacket)->source);
 			dport = ntohs(((struct tcphdr *) tpacket)->dest);
-		    } else if (((struct iphdr *) ipacket)->protocol == IPPROTO_UDP) {
+		    } else if (((struct iphdr *) ipacket)->protocol ==
+			       IPPROTO_UDP) {
 			sport = ntohs(((struct udphdr *) tpacket)->source);
 			dport = ntohs(((struct udphdr *) tpacket)->dest);
 		    }
 		}
 
-		if ((((struct iphdr *) ipacket)->protocol == IPPROTO_TCP) ||
-		 (((struct iphdr *) ipacket)->protocol == IPPROTO_UDP)) {
+		if ((((struct iphdr *) ipacket)->protocol == IPPROTO_TCP)
+		    || (((struct iphdr *) ipacket)->protocol ==
+			IPPROTO_UDP)) {
 		    printport(&list, ((struct iphdr *) ipacket)->protocol,
-		       sport, ntohs(((struct iphdr *) ipacket)->tot_len),
+			      sport,
+			      ntohs(((struct iphdr *) ipacket)->tot_len),
 			      idx, 0, ports, options->servnames);
 		    printport(&list, ((struct iphdr *) ipacket)->protocol,
-		       dport, ntohs(((struct iphdr *) ipacket)->tot_len),
+			      dport,
+			      ntohs(((struct iphdr *) ipacket)->tot_len),
 			      idx, 1, ports, options->servnames);
 		}
 	    }
@@ -517,15 +750,18 @@ void servmon(char *ifname, struct porttab *ports,
 	unow = tv.tv_sec * 1e+6 + tv.tv_usec;
 
 	if (now - timeint >= 5) {
-	    printelapsedtime(starttime, now, LINES - 3, 20, list.borderwin);
+	    printelapsedtime(starttime, now, LINES - 3, 20,
+			     list.borderwin);
 	    timeint = now;
 	}
 	if (((now - startlog) >= options->logspan) && (logging)) {
 	    writeutslog(list.head, now - starttime, logfile);
 	    startlog = now;
 	}
-	if (((options->updrate != 0) && (now - updtime >= options->updrate)) ||
-	    ((options->updrate == 0) && (unow - updtime_usec >= DEFAULT_UPDATE_DELAY))) {
+	if (((options->updrate != 0)
+	     && (now - updtime >= options->updrate))
+	    || ((options->updrate == 0)
+		&& (unow - updtime_usec >= DEFAULT_UPDATE_DELAY))) {
 	    update_panels();
 	    doupdate();
 	    updtime = now;
@@ -533,24 +769,34 @@ void servmon(char *ifname, struct porttab *ports,
 	}
 	check_rotate_flag(&logfile, logging);
 
-	if ((facilitytime != 0) && (((now - starttime) / 60) >= facilitytime))
+	if ((facilitytime != 0)
+	    && (((now - starttime) / 60) >= facilitytime))
 	    exitloop = 1;
     }
 
     if (logging) {
 	signal(SIGUSR1, SIG_DFL);
 	writeutslog(list.head, time((time_t *) NULL) - starttime, logfile);
-	writelog(logging, logfile, "******** TCP/UDP service monitor stopped");
+	writelog(logging, logfile,
+		 "******** TCP/UDP service monitor stopped ********");
 	fclose(logfile);
     }
     if (options->servnames)
 	endservent();
 
+    if ((options->promisc) && (is_last_instance())) {
+	load_promisc_list(&promisc_list);
+	srpromisc(0, promisc_list);
+	destroy_promisc_list(&promisc_list);
+    }
+
+    adjust_process_count(-1);
+
     del_panel(list.panel);
     delwin(list.win);
     del_panel(list.borderpanel);
     delwin(list.borderwin);
-    unlink(TCPUDPIDFILE);
+    unmark_facility(TCPUDPIDFILE);
     update_panels();
     doupdate();
     destroyportlist(&list);
@@ -582,20 +828,13 @@ void portdlg(unsigned int *port_min, int *port_max, int *aborted, int mode)
     stdwinset(win);
     wtimeout(win, -1);
 
-    wmove(win, 1, 1);
-    wprintw(win, "Port numbers below 1024 are reserved for");
-    wmove(win, 2, 1);
-    wprintw(win, "TCP/IP services, and are normally the only");
-    wmove(win, 3, 1);
-    wprintw(win, "ones monitored by the TCP/UDP statistics");
-    wmove(win, 4, 1);
-    wprintw(win, "module.  If you wish to monitor a higher-");
-    wmove(win, 5, 1);
-    wprintw(win, "numbered port or range of ports, enter it");
-    wmove(win, 6, 1);
-    wprintw(win, "here.  Fill just the first field for a");
-    wmove(win, 7, 1);
-    wprintw(win, "single port, or both fields for a range.");
+    mvwprintw(win, 1, 1, "Port numbers below 1024 are reserved for");
+    mvwprintw(win, 2, 1, "TCP/IP services, and are normally the only");
+    mvwprintw(win, 3, 1, "ones monitored by the TCP/UDP statistics");
+    mvwprintw(win, 4, 1, "module.  If you wish to monitor a higher-");
+    mvwprintw(win, 5, 1, "numbered port or range of ports, enter it");
+    mvwprintw(win, 6, 1, "here.  Fill just the first field for a");
+    mvwprintw(win, 7, 1, "single port, or both fields for a range.");
 
     wmove(win, 11, 1);
     tabkeyhelp(win);
@@ -603,8 +842,7 @@ void portdlg(unsigned int *port_min, int *port_max, int *aborted, int mode)
 
     initfields(&list, 1, 20, (LINES - 14) / 2 + 10, (COLS - 50) / 2 - 8);
     wattrset(list.fieldwin, STDATTR);
-    wmove(list.fieldwin, 0, 6);
-    wprintw(list.fieldwin, "to");
+    mvwprintw(list.fieldwin, 0, 6, "to");
 
     addfield(&list, 5, 0, 0, "");
     addfield(&list, 5, 0, 9, "");
@@ -651,7 +889,8 @@ void saveportlist(struct porttab *table)
     close(fd);
 }
 
-int dup_portentry(struct porttab *table, unsigned int min, unsigned int max)
+int dup_portentry(struct porttab *table, unsigned int min,
+		  unsigned int max)
 {
     struct porttab *ptmp = table;
 
@@ -839,7 +1078,8 @@ void operate_portselect(struct porttab **table, struct porttab **node,
     } while (!endloop);
 }
 
-void selectport(struct porttab **table, struct porttab **node, int *aborted)
+void selectport(struct porttab **table, struct porttab **node,
+		int *aborted)
 {
     WINDOW *bw;
     PANEL *bp;
