@@ -32,6 +32,8 @@ unsigned int imaxy = 0;
 
 void writetcplog(int logging, FILE * fd, struct tcptableent *entry,
 		 unsigned int pktlen, char *message);
+void write_timeout_log(int logging, FILE * logfile,
+		       struct tcptableent *tcpnode);
 
 void setlabels(WINDOW * win, int mode)
 {
@@ -189,6 +191,12 @@ void del_tcp_hash_node(struct tcptable *table, struct tcptableent *entry)
     free(ptmp);
 }
 
+void print_tcp_num_entries(struct tcptable *table)
+{
+    mvwprintw(table->borderwin, table->bmaxy - 1, 1, " TCP: %6u entries ",
+	      table->count);
+}
+
 /*
  * Add a new entry to the TCP screen table
  */
@@ -241,16 +249,17 @@ struct tcptableent *addentry(struct tcptable *table,
 	new_entry->next_entry->next_entry = NULL;
 
 
-	if (new_entry->oth_connection->index <= table->firstvisible->index + (table->imaxy - 1))
+	if (new_entry->oth_connection->index <=
+	    table->firstvisible->index + (table->imaxy - 1))
 	    table->lastvisible = new_entry->oth_connection;
-	else if (new_entry->index <= table->firstvisible->index + (table->imaxy - 1))
+	else if (new_entry->index <=
+		 table->firstvisible->index + (table->imaxy - 1))
 	    table->lastvisible = new_entry;
 
 	new_entry->reused = new_entry->oth_connection->reused = 0;
 	table->count++;
 
-	wmove(table->borderwin, table->bmaxy - 1, 1);
-	wprintw(table->borderwin, " TCP: %u entries ", table->count);
+	print_tcp_num_entries(table);
     } else {
 	/*
 	 * If we reach this point, we're allocating off the list of closed
@@ -288,8 +297,10 @@ struct tcptableent *addentry(struct tcptable *table,
      * Fill in address fields with raw IP addresses
      */
 
-    new_entry->saddr.s_addr = new_entry->oth_connection->daddr.s_addr = saddr;
-    new_entry->daddr.s_addr = new_entry->oth_connection->saddr.s_addr = daddr;
+    new_entry->saddr.s_addr = new_entry->oth_connection->daddr.s_addr =
+	saddr;
+    new_entry->daddr.s_addr = new_entry->oth_connection->saddr.s_addr =
+	daddr;
     new_entry->protocol = protocol;
 
     /*
@@ -298,7 +309,9 @@ struct tcptableent *addentry(struct tcptable *table,
 
     new_entry->pcount = new_entry->bcount = 0;
     new_entry->win = new_entry->psize = 0;
-    new_entry->oth_connection->pcount = new_entry->oth_connection->bcount = 0;
+    new_entry->timedout = new_entry->oth_connection->timedout = 0;
+    new_entry->oth_connection->pcount = new_entry->oth_connection->bcount =
+	0;
     new_entry->oth_connection->win = new_entry->oth_connection->psize = 0;
 
     /*
@@ -404,8 +417,6 @@ struct tcptableent *in_table(struct tcptable *table, unsigned long saddr,
     struct tcp_hashentry *hashptr;
     unsigned int hp;
 
-    char msgstring[MSGSTRING_MAX];
-
     int hastimeouts = 0;
 
     time_t now;
@@ -435,18 +446,16 @@ struct tcptableent *in_table(struct tcptable *table, unsigned long saddr,
 	 * find any closed ones.
 	 */
 
-	if ((tm > 0) && ((now - hashptr->tcpnode->lastupdate) / 60 > tm) && (!(hashptr->tcpnode->inclosed))) {
+	if ((tm > 0) && ((now - hashptr->tcpnode->lastupdate) / 60 > tm)
+	    && (!(hashptr->tcpnode->inclosed))) {
+	    hashptr->tcpnode->timedout = 1;
+	    hashptr->tcpnode->oth_connection->timedout = 1;
 	    addtoclosedlist(table, hashptr->tcpnode, nomem);
 	    if (!(*nomem))
 		hastimeouts = 1;
 
-	    if (logging) {
-		sprintf(msgstring, "TCP; Connection %s:%s <-> %s:%s idle for %u minutes, marking for timeout, %lu packets, %lu bytes",
-		     hashptr->tcpnode->s_fqdn, hashptr->tcpnode->s_sname,
-		     hashptr->tcpnode->d_fqdn, hashptr->tcpnode->d_sname,
-		 tm, hashptr->tcpnode->pcount, hashptr->tcpnode->bcount);
-		writelog(logging, logfile, msgstring);
-	    }
+	    if (logging)
+		write_timeout_log(logging, logfile, hashptr->tcpnode);
 	}
 	hashptr = hashptr->next_entry;
     }
@@ -498,7 +507,8 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
      */
 
     if ((ntohs(fragofs) & 0x1fff) != 0) {
-	tableentry->lastupdate = tableentry->oth_connection->lastupdate = time(NULL);
+	tableentry->lastupdate = tableentry->oth_connection->lastupdate =
+	    time(NULL);
 	return;
     }
     /*
@@ -532,12 +542,13 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 	 */
 
 	if ((tableentry->oth_connection->finsent == 1) &&
-	(ntohl(transpacket->seq) == tableentry->oth_connection->finack)) {
+	    (ntohl(transpacket->seq) ==
+	     tableentry->oth_connection->finack)) {
 	    tableentry->oth_connection->finsent = 2;
 
 	    if (logging) {
-		writetcplog(logging, logfile, tableentry, tableentry->psize,
-			    "FIN acknowleged");
+		writetcplog(logging, logfile, tableentry,
+			    tableentry->psize, "FIN acknowleged");
 	    }
 	}
     }
@@ -581,7 +592,8 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 	    sprintf(msgstring, "FIN sent; %lu packets, %lu bytes",
 		    tableentry->pcount, tableentry->bcount);
 
-	    writetcplog(logging, logfile, tableentry, tableentry->psize, msgstring);
+	    writetcplog(logging, logfile, tableentry, tableentry->psize,
+			msgstring);
 	}
     }
     if (transpacket->rst) {
@@ -590,11 +602,13 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 	    addtoclosedlist(table, tableentry, nomem);
 
 	if (logging) {
-	    sprintf(msgstring, "Connection reset; %lu packets, %lu bytes; opposite direction %lu packets, %lu bytes",
+	    sprintf(msgstring,
+		    "Connection reset; %lu packets, %lu bytes; opposite direction %lu packets, %lu bytes",
 		    tableentry->pcount, tableentry->bcount,
 		    tableentry->oth_connection->pcount,
 		    tableentry->oth_connection->bcount);
-	    writetcplog(logging, logfile, tableentry, tableentry->psize, msgstring);
+	    writetcplog(logging, logfile, tableentry, tableentry->psize,
+			msgstring);
 	}
     }
     if (transpacket->psh)
@@ -603,15 +617,20 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
     if (transpacket->urg)
 	tableentry->stat |= FLAG_URG;
 
-    tableentry->lastupdate = tableentry->oth_connection->lastupdate = time(NULL);
+    tableentry->lastupdate = tableentry->oth_connection->lastupdate =
+	time(NULL);
     /*
      * Shall we add this entry to the closed entry list?  If both
      * directions have their state indicators set to 2, that's it.
      */
 
     if ((!tableentry->inclosed) &&
-	(((tableentry->finsent == 2) && ((tableentry->oth_connection->finsent == 1) || (tableentry->oth_connection->finsent == 2))) ||
-	 ((tableentry->oth_connection->finsent == 2) && ((tableentry->finsent == 1) || (tableentry->finsent == 2)))))
+	(((tableentry->finsent
+	   == 2) && ((tableentry->oth_connection->finsent == 1)
+		     || (tableentry->oth_connection->finsent == 2)))
+	 || ((tableentry->oth_connection->finsent == 2)
+	     && ((tableentry->finsent == 1)
+		 || (tableentry->finsent == 2)))))
 	addtoclosedlist(table, tableentry, nomem);
 
 }
@@ -682,7 +701,8 @@ void printentry(struct tcptable *table, struct tcptableent *tableentry,
     wattrset(table->tcpscreen, STDATTR);
 
     wmove(table->tcpscreen, target_row, 1);
-    wprintw(table->tcpscreen, sp_buf, tableentry->s_fqdn, tableentry->s_sname);
+    wprintw(table->tcpscreen, sp_buf, tableentry->s_fqdn,
+	    tableentry->s_sname);
 
     wattrset(table->tcpscreen, HIGHATTR);
 
@@ -745,15 +765,20 @@ void printentry(struct tcptable *table, struct tcptableent *tableentry,
     wprintw(table->tcpscreen, "%s", tableentry->ifname);
 }
 
+/*
+ * Redraw the TCP window
+ */
+
 void refreshtcpwin(struct tcptable *table, unsigned int idx, int mode)
 {
     struct tcptableent *ptmp;
     int endloop = 0;
 
     setlabels(table->borderwin, mode);
+    colorwin(table->tcpscreen);
     ptmp = table->firstvisible;
 
-    if (ptmp != NULL)
+    if (ptmp != NULL) {
 	do {
 	    printentry(table, ptmp, idx, mode);
 	    ptmp = ptmp->next_entry;
@@ -762,17 +787,43 @@ void refreshtcpwin(struct tcptable *table, unsigned int idx, int mode)
 	    else if (ptmp->prev_entry == table->lastvisible)
 		endloop = 1;
 	} while (!endloop);
+    }
+    wmove(table->borderwin, table->bmaxy - 1, 1);
+
+    print_tcp_num_entries(table);
 
     update_panels();
     doupdate();
 }
 
+void destroy_closed_entries(struct tcptable *table)
+{
+    struct closedlist *closedtemp;
+    struct closedlist *closedtemp_next;
+
+    if (table->closedentries != NULL) {
+	closedtemp = table->closedentries;
+	closedtemp_next = table->closedentries->next_entry;
+
+	while (closedtemp != NULL) {
+	    free(closedtemp);
+	    closedtemp = closedtemp_next;
+	    if (closedtemp_next != NULL)
+		closedtemp_next = closedtemp_next->next_entry;
+	}
+
+	table->closedentries = NULL;
+	table->closedtail = NULL;
+    }
+}
+
+/*
+ * Kill the entire TCP table
+ */
 void destroytcptable(struct tcptable *table)
 {
     struct tcptableent *ctemp;
     struct tcptableent *c_next_entry;
-    struct closedlist *closedtemp;
-    struct closedlist *closedtemp_next;
     struct tcp_hashentry *hashtemp;
     struct tcp_hashentry *hashtemp_next;
 
@@ -798,17 +849,9 @@ void destroytcptable(struct tcptable *table)
      * Destroy list of closed entries
      */
 
-    if (table->closedentries != NULL) {
-	closedtemp = table->closedentries;
-	closedtemp_next = table->closedentries->next_entry;
 
-	while (closedtemp != NULL) {
-	    free(closedtemp);
-	    closedtemp = closedtemp_next;
-	    if (closedtemp_next != NULL)
-		closedtemp_next = closedtemp_next->next_entry;
-	}
-    }
+    destroy_closed_entries(table);
+
     /*
      * Destroy hash table
      */
@@ -828,3 +871,113 @@ void destroytcptable(struct tcptable *table)
 	}
     }
 }
+
+/*
+ * Kill an entry from the TCP table
+ */
+
+void destroy_tcp_entry(struct tcptable *table, struct tcptableent *ptmp)
+{
+    if (ptmp->prev_entry != NULL)
+	ptmp->prev_entry->next_entry = ptmp->next_entry;
+    else
+	table->head = ptmp->next_entry;
+
+    if (ptmp->next_entry != NULL)
+	ptmp->next_entry->prev_entry = ptmp->prev_entry;
+    else
+	table->tail = ptmp->prev_entry;
+
+    free(ptmp);
+
+    if (table->head == NULL) {
+	table->firstvisible = NULL;
+	table->lastvisible = NULL;
+    }
+}
+
+/*
+ * Kill all closed entries from the table, and clear the list of closed
+ * entries.
+ */
+
+void flushclosedentries(struct tcptable *table, time_t tm,
+			unsigned long *screen_idx,
+			int logging, FILE * logfile)
+{
+    struct tcptableent *ptmp = table->head;
+    struct tcptableent *ctmp = NULL;
+    unsigned long idx = 1;
+    time_t now;
+    time_t lastupdated = 0;
+
+    while (ptmp != NULL) {
+	now = time(NULL);
+	lastupdated = (now - ptmp->lastupdate) / 60;
+
+	if ((ptmp->inclosed) || (lastupdated > tm)) {
+	    ctmp = ptmp;
+	    /*
+	     * Mark and flush timed out TCP entries.
+	     */
+	    if (lastupdated > tm) {
+		if ((!(ptmp->timedout)) && (!(ptmp->inclosed))) {
+		    write_timeout_log(logging, logfile, ptmp);
+		    ptmp->timedout = ptmp->oth_connection->timedout = 1;
+		}
+	    }
+
+	    /*
+	     * Advance to next entry and destroy target entry.
+	     */
+	    ptmp = ptmp->next_entry;
+	    del_tcp_hash_node(table, ctmp);
+	    destroy_tcp_entry(table, ctmp);
+	} else {
+	    if (idx == *screen_idx)
+		table->firstvisible = ptmp;
+
+	    if (idx <= (*screen_idx) + (table->imaxy - 1))
+		table->lastvisible = ptmp;
+
+	    ptmp->index = idx;
+	    idx++;
+	    ptmp = ptmp->next_entry;
+	}
+    }
+
+    table->lastpos = idx - 1;
+    table->count = table->lastpos / 2;
+    destroy_closed_entries(table);
+
+    /*
+     * Shift entries down if the deletion causes the last entry to
+     * occupy anywhere other than the last line of the TCP display
+     * window.
+     */
+
+    if (table->head != NULL) {
+
+	/*
+	 * Point screen index to the last table entry if the tail entry is
+	 * "above" the screen index.  Set the firstvisible pointer to that
+	 * as well.
+	 */
+	if (table->tail->index < *screen_idx) {
+	    *screen_idx = table->tail->index;
+	    table->firstvisible = table->tail;
+	}
+
+	/*
+	 * Move the screen index and firstvisible entry up until the tail
+	 * hits the bottom of the window (tail is at screen index plus
+	 * screen length minus 1) or the firstvisible pointer hits the
+	 * head of the table.
+	 */
+	while ((table->tail->index < *screen_idx + table->imaxy - 1) &&
+	       (table->firstvisible->prev_entry != NULL)) {
+	    table->firstvisible = table->firstvisible->prev_entry;
+	    (*screen_idx)--;
+	}
+    }
+}
