@@ -27,6 +27,8 @@
 #include "servname.h"
 #include "tcphdr.h"
 
+#define max(a, b) (( a > b) ? a : b)
+
 #define FLAG_SYN	1
 #define FLAG_RST	2
 #define FLAG_PSH	4
@@ -61,6 +63,7 @@ struct tcptableent {
     char ifname[8];
     unsigned int index;
     int reused;
+    int timedout;
     int inclosed;
     int half_bracket;
     struct tcp_hashentry *hash_node;
@@ -136,3 +139,6 @@ void printentry(struct tcptable *table, struct tcptableent *tableentry,
 void refreshtcpwin(struct tcptable *table, unsigned int idx, int mode);
 
 void destroytcptable(struct tcptable *table);
+
+void flushclosedentries(struct tcptable *table, time_t tm, unsigned long *screen_idx,
+                        int logging, FILE *logfile);
