@@ -41,6 +41,8 @@ details.
 #include "error.h"
 #include "instances.h"
 
+extern int daemonized;
+
 /*
  * Generate a string representation of a number to be used as a name.
  */
@@ -52,12 +54,10 @@ void genname(unsigned long n, char *m)
 
 void listfileerr(int code)
 {
-    int resp;
-
     if (code == 1)
-	errbox("Error loading TCP/UDP filter list file", ANYKEY_MSG, &resp);
+	write_error("Error loading TCP/UDP filter list file", daemonized);
     else
-	errbox("Error writing TCP/UDP filter list file", ANYKEY_MSG, &resp);
+	write_error("Error writing TCP/UDP filter list file", daemonized);
 }
 
 void gethostparams(struct hostparams *hp, int *aborted,
@@ -98,7 +98,8 @@ void gethostparams(struct hostparams *hp, int *aborted,
     update_panels();
     doupdate();
 
-    initfields(&fieldlist, 8, 52, (LINES - 12) / 2 + 1, (COLS - 80) / 2 + 27);
+    initfields(&fieldlist, 8, 52, (LINES - 12) / 2 + 1,
+	       (COLS - 80) / 2 + 27);
 
     addfield(&fieldlist, 25, 1, 0, initsrc);
     addfield(&fieldlist, 25, 3, 0, initsmask);
@@ -139,14 +140,16 @@ int mark_filter_change(int protocol)
 	if (!facility_active(TCPFLTIDFILE))
 	    mark_facility(TCPFLTIDFILE, "TCP filter change");
 	else {
-	    errbox("TCP filter data file in use; try again later", ANYKEY_MSG, &resp);
+	    errbox("TCP filter data file in use; try again later",
+		   ANYKEY_MSG, &resp);
 	    return 0;
 	}
     } else {
 	if (!facility_active(UDPFLTIDFILE))
 	    mark_facility(UDPFLTIDFILE, "UDP filter change");
 	else {
-	    errbox("UDP filter data file in use; try again later", ANYKEY_MSG, &resp);
+	    errbox("UDP filter data file in use; try again later",
+		   ANYKEY_MSG, &resp);
 	    return 0;
 	}
     }
@@ -186,20 +189,25 @@ void definefilter(int protocol, int *aborted)
     }
     genname(time((time_t *) NULL), fntemp);
 
-    pfd = open(get_path(T_WORKDIR, fntemp), O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR);
+    pfd =
+	open(get_path(T_WORKDIR, fntemp), O_CREAT | O_WRONLY | O_TRUNC,
+	     S_IRUSR | S_IWUSR);
     if (pfd < 0) {
-	errbox("Cannot create TCP/UDP filter record file", ANYKEY_MSG, &resp);
+	errbox("Cannot create TCP/UDP filter record file", ANYKEY_MSG,
+	       &resp);
 	*aborted = 1;
 	clear_flt_tag(protocol);
 	return;
     }
     do {
-	gethostparams(&hp, aborted, "", "", "", "0.0.0.0", "0.0.0.0", "0", "I");
+	gethostparams(&hp, aborted, "", "", "", "0.0.0.0", "0.0.0.0", "0",
+		      "I");
 	if (!(*aborted)) {
 	    bw = write(pfd, &hp, sizeof(struct hostparams));
 
 	    if (bw < 0) {
-		errbox("Unable to write TCP/UDP filter record", ANYKEY_MSG, &resp);
+		errbox("Unable to write TCP/UDP filter record", ANYKEY_MSG,
+		       &resp);
 		close(pfd);
 		clear_flt_tag(protocol);
 		return;
@@ -210,9 +218,13 @@ void definefilter(int protocol, int *aborted)
     close(pfd);
 
     if (protocol == F_TCP)
-	pfd = open(TCPFLNAME, O_CREAT | O_WRONLY | O_APPEND, S_IRUSR | S_IWUSR);
+	pfd =
+	    open(TCPFLNAME, O_CREAT | O_WRONLY | O_APPEND,
+		 S_IRUSR | S_IWUSR);
     else
-	pfd = open(UDPFLNAME, O_CREAT | O_WRONLY | O_APPEND, S_IRUSR | S_IWUSR);
+	pfd =
+	    open(UDPFLNAME, O_CREAT | O_WRONLY | O_APPEND,
+		 S_IRUSR | S_IWUSR);
 
     if (pfd < 0) {
 	listfileerr(1);
@@ -253,8 +265,8 @@ void displayfilters(struct ffnode *ffiles, WINDOW * win)
     doupdate();
 }
 
-void operate_select(struct ffnode *ffiles, WINDOW * win, struct ffnode **item,
-		    int *aborted)
+void operate_select(struct ffnode *ffiles, WINDOW * win,
+		    struct ffnode **item, int *aborted)
 {
     unsigned int row = 0;
     struct ffnode *pptr;
@@ -383,31 +395,32 @@ void selectfilter(unsigned int protocol,
     destroyfilterlist(ffiles);
 }
 
-unsigned long int nametoaddr(char *ascname)
+unsigned long int nametoaddr(char *ascname, int *err)
 {
     unsigned long int result;
     struct hostent *he;
     char imsg[45];
-    int resp;
+    struct in_addr inp;
+    int resolv_err = 0;
 
-    result = inet_addr(ascname);
-    if (result == -1) {
-	bzero(imsg, 45);
-	strcpy(imsg, "Resolving ");
-	strncat(imsg, ascname, 35);
+    resolv_err = inet_aton(ascname, &inp);
+    if (resolv_err == 0) {
+	snprintf(imsg, 45, "Resolving %s", ascname);
 	indicate(imsg);
 	he = gethostbyname(ascname);
 	if (he != NULL)
 	    bcopy((he->h_addr_list)[0], &result, he->h_length);
 	else {
-	    bzero(imsg, 45);
-	    strcpy(imsg, "Unable to resolve ");
-	    strncat(imsg, ascname, 22);
-	    errbox(imsg, ANYKEY_MSG, &resp);
-	    result = -1;
+	    snprintf(imsg, 45, "Unable to resolve %s", ascname);
+	    write_error(imsg, daemonized);
+	    *err = 1;
+	    return (-1);
 	}
-    }
-    return result;
+    } else
+	result = inp.s_addr;
+
+    return (result);
+    *err = 0;
 }
 
 /* 
@@ -420,14 +433,14 @@ int loadfilter(char *filename, struct filterlist *fl, int resolve)
     int pfd;
     unsigned int idx = 0;
     int br;
-    int resp;
+    int resolv_err = 0;
 
     init_filter_table(fl);
 
     pfd = open(filename, O_RDONLY);
 
     if (pfd < 0) {
-	errbox("Error opening TCP/UDP filter file", ANYKEY_MSG, &resp);
+	write_error("Error opening TCP/UDP filter file", daemonized);
 	fl->head = NULL;
 	return 1;
     }
@@ -438,13 +451,14 @@ int loadfilter(char *filename, struct filterlist *fl, int resolve)
 	if (br > 0) {
 	    fe->index = idx;
 	    if (resolve) {
-		fe->saddr = nametoaddr(fe->hp.s_fqdn);
-		fe->daddr = nametoaddr(fe->hp.d_fqdn);
+		fe->saddr = nametoaddr(fe->hp.s_fqdn, &resolv_err);
+		fe->daddr = nametoaddr(fe->hp.d_fqdn, &resolv_err);
 
-		if ((fe->saddr == -1) || (fe->daddr == -1)) {
+		if (resolv_err) {
 		    free(fe);
 		    continue;
 		}
+
 		fe->smask = inet_addr(fe->hp.s_mask);
 		fe->dmask = inet_addr(fe->hp.d_mask);
 	    }
@@ -465,17 +479,18 @@ int loadfilter(char *filename, struct filterlist *fl, int resolve)
 
     if (br == 0)
 	close(pfd);
-	
+
     return 0;
 }
 
-void savefilter(unsigned int protocol, char *filename, struct filterlist *fl)
+void savefilter(unsigned int protocol, char *filename,
+		struct filterlist *fl)
 {
     struct filterent *fe = fl->head;
     int pfd;
     int bw;
     int resp;
-    
+
     pfd = open(filename, O_CREAT | O_TRUNC | O_WRONLY, S_IRUSR | S_IWUSR);
 
     while (fe != NULL) {
@@ -496,7 +511,9 @@ void marktcpfilter(char *filename)
     int fd;
     int resp;
 
-    fd = open(TCPFILTERSAVE, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
+    fd =
+	open(TCPFILTERSAVE, O_WRONLY | O_CREAT | O_TRUNC,
+	     S_IRUSR | S_IWUSR);
 
     if (write(fd, filename, FLT_FILENAME_MAX) < 0) {
 	errbox("Unable to save TCP/UDP filter name", ANYKEY_MSG, &resp);
@@ -522,7 +539,8 @@ void loadsavedtcpfilter(struct filterlist *fl, int *filtered)
 
     bzero(filename, FLT_FILENAME_MAX);
     if (read(fd, filename, FLT_FILENAME_MAX) < 0) {
-	errbox("Unable to retrieve saved TCP/UDP filter", ANYKEY_MSG, &resp);
+	errbox("Unable to retrieve saved TCP/UDP filter", ANYKEY_MSG,
+	       &resp);
 	if (fd > 0)
 	    close(fd);
 
@@ -539,16 +557,20 @@ void loadsavedtcpfilter(struct filterlist *fl, int *filtered)
 	*filtered = 1;
 }
 
-/* the display filter */
+/*
+ * The TCP/UDP display filter
+ */
 
 int utfilter(struct filterlist *fl,
 	     unsigned long source, unsigned long dest,
-	     unsigned int sport, unsigned int dport)
+	     unsigned int sport, unsigned int dport, unsigned int protocol)
 {
     struct filterent *fe;
     unsigned long fsaddr, fdaddr;
     unsigned long csaddr, cdaddr;
     unsigned long crsaddr, crdaddr;
+    int flt_expr1 = 0;
+    int flt_expr2 = 0;
 
     fe = fl->head;
 
@@ -560,11 +582,22 @@ int utfilter(struct filterlist *fl,
 	crsaddr = source & fe->dmask;
 	crdaddr = dest & fe->smask;
 
-	if (((csaddr == fsaddr) && ((fe->hp.sport == sport) || (fe->hp.sport == 0)) &&
-	     (cdaddr == fdaddr) && ((fe->hp.dport == dport) || (fe->hp.dport == 0))) ||
-	    ((crsaddr == fdaddr) && ((fe->hp.dport == sport) || (fe->hp.dport == 0)) &&
-	     (crdaddr == fsaddr) && ((fe->hp.sport == dport) || (fe->hp.sport == 0)))) {
-	    if (toupper(fe->hp.reverse) == 'E')		/* Exclusive */
+	/*
+	 * Just using two variables to make it easier to read
+	 */
+
+	flt_expr1 = ((csaddr == fsaddr)
+		     && ((fe->hp.sport == sport) || (fe->hp.sport == 0)))
+	    && ((cdaddr == fdaddr)
+		&& ((fe->hp.dport == dport) || (fe->hp.dport == 0)));
+
+	flt_expr2 = ((crsaddr == fdaddr)
+		     && ((fe->hp.dport == sport) || (fe->hp.dport == 0)))
+	    && ((crdaddr == fsaddr)
+		&& ((fe->hp.sport == dport) || (fe->hp.sport == 0)));
+
+	if (flt_expr1 || flt_expr2) {
+	    if (toupper(fe->hp.reverse) == 'E')	/* Exclusive */
 		return 0;
 
 	    return 1;
@@ -575,7 +608,9 @@ int utfilter(struct filterlist *fl,
     return 0;
 }
 
-/* remove a currently applied filter from memory */
+/* 
+ * Remove a currently applied filter from memory
+ */
 
 void destroyfilter(struct filterlist *fl)
 {
@@ -598,7 +633,9 @@ void destroyfilter(struct filterlist *fl)
 }
 
 
-/* delete a filter record from the disk */
+/*
+ * Delete a filter record from the disk
+ */
 
 void delfilter(unsigned int protocol, int *aborted)
 {
@@ -609,6 +646,7 @@ void delfilter(unsigned int protocol, int *aborted)
 	return;
 
     if (loadfilterlist(protocol, &fltlist) == 1) {
+	*aborted = 1;
 	listfileerr(1);
 	destroyfilterlist(fltlist);
 	clear_flt_tag(protocol);
@@ -637,28 +675,40 @@ void delfilter(unsigned int protocol, int *aborted)
 
     save_filterlist(protocol, fltlist);
     clear_flt_tag(protocol);
+    *aborted = 0;
 }
 
 void makefiltermenu(struct MENU *menu)
 {
     initmenu(menu, 8, 31, (LINES - 7) / 2 + 3, (COLS - 31) / 2 + 15);
-    additem(menu, " ^D^efine new filter...", "Defines a new set of filter parameters");
-    additem(menu, " ^A^pply filter...", "Applies filter to be used for TCP display");
-    additem(menu, " Detac^h^ filter", "Removes the currently applied filter");
-    additem(menu, " ^E^dit filter...", "Modifies existing TCP filter data");
-    additem(menu, " Dele^t^e filter...", "Removes a TCP filter from the filter list");
+    additem(menu, " ^D^efine new filter...",
+	    "Defines a new set of filter parameters");
+    additem(menu, " ^A^pply filter...",
+	    "Applies filter to be used for TCP display");
+    additem(menu, " Detac^h^ filter",
+	    "Removes the currently applied filter");
+    additem(menu, " ^E^dit filter...",
+	    "Modifies existing TCP filter data");
+    additem(menu, " Dele^t^e filter...",
+	    "Removes a TCP filter from the filter list");
     additem(menu, " E^x^it menu", "Returns to the main menu");
 }
 
 void makeudpfiltermenu(struct MENU *menu)
 {
     initmenu(menu, 9, 31, (LINES - 11) / 2, (COLS - 30) / 2 - 21);
-    additem(menu, " Show ^a^ll UDP packets", "Permits display of all UDP packets");
-    additem(menu, " Show ^n^o UDP packets", "Omits all UDP packets from display");
-    additem(menu, " ^D^efine custom UDP filter...", "Defines a custom UDP filter");
-    additem(menu, " Apply ^c^ustom UDP filter...", "Applies a custom UDP filter");
-    additem(menu, " ^E^dit custom UDP filter...", "Modifies a custom UDP filter");
-    additem(menu, " Dele^t^e custom UDP filter...", "Removes a custom UDP filter");
+    additem(menu, " Show ^a^ll UDP packets",
+	    "Permits display of all UDP packets");
+    additem(menu, " Show ^n^o UDP packets",
+	    "Omits all UDP packets from display");
+    additem(menu, " ^D^efine custom UDP filter...",
+	    "Defines a custom UDP filter");
+    additem(menu, " Apply ^c^ustom UDP filter...",
+	    "Applies a custom UDP filter");
+    additem(menu, " ^E^dit custom UDP filter...",
+	    "Modifies a custom UDP filter");
+    additem(menu, " Dele^t^e custom UDP filter...",
+	    "Removes a custom UDP filter");
     additem(menu, " E^x^it menu", "Returns to the main menu");
 }
 
@@ -684,12 +734,14 @@ void udpfilterselect(struct filterlist *fl,
 	switch (frow) {
 	case 1:
 	    *filtercode = 1;
+	    infobox("All UDP packets will be displayed", ANYKEY_MSG);
 	    break;
 	case 2:
 	    if (*filtercode == 2)
 		destroyfilter(fl);
 
 	    *filtercode = 0;
+	    infobox("No UDP packets will be displayed", ANYKEY_MSG);
 	    break;
 	case 3:
 	    definefilter(F_UDP, faborted);
@@ -697,7 +749,8 @@ void udpfilterselect(struct filterlist *fl,
 	case 4:
 	    selectfilter(F_UDP, &ffe, faborted);
 	    if (!(*faborted)) {
-		strncpy(filename, get_path(T_WORKDIR, ffe.filename), FLT_FILENAME_MAX - 1);
+		strncpy(filename, get_path(T_WORKDIR, ffe.filename),
+			FLT_FILENAME_MAX - 1);
 		loadfilter(filename, fl, FLT_RESOLVE);
 		*filtercode = 2;
 	    }
@@ -707,6 +760,8 @@ void udpfilterselect(struct filterlist *fl,
 	    break;
 	case 6:
 	    delfilter(F_UDP, faborted);
+	    if (!(*faborted))
+	        infobox("Custom UDP filter deleted", ANYKEY_MSG);
 	    break;
 	}
     } while (frow != 7);
@@ -743,7 +798,8 @@ void tcpfilterselect(struct filterlist *fl,
 	case 2:
 	    selectfilter(F_TCP, &ffe, faborted);
 	    if (!(*faborted)) {
-		strncpy(filename, get_path(T_WORKDIR, ffe.filename), FLT_FILENAME_MAX - 1);
+		strncpy(filename, get_path(T_WORKDIR, ffe.filename),
+			FLT_FILENAME_MAX - 1);
 		if (!loadfilter(filename, fl, FLT_RESOLVE)) {
 		    *filtered = 1;
 		    marktcpfilter(filename);
@@ -756,12 +812,15 @@ void tcpfilterselect(struct filterlist *fl,
 		*filtered = 0;
 	    }
 	    unlink(TCPFILTERSAVE);
+	    infobox("TCP filter deactivated", ANYKEY_MSG);
 	    break;
 	case 4:
 	    editfilter(F_TCP, faborted);
 	    break;
 	case 5:
 	    delfilter(F_TCP, faborted);
+	    if (!(*faborted))
+	        infobox("TCP filter deleted", ANYKEY_MSG);
 	    break;
 	}
     } while (frow != 6);
